@@ -1,4 +1,4 @@
-##  $Id: checkgroups.pl 8550 2009-07-18 13:07:05Z iulius $
+##  $Id: checkgroups.pl 8433 2009-04-26 07:52:35Z iulius $
 ##
 ##  checkgroups control message handler.
 ##
@@ -36,9 +36,7 @@ sub control_checkgroups {
         my ($ngname, $ngdesc) = split(/\s+/, $_, 2);
         my $charset_newsgroup = $charset_message;
 
-        next if ($ngname !~ /$newsgrouppats/);
-        next if ($exclusionpats and $ngname =~ /$exclusionpats/);
-        next if ($droppats and $ngname =~ /$droppats/);
+        next if ($ngname =~ /$exclusionpats/ or $ngname =~ /$droppats/);
 
         # Find the right charset if absent or forced by control.ctl.
         foreach (@$charset_from) {
@@ -64,10 +62,7 @@ sub control_checkgroups {
         push(@newbody, $ngname."\t".$ngdesc);
     }
 
-    # We do not go on if there is no changes to do.
-    return if ($#newbody < 0);
-
-    if ($action eq 'mail') {
+    if ($action eq 'mail' and $#newbody >= 0) {
         my $mail = sendmail("checkgroups by $sender");
         print $mail "$sender posted the following checkgroups message:\n\n";
         print $mail map { s/^~/~~/; "$_\n" } @headers;
