@@ -1,4 +1,4 @@
-##  $Id: rmgroup.pl 9303 2011-08-04 22:09:57Z iulius $
+##  $Id: rmgroup.pl 8281 2009-01-10 11:52:16Z iulius $
 ##
 ##  rmgroup control message handler.
 ##
@@ -37,18 +37,18 @@ sub control_rmgroup {
     close ACTIVE;
     my $status;
     if (not @oldgroup) {
-        $status = 'not change';
+        $status = 'no change';
     } elsif (not $approved) {
-        $status = 'be unapproved';
+        $status = 'unapproved';
     } else {
-        $status = 'be removed';
+        $status = 'removed';
     }
 
-    if ($action eq 'mail' and $status !~ /(not change|be unapproved)/) {
+    if ($action eq 'mail' and $status !~ /(no change|unapproved)/) {
         my $mail = sendmail("rmgroup $groupname $sender");
         print $mail <<END;
 $sender asks for $groupname
-to $status.
+to be $status.
 
 If this is acceptable, type:
   $INN::Config::newsbin/ctlinnd rmgroup $groupname
@@ -66,18 +66,14 @@ END
     } elsif ($action eq 'log') {
         if ($log) {
             logger($log, "skipping rmgroup $groupname"
-                . " $sender (would $status)", $article);
+                . " $sender (would be $status)", $article);
         } else {
-            logmsg("skipping rmgroup $groupname $sender (would $status)");
+            logmsg("skipping rmgroup $groupname $sender (would be $status)");
         }
-    } elsif ($action eq 'doit' and $status !~ /(not change|be unapproved)/) {
+    } elsif ($action eq 'doit' and $status !~ /(no change|unapproved)/) {
         ctlinnd('rmgroup', $groupname);
         # Update newsgroups too.
-        my $lockfile = "$INN::Config::locks/LOCK.newsgroups";
-
-        # Acquire a lock.
-        INN::Utils::Shlock::lock($lockfile, 60) or logdie("Cannot create lockfile $lockfile");
-
+        shlock("$INN::Config::locks/LOCK.newsgroups");
         open(NEWSGROUPS, $INN::Config::newsgroups)
             or logdie("Cannot open $INN::Config::newsgroups: $!");
         my $tempfile = "$INN::Config::newsgroups.$$";
@@ -89,11 +85,9 @@ END
         close NEWSGROUPS;
         rename($tempfile, $INN::Config::newsgroups)
             or logdie("Cannot rename $tempfile: $!");
+        unlink "$INN::Config::locks/LOCK.newsgroups";
         unlink $tempfile;
 
-        # Unlock.
-        INN::Utils::Shlock::unlock($lockfile);
-
         logger($log, "rmgroup $groupname $status $sender", $article)
             if $log;
     }
