@@ -1,4 +1,4 @@
-/*  $Id: fastrm.c,v 1.21.2.2 2001/02/03 07:36:45 rra Exp $
+/*  $Id: fastrm.c 8225 2008-12-14 05:05:09Z eagle $
 **
 **  Delete a list of filenames or tokens from stdin.
 **
@@ -18,14 +18,14 @@
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
-#include <stdarg.h>
 #include <sys/stat.h>
 #include <syslog.h>
 
-#include "libinn.h"
-#include "macros.h"
-#include "qio.h"
-#include "storage.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/qio.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 
 /* We reject any path names longer than this. */
 #define MAX_DIR_LEN 2048
@@ -62,7 +62,7 @@ static int relative_threshold = 0;
 static int sort_threshold = 0;
 
 /* True if we should only print what we would do, not actually do it. */
-static BOOL debug_only = FALSE;
+static bool debug_only = false;
 
 /* A string used for constructing relative paths. */
 static const char dotdots[] = "../../../../";
@@ -71,97 +71,12 @@ static const char dotdots[] = "../../../../";
 static int error_count = 0;
 
 /* Whether the storage manager has been initialized. */
-static BOOL sm_initialized = FALSE;
+static bool sm_initialized = false;
 
 /* True if unlink may be able to remove directories. */
-static BOOL unlink_dangerous = FALSE;
+static bool unlink_dangerous = false;
 
 
-/*
-**  Exit handler for die.  Shut down the storage manager before exiting.
-*/
-static int
-sm_cleanup(void)
-{
-    if (sm_initialized)
-        SMshutdown();
-    return 1;
-}
-
-
-/*
-**  Warn to stderr.
-*/
-static void
-warn(const char *format, ...)
-{
-    va_list args;
-
-    fprintf(stderr, "fastrm: ");
-    va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    fprintf(stderr, "\n");
-}
-
-
-/*
-**  Warn to stderr, including the results of strerror.
-*/
-static void
-syswarn(const char *format, ...)
-{
-    int oerrno;
-    va_list args;
-
-    oerrno = errno;
-    fprintf(stderr, "fastrm: ");
-    va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    fprintf(stderr, ": %s\n", strerror(oerrno));
-}
-
-
-/*
-**  Report a fatal error and exit.
-*/
-static void
-die(const char *format, ...)
-{
-    va_list args;
-
-    fflush(stdout);
-    fprintf(stderr, "fastrm: ");
-    va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    fprintf(stderr, "\n");
-    sm_cleanup();
-    exit(1);
-}
-
-
-/*
-**  Report a fatal error, including the results of strerror, and exit.
-*/
-static void
-sysdie(const char *format, ...)
-{
-    int oerrno;
-    va_list args;
-
-    oerrno = errno;
-    fflush(stdout);
-    fprintf(stderr, "fastrm: ");
-    va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    fprintf(stderr, ": %s\n", strerror(oerrno));
-    sm_cleanup();
-    exit(1);
-}
-
 
 /*
 **  Sorting predicate for qsort and bsearch.
@@ -171,8 +86,8 @@ file_compare(const void *a, const void *b)
 {
     const char *f1, *f2;
 
-    f1 = *((const char **) a);
-    f2 = *((const char **) b);
+    f1 = *((const char *const *) a);
+    f2 = *((const char *const *) b);
     return strcmp(f1, f2);
 }
 
@@ -185,7 +100,7 @@ filelist_new(char *dir)
 {
     filelist *new;
 
-    new = NEW(filelist, 1);
+    new = xmalloc(sizeof(filelist));
     new->count = 0;
     new->size = 0;
     new->dir = dir;
@@ -202,9 +117,9 @@ filelist_insert(filelist *list, char *name)
 {
     if (list->count == list->size) {
         list->size = (list->size == 0) ? 16 : list->size * 2;
-        RENEW(list->files, char *, list->size);
+        list->files = xrealloc(list->files, list->size * sizeof(char *));
     }
-    list->files[list->count++] = COPY(name);
+    list->files[list->count++] = xstrdup(name);
 }
 
 
@@ -231,6 +146,8 @@ filelist_empty(filelist *list)
 {
     int i;
 
+    if (list->files == NULL)
+        return;
     for (i = 0; i < list->count; i++)
         free(list->files[i]);
     list->count = 0;
@@ -252,6 +169,17 @@ filelist_free(filelist *list)
 }
 
 
+/*
+**  Exit handler for die.  Shut down the storage manager before exiting.
+*/
+static int
+sm_cleanup(void)
+{
+    SMshutdown();
+    return 1;
+}
+
+
 /*
 **  Initialize the storage manager.  This includes parsing inn.conf, which
 **  fastrm doesn't need for any other purpose.
@@ -259,16 +187,17 @@ filelist_free(filelist *list)
 static void
 sm_initialize(void)
 {
-    BOOL value;
+    bool value;
 
-    if (ReadInnConf() < 0)
+    if (!innconf_read(NULL))
         exit(1);
-    value = TRUE;
+    value = true;
     if (!SMsetup(SM_RDWR, &value) || !SMsetup(SM_PREOPEN, &value))
         die("can't set up storage manager");
     if (!SMinit())
         die("can't initialize storage manager: %s", SMerrorstr);
-    sm_initialized = TRUE;
+    sm_initialized = true;
+    message_fatal_cleanup = sm_cleanup;
 }
 
 
@@ -288,8 +217,6 @@ get_line(QIOSTATE *qp)
     while (QIOtoolong(qp) || (p != NULL && strlen(p) >= MAX_DIR_LEN)) {
         warn("line %d too long", count);
         error_count++;
-        while (p == NULL && QIOtoolong(qp))
-            p = QIOread(qp);
         p = QIOread(qp);
     }
     if (p == NULL) {
@@ -314,8 +241,9 @@ process_line(QIOSTATE *qp, int *queued, int *deleted)
 {
     static char *line = NULL;
     filelist *list = NULL;
-    char *p, *dir;
-    int dlen;
+    char *p;
+    char *dir = NULL;
+    int dlen = -1;
 
     *queued = 0;
     *deleted = 0;
@@ -343,7 +271,7 @@ process_line(QIOSTATE *qp, int *queued, int *deleted)
                 if (p != NULL) {
                     *p++ = '\0';
                     dlen = strlen(line);
-                    dir = COPY(line);
+                    dir = xstrdup(line);
                 } else {
                     p = line;
                     dlen = -1;
@@ -456,11 +384,11 @@ unlink_file(const char *file)
 **  isn't it instead just sets prefix_dir and prefix_len to point to the new
 **  directory without changing the working directory.
 */
-static BOOL
+static bool
 chdir_checked(const char *path, int filecount)
 {
     if (filecount < chdir_threshold) {
-        strcpy(prefix_dir, path);
+        strlcpy(prefix_dir, path, sizeof(prefix_dir));
         prefix_len = strlen(path);
     } else {
         prefix_len = 0;
@@ -469,11 +397,11 @@ chdir_checked(const char *path, int filecount)
                 sysdie("can't chdir from %s to %s", current_dir, path);
             else {
                 syswarn("can't chdir from %s to %s", current_dir, path);
-                return FALSE;
+                return false;
             }
         }
     }
-    return TRUE;
+    return true;
 }
 
 
@@ -486,7 +414,7 @@ chdir_checked(const char *path, int filecount)
 **  directory doesn't exist (and therefore all files in it have already been
 **  removed; otherwise, return true.
 */
-static BOOL
+static bool
 setup_dir(char *dir, int filecount)
 {
     char *p, *q, *absolute;
@@ -499,7 +427,7 @@ setup_dir(char *dir, int filecount)
     else if (*dir == '/')
         absolute = dir;
     else if (*dir == '\0') {
-        strcpy(path, "/");
+        strlcpy(path, "/", sizeof(path));
         absolute = path;
     } else {
         /* Strip off leading "./". */
@@ -510,7 +438,7 @@ setup_dir(char *dir, int filecount)
         /* Handle any leading "../", but only up to the number of segments
            in base_dir. */
         base_depth = slashcount(base_dir);
-        while (base_depth > 0 && EQn(dir, "../", 3))
+        while (base_depth > 0 && strncmp(dir, "../", 3) == 0)
             for (base_depth--, dir += 3; *dir == '/'; dir++)
                 ;
         if (base_depth <= 0)
@@ -518,8 +446,8 @@ setup_dir(char *dir, int filecount)
         copy_segments(path, base_dir, base_depth + 1);
         if (strlen(path) + strlen(dir) + 2 > MAX_DIR_LEN)
             die("path %s too long", dir);
-        strcat(path, "/");
-        strcat(path, dir);
+        strlcat(path, "/", sizeof(path));
+        strlcat(path, dir, sizeof(path));
         absolute = path;
     }
 
@@ -529,17 +457,17 @@ setup_dir(char *dir, int filecount)
        happen with odd relative paths). */
     for (p = absolute, q = current_dir; *p == *q; p++, q++)
         if (*p == '\0')
-            return TRUE;
+            return true;
 
     /* If we reached the end of current_dir and there's more left of
        absolute, we're changing to a subdirectory of where we were. */
     if (*q == '\0' && *p == '/') {
         p++;
         if (!chdir_checked(p, filecount))
-            return FALSE;
+            return false;
         if (prefix_len == 0)
-            strcpy(current_dir, absolute);
-        return TRUE;
+            strlcpy(current_dir, absolute, sizeof(current_dir));
+        return true;
     }
 
     /* Otherwise, if we were promised that we have a pure tree (in other
@@ -549,25 +477,29 @@ setup_dir(char *dir, int filecount)
        we're willing to use; the default of 1 seems fractionally faster than
        2 and 0 indicates to always use absolute paths.  Values larger than 3
        would require extending the dotdots string, but are unlikely to be
-       worth it. */
+       worth it.
+
+       FIXME: It's too hard to figure out what this code does.  It needs to be
+       rewritten. */
     if (p != '\0' && relative_threshold > 0) {
         depth = slashcount(q);
         if (depth <= relative_threshold) {
             while (p > absolute && *--p != '/')
                 ;
             p++;
-            strcpy(prefix_dir, dotdots + 9 - depth * 3);
-            strcpy(prefix_dir + (depth + 1) * 3, p);
+            strlcpy(prefix_dir, dotdots + 9 - depth * 3, sizeof(prefix_dir));
+            strlcat(prefix_dir, p, sizeof(prefix_dir));
             if (!chdir_checked(prefix_dir, filecount))
-                return FALSE;
+                return false;
 
             /* Now patch up current_dir to reflect where we are. */
             if (prefix_len == 0) {
                 while (q > current_dir && *--q != '/')
                     ;
-                strcpy(q + 1, p);
+                q[1] = '\0';
+                strlcat(current_dir, p, sizeof(current_dir));
             }
-            return TRUE;
+            return true;
         }
     }
 
@@ -578,10 +510,10 @@ setup_dir(char *dir, int filecount)
        cares about speed (it usually doesn't happen with sorted input).  So
        we don't bother. */
     if (!chdir_checked(absolute, filecount))
-        return FALSE;
+        return false;
     if (prefix_len == 0)
-        strcpy(current_dir, absolute);
-    return TRUE;
+        strlcpy(current_dir, absolute, sizeof(current_dir));
+    return true;
 }
 
 
@@ -591,7 +523,7 @@ setup_dir(char *dir, int filecount)
 static void
 unlink_filelist(filelist *list, int filecount)
 {
-    BOOL sorted;
+    bool sorted;
     DIR *dir;
     struct dirent *entry;
     char *file;
@@ -618,7 +550,7 @@ unlink_filelist(filelist *list, int filecount)
        the threshold, just remove the files. */
     if (sort_threshold == 0 || filecount < sort_threshold) {
         for (i = 0; i < list->count; i++) {
-            strcpy(file, list->files[i]);
+            strlcpy(file, list->files[i], sizeof(prefix_dir) - prefix_len);
             unlink_file(prefix_dir);
         }
         filelist_free(list);
@@ -628,7 +560,7 @@ unlink_filelist(filelist *list, int filecount)
     /* We have enough files to remove in this directory that it's worth
        optimizing.  First, make sure the list of files is sorted.  It's not
        uncommon for the files to already be sorted, so check first. */
-    for (sorted = TRUE, i = 1; sorted && i < list->count; i++)
+    for (sorted = true, i = 1; sorted && i < list->count; i++)
         sorted = (strcmp(list->files[i - 1], list->files[i]) <= 0);
     if (!sorted)
         qsort(list->files, list->count, sizeof(char *), file_compare);
@@ -655,7 +587,7 @@ unlink_filelist(filelist *list, int filecount)
     for (i = 0, entry = readdir(dir); entry != NULL; entry = readdir(dir))
         if (filelist_lookup(list, entry->d_name) != NULL) {
             i++;
-            strcpy(file, entry->d_name);
+            strlcpy(file, entry->d_name, sizeof(prefix_dir) - prefix_len);
             unlink_file(prefix_dir);
             if (i == list->count)
                 break;
@@ -670,22 +602,22 @@ unlink_filelist(filelist *list, int filecount)
 **  ensures that it doesn't contain elements like "./" or "../" and doesn't
 **  contain doubled slashes.
 */
-static BOOL
+static bool
 bad_path(const char *p)
 {
     if (strlen(p) >= MAX_DIR_LEN)
-        return TRUE;
+        return true;
     while (*p) {
         if (p[0] == '.' && (p[1] == '/' || (p[1] == '.' && p[2] == '/')))
-            return TRUE;
+            return true;
         while (*p && *p != '/')
             p++;
         if (p[0] == '/' && p[1] == '/')
-            return TRUE;
+            return true;
         if (*p == '/')
             p++;
     }
-    return FALSE;
+    return false;
 }
 
 
@@ -702,7 +634,7 @@ main(int argc, char *argv[])
     QIOSTATE *qp;
     filelist *list;
     int filecount, deleted;
-    BOOL empty_error = FALSE;
+    bool empty_error = false;
 
     /* Establish our identity.  Since we use the storage manager, we need to
        set up syslog as well, although we won't use it ourselves. */
@@ -714,6 +646,7 @@ main(int argc, char *argv[])
         if (p != NULL)
             name = p + 1;
     }
+    message_program_name = name;
     openlog(name, LOG_CONS | LOG_PID, LOG_INN_PROG);
 
     /* If we're running as root, unlink may remove directories. */
@@ -738,10 +671,10 @@ main(int argc, char *argv[])
                 chdir_threshold = atoi(p + 1);
                 break;
             case 'd':
-                debug_only = TRUE;
+                debug_only = true;
                 continue;
             case 'e':
-                empty_error = TRUE;
+                empty_error = true;
                 continue;
             case 's':
                 sort_threshold = 5;
@@ -771,7 +704,7 @@ main(int argc, char *argv[])
     base_dir = *arg;
     if (*base_dir != '/' || bad_path(base_dir))
         die("bad base path %s", base_dir);
-    strcpy(current_dir, base_dir);
+    strlcpy(current_dir, base_dir, sizeof(current_dir));
     if (chdir(current_dir) < 0)
         sysdie("can't chdir to base path %s", current_dir);
 
@@ -781,11 +714,11 @@ main(int argc, char *argv[])
     if (qp == NULL)
         sysdie("can't reopen stdin");
     while ((list = process_line(qp, &filecount, &deleted)) != NULL) {
-        empty_error = FALSE;
+        empty_error = false;
         unlink_filelist(list, filecount);
     }
     if (deleted > 0)
-        empty_error = FALSE;
+        empty_error = false;
 
     /* All done. */
     SMshutdown();
