@@ -1,4 +1,4 @@
-/*  $Id: makehistory.c,v 1.112.2.9 2004/12/19 08:21:49 rra Exp $
+/*  $Id: makehistory.c 8409 2009-04-11 22:13:41Z iulius $
 **
 **  Rebuild history/overview databases.
 */
@@ -8,34 +8,36 @@
 #include "portable/wait.h"
 #include <assert.h>
 #include <errno.h>
-#include <pwd.h>
-#include <syslog.h>  
+#include <syslog.h>
+#include <time.h>
 
 #include "inn/buffer.h"
 #include "inn/history.h"
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/messages.h"
+#include "inn/newsuser.h"
+#include "inn/ov.h"
+#include "inn/paths.h"
 #include "inn/qio.h"
+#include "inn/storage.h"
+#include "inn/vector.h"
 #include "inn/wire.h"
-#include "libinn.h"
-#include "ov.h"
-#include "paths.h"
-#include "storage.h"
 
 static const char usage[] = "\
-Usage: makehistory [-bOIax] [-f file] [-l count] [-s size] [-T tmpdir]\n\
+Usage: makehistory [-abFIOSx] [-f file] [-l count] [-s size] [-T tmpdir]\n\
 \n\
-    -b          delete bad articles from spool\n\
-    -e          read entire articles to compute proper byte count\n\
-    -f          write history entries to file (default $pathdb/history)\n\
-    -s size     size new history database for approximately size entries\n\
     -a          open output history file in append mode\n\
-    -O          create overview entries for articles\n\
+    -b          delete bad articles from spool\n\
+    -F          fork when writing overview\n\
+    -f file     write history entries to file (default $pathdb/history)\n\
     -I          do not create overview for articles numbered below lowmark\n\
     -l count    size of overview updates (default 10000)\n\
-    -x          don't write history entries\n\
+    -O          create overview entries for articles\n\
+    -S          write overview data to standard output\n\
+    -s size     size new history database for approximately size entries\n\
     -T tmpdir   use directory tmpdir for temporary files\n\
-    -F          fork when writing overview\n";
+    -x          don't write history entries\n";
 
 
 /*
@@ -53,7 +55,6 @@ typedef struct _ARTOVERFIELD {
 #define DEFAULT_SEGSIZE	10000;
 
 bool NukeBadArts;
-char *SchemaPath = NULL;
 char *ActivePath = NULL;
 char *HistoryPath = NULL;
 struct history *History;
@@ -67,44 +68,30 @@ FILE *OverTmpFile;
 char *OverTmpPath = NULL;
 bool NoHistory;
 OVSORTTYPE sorttype;
-int RetrMode;
-
-TIMEINFO Now;
+bool WriteStdout = false;
 
 /* Misc variables needed for the overview creation code. */
+static char             BYTES[] = "Bytes";
+static char             DATE[] = "Date";
+static char             EXPIRES[] = "Expires";
+static char             LINES[] = "Lines";
 static char		MESSAGEID[] = "Message-ID";
-static char		EXPIRES[] = "Expires";
-static char		DATE[] = "Date";
 static char		XREF[] = "Xref";
-static ARTOVERFIELD	*ARTfields; /* overview fields listed in overview.fmt */
+static ARTOVERFIELD	*ARTfields; /* Overview fields for which overview data
+                                     * is generated. */
 static size_t		ARTfieldsize;
-static ARTOVERFIELD	*Datep = (ARTOVERFIELD *)NULL;
-static ARTOVERFIELD	*Msgidp = (ARTOVERFIELD *)NULL;
-static ARTOVERFIELD	*Expp = (ARTOVERFIELD *)NULL;
-static ARTOVERFIELD	*Xrefp = (ARTOVERFIELD *)NULL;
-static ARTOVERFIELD	*Missfields; /* header fields not listed in 
-					overview.fmt, but ones that we need
-					(e.g. message-id */
+static ARTOVERFIELD     *Bytesp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *Datep = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *Expp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *Linesp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *Msgidp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *Xrefp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD	*Missfields; /* Header fields not used in overview
+                                      * but that we need (e.g. Expires:). */
 static size_t		Missfieldsize = 0;
 
 static void OverAddAllNewsgroups(void);
 
-/*
-**  Check and parse an date header line.  Return the new value or
-**  zero on error.
-*/
-static long
-GetaDate(char *p)
-{
-    time_t		t;
-
-    while (ISWHITE(*p))
-	p++;
-    if ((t = parsedate(p, &Now)) == -1)
-	return 0L;
-    return (long)t;
-}
-
 /*
 **  Check and parse a Message-ID header line.  Return private space.
 */
@@ -206,13 +193,13 @@ FlushOverTmpFile(void)
         Fork ? _exit(1) : exit(1);
     }
     close(fd);
-    snprintf(temp, sizeof(temp), "exec %s -T %s -t'%c' -o %s %s", _PATH_SORT,
-             TmpDir, '\t', SortedTmpPath, OverTmpPath);
+    snprintf(temp, sizeof(temp), "exec %s -T %s -t'%c' -o %s %s",
+             INN_PATH_SORT, TmpDir, '\t', SortedTmpPath, OverTmpPath);
     
     i = system(temp) >> 8;
     if (i != 0) {
         syswarn("cannot sort temporary overview file (%s exited %d)",
-                _PATH_SORT, i);
+                INN_PATH_SORT, i);
 	OVclose();
 	Fork ? _exit(1) : exit(1);
     }
@@ -294,8 +281,10 @@ FlushOverTmpFile(void)
 	    
 
 /*
- * Write a line to the overview temp file. 
- */
+**  Write a line to the overview temp file, standard output, our child process
+**  that's doing the writing, or directly to overview, whichever is
+**  appropriate.
+*/
 static void
 WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen, 
 	      char *overdata, int overlen, time_t arrived, time_t expires)
@@ -304,6 +293,19 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
     const char *p, *q, *r;
     int i, fd;
 
+    /* If WriteStdout is set, just print the overview information to standard
+       output and return. */
+    if (WriteStdout) {
+        printf("%s %ld %ld ", TokenToText(*token), (long) arrived,
+               (long) expires);
+        fwrite(overdata, 1, overlen, stdout);
+        fputc('\n', stdout);
+        return;
+    }
+
+    /* If the overview method doesn't care about sorted data, don't bother
+       with the temporary file and just write the data out to our child
+       process or to the overview database directly. */
     if (sorttype == OVNOSORT) {
 	if (Fork) {
 	    fprintf(Overchan, "%s %ld %ld ", TokenToText(*token), (long)arrived, (long)expires);
@@ -321,6 +323,9 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
 	}
 	return;
     }
+
+    /* Otherwise, we need the temporary file.  Create it if it doesn't already
+       exist. */
     if (OverTmpPath == NULL) {
 	/* need new temp file, so create it. */
         OverTmpPath = concatpath(TmpDir, "histXXXXXX");
@@ -332,6 +337,9 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
             sysdie("cannot open %s", OverTmpPath);
 	OverTmpSegCount = 0;
     }
+
+    /* Print out the data to the teporary file with the appropriate keys for
+       sorting. */
     if (sorttype == OVNEWSGROUP) {
 	/* find first ng name in xref. */
 	for (p = xrefs, q=NULL ; p < xrefs+xrefslen ; ++p) {
@@ -358,7 +366,7 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
 	    return;
 	}
 	/* q points to start of ng name, r points to its end. */
-        assert(sizeof(temp) > r - q + 1);
+        assert((ptrdiff_t) sizeof(temp) > r - q + 1);
 	memcpy(temp, q, r - q + 1);
         temp[r - q + 1] = '\0';
 	fprintf(OverTmpFile, "%s\t%10lu\t%lu\t%s\t", temp,
@@ -385,95 +393,120 @@ WriteOverLine(TOKEN *token, const char *xrefs, int xrefslen,
 static void
 ARTreadschema(bool Overview)
 {
-    FILE                        *F;
-    char                        *p;
+    const struct cvector        *standardoverview;
+    const struct vector         *extraoverview;
     ARTOVERFIELD                *fp;
-    int                         i;
-    char                        buff[SMBUF];
-    bool                        foundxreffull = false;
+    unsigned int                i;
 
     if (Overview) {
-	/* Open file, count lines. */
-	if ((F = fopen(SchemaPath, "r")) == NULL)
-            sysdie("cannot open %s", SchemaPath);
-	for (i = 0; fgets(buff, sizeof buff, F) != NULL; i++)
-	    continue;
-	fseeko(F, 0, SEEK_SET);
-	ARTfields = xmalloc((i + 1) * sizeof(ARTOVERFIELD));
+	/* Count the number of overview fields and allocate ARTfields. */
+        standardoverview = overview_fields();
+        extraoverview = overview_extra_fields(true);
+	ARTfields = xmalloc((standardoverview->count + extraoverview->count + 1)
+                            * sizeof(ARTOVERFIELD));
 
 	/* Parse each field. */
-	for (fp = ARTfields; fgets(buff, sizeof buff, F) != NULL; ) {
-	    /* Ignore blank and comment lines. */
-	    if ((p = strchr(buff, '\n')) != NULL)
-		*p = '\0';
-	    if ((p = strchr(buff, '#')) != NULL)
-		*p = '\0';
-	    if (buff[0] == '\0')
-		continue;
-	    if ((p = strchr(buff, ':')) != NULL) {
-		*p++ = '\0';
-		fp->NeedHeadername = (strcmp(p, "full") == 0);
-	    }
-	    else
-		fp->NeedHeadername = false;
-	    fp->Headername = xstrdup(buff);
-	    fp->HeadernameLength = strlen(buff);
+        for (i = 0, fp = ARTfields; i < standardoverview->count; i++) {
+            fp->NeedHeadername = false;
+	    fp->Headername = xstrdup(standardoverview->strings[i]);
+	    fp->HeadernameLength = strlen(standardoverview->strings[i]);
 	    fp->Header = (char *)NULL;
 	    fp->HasHeader = false;
 	    fp->HeaderLength = 0;
-	    if (strncasecmp(buff, DATE, strlen(DATE)) == 0)
-		Datep = fp;
-	    if (strncasecmp(buff, MESSAGEID, strlen(MESSAGEID)) == 0)
-		Msgidp = fp;
-	    if (strncasecmp(buff, EXPIRES, strlen(EXPIRES)) == 0)
-		Expp = fp;
-	    if (strncasecmp(buff, XREF, strlen(XREF)) == 0) {
-		Xrefp = fp;
-		foundxreffull = fp->NeedHeadername;
-            }
+            /* We need to keep the value of fp for :bytes and :lines
+             * because we will need them afterwards; we will then not
+             * have to compare fp->Headername to both "Bytes" and "Lines"
+             * for each fp. */
+            if (strncasecmp(standardoverview->strings[i],
+                            BYTES, strlen(BYTES)) == 0)
+                Bytesp = fp;
+            if (strncasecmp(standardoverview->strings[i],
+                            DATE, strlen(DATE)) == 0)
+                Datep = fp;
+            if (strncasecmp(standardoverview->strings[i],
+                            LINES, strlen(LINES)) == 0)
+                Linesp = fp;
+            if (strncasecmp(standardoverview->strings[i],
+                            MESSAGEID, strlen(MESSAGEID)) == 0)
+                Msgidp = fp;
 	    fp++;
 	}
+        for (i = 0; i < extraoverview->count; i++) {
+            fp->NeedHeadername = true;
+            fp->Headername = xstrdup(extraoverview->strings[i]);
+            fp->HeadernameLength = strlen(extraoverview->strings[i]);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            if (strncasecmp(extraoverview->strings[i],
+                            XREF, strlen(XREF)) == 0)
+                Xrefp = fp;
+            if (strncasecmp(extraoverview->strings[i],
+                            EXPIRES, strlen(EXPIRES)) == 0)
+                Expp = fp;
+            fp++;
+        }
+
 	ARTfieldsize = fp - ARTfields;
-	fclose(F);
     }
+    if (Bytesp == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
+    if (Datep == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
+    if (Expp == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
+    if (Linesp == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
     if (Msgidp == (ARTOVERFIELD *)NULL)
 	Missfieldsize++;
-    if (Datep == (ARTOVERFIELD *)NULL)
-	Missfieldsize++;
-    if (Expp == (ARTOVERFIELD *)NULL)
-	Missfieldsize++;
-    if (Overview && (Xrefp == (ARTOVERFIELD *)NULL || !foundxreffull))
-        die("Xref:full must be included in %s", SchemaPath);
     if (Missfieldsize > 0) {
 	Missfields = xmalloc(Missfieldsize * sizeof(ARTOVERFIELD));
         fp = Missfields;
-	if (Msgidp == (ARTOVERFIELD *)NULL) {
-	    fp->NeedHeadername = false;
-	    fp->Headername = xstrdup(MESSAGEID);
-	    fp->HeadernameLength = strlen(MESSAGEID);
-	    fp->Header = (char *)NULL;
-	    fp->HasHeader = false;
-	    fp->HeaderLength = 0;
-	    Msgidp = fp++;
-	}
-	if (Datep == (ARTOVERFIELD *)NULL) {
-	    fp->NeedHeadername = false;
-	    fp->Headername = xstrdup(DATE);
-	    fp->HeadernameLength = strlen(DATE);
-	    fp->Header = (char *)NULL;
-	    fp->HasHeader = false;
-	    fp->HeaderLength = 0;
-	    Datep = fp++;
-	}
-	if (Expp == (ARTOVERFIELD *)NULL) {
-	    fp->NeedHeadername = false;
-	    fp->Headername = xstrdup(EXPIRES);
-	    fp->HeadernameLength = strlen(EXPIRES);
-	    fp->Header = (char *)NULL;
-	    fp->HasHeader = false;
-	    fp->HeaderLength = 0;
-	    Expp = fp++;
-	}
+        if (Bytesp == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = false;
+            fp->Headername = xstrdup(BYTES);
+            fp->HeadernameLength = strlen(BYTES);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            Bytesp = fp++;
+        }
+        if (Datep == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = false;
+            fp->Headername = xstrdup(DATE);
+            fp->HeadernameLength = strlen(DATE);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            Datep = fp++;
+        }
+        if (Expp == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = false;
+            fp->Headername = xstrdup(EXPIRES);
+            fp->HeadernameLength = strlen(EXPIRES);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            Expp = fp++;
+        }
+        if (Linesp == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = false;
+            fp->Headername = xstrdup(LINES);
+            fp->HeadernameLength = strlen(LINES);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            Linesp = fp++;
+        }
+        if (Msgidp == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = false;
+            fp->Headername = xstrdup(MESSAGEID);
+            fp->HeadernameLength = strlen(MESSAGEID);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            Msgidp = fp++;
+        }
         if (Overview && Xrefp == (ARTOVERFIELD *)NULL) {
 	    fp->NeedHeadername = false;
 	    fp->Headername = xstrdup(XREF);
@@ -505,8 +538,12 @@ DoArt(ARTHANDLE *art)
     time_t			Expires;
     time_t			Posted;
     char			overdata[BIG_BUFFER];
-    char			bytes[BIG_BUFFER];
+    char                        Bytes[BIG_BUFFER];
+    char                        Lines[BIG_BUFFER];
     struct artngnum		ann;
+    int                         DotStuffedLines = 0;
+    int                         lines = 0;
+    bool                        hasCounts = false;
 
     /* Set up place to store headers. */
     for (fp = ARTfields, i = 0; i < ARTfieldsize; i++, fp++) {
@@ -527,6 +564,64 @@ DoArt(ARTHANDLE *art)
     }
     for (fp = ARTfields, i = 0; i < ARTfieldsize; i++, fp++) {
         fp->Header = wire_findheader(art->data, art->len, fp->Headername);
+
+        /* Someone managed to break their server so that they were appending
+           multiple Xref headers, and INN had a bug where it wouldn't notice
+           this and reject the article.  Just in case, see if there are
+           multiple Xref headers and use the last one. */
+        if (fp == Xrefp) {
+            const char *next = fp->Header;
+            size_t left;
+
+            while (next != NULL) {
+                next = wire_endheader(fp->Header, art->data + art->len - 1);
+                if (next == NULL)
+                    break;
+                next++;
+                left = art->len - (next - art->data);
+                next = wire_findheader(next, left, fp->Headername);
+                if (next != NULL)
+                    fp->Header = next;
+            }
+        }
+
+        /* Work out the real values for :bytes and :lines. */
+        if (fp == Bytesp || fp == Linesp) {
+            /* Parse the article only once. */
+            if (!hasCounts && (p = wire_findbody(art->data, art->len)) != NULL) {
+                end = art->data + art->len;
+                /* p is at the beginning of the body, if any. */
+                for (; *p != '\0';) {
+                    /* p is at the beginning of a new line, if any. */
+                    lines++;
+                    if (*p == '.')
+                        DotStuffedLines++;
+                    if ((p = wire_nextline(p, end)) == NULL) {
+                        /* p is at the final ".\r\n". */
+                        lines--;
+                        DotStuffedLines--;
+                        break;
+                    }
+                }
+            }
+
+            if (fp == Bytesp) {
+                /* :bytes does not count the final ".\r\n", so remove 3. */
+                snprintf(Bytes, sizeof(Bytes), "%lu",
+                         (unsigned long) art->len - DotStuffedLines - 3);
+                fp->Header = Bytes;
+                fp->HeaderLength = strlen(Bytes);
+            } else {
+                snprintf(Lines, sizeof(Lines), "%lu", (unsigned long) lines);
+                fp->Header = Lines;
+                fp->HeaderLength = strlen(Lines);
+            }
+            fp->HasHeader = true;
+            hasCounts = true;
+            continue;
+        }
+
+        /* Now, if we have a header, find and record its length. */
         if (fp->Header != NULL) {
 	    fp->HasHeader = true;
             p = wire_endheader(fp->Header, art->data + art->len - 1);
@@ -538,13 +633,8 @@ DoArt(ARTHANDLE *art)
                peel off the \r\n (we're guaranteed we're dealing with
                wire-format articles. */
             fp->HeaderLength = p - fp->Header - 1;
-	} else if (RetrMode == RETR_ALL 
-                   && strcmp(fp->Headername, "Bytes") == 0) {
-            snprintf(bytes, sizeof(bytes), "%lu", (unsigned long) art->len);
-	    fp->HasHeader = true;
-	    fp->Header = bytes;
-	    fp->HeaderLength = strlen(bytes);
 	}
+
     }
     if (Missfieldsize > 0) {
 	for (fp = Missfields, i = 0; i < Missfieldsize; i++, fp++) {
@@ -616,20 +706,17 @@ DoArt(ARTHANDLE *art)
     } else {
         buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
         buffer_append(&buffer, NUL, 1);
-	for (i = 0, q = buffer.data; i < buffer.left; q++, i++)
-	    if (*q == '\t' || *q == '\n' || *q == '\r')
-		*q = ' ';
-	if ((Posted = GetaDate(buffer.data)) == 0)
+        Posted = parsedate_rfc2822_lax(buffer.data);
+        if (Posted == (time_t) -1)
 	    Posted = Arrived;
     }
 
     if (Expp->HasHeader) {
         buffer_set(&buffer, Expp->Header, Expp->HeaderLength);
         buffer_append(&buffer, NUL, 1);
-	for (i = 0, q = buffer.data; i < buffer.left; q++, i++)
-	    if (*q == '\t' || *q == '\n' || *q == '\r')
-		*q = ' ';
-	Expires = GetaDate(buffer.data);
+        Expires = parsedate_rfc2822_lax(buffer.data);
+        if (Expires == (time_t) -1)
+            Expires = 0;
     }
 
     if (DoOverview && Xrefp->HeaderLength > 0) {
@@ -718,25 +805,6 @@ OverAddAllNewsgroups(void)
 }
 
 
-/*
-**  Change to the news user if possible, and if not, die.  Used for operations
-**  that may create new database files, so as not to mess up the ownership.
-*/
-static void
-setuid_news(void)
-{
-    struct passwd *pwd;
-
-    pwd = getpwnam(NEWSUSER);
-    if (pwd == NULL)
-        die("can't resolve %s to a UID (account doesn't exist?)", NEWSUSER);
-    if (getuid() == 0)
-        setuid(pwd->pw_uid);
-    if (getuid() != pwd->pw_uid)
-        die("must be run as %s", NEWSUSER);
-}
-
-
 int
 main(int argc, char **argv)
 {
@@ -745,9 +813,11 @@ main(int argc, char **argv)
     int i;
     bool val;
     char *HistoryDir;
+    char *RebuiltflagPath;
     char *p;
     char *buff;
     size_t npairs = 0;
+    FILE *F;
 
     /* First thing, set up logging and our identity. */
     openlog("makehistory", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
@@ -756,10 +826,10 @@ main(int argc, char **argv)
     /* Set defaults. */
     if (!innconf_read(NULL))
         exit(1);
-    HistoryPath = concatpath(innconf->pathdb, _PATH_HISTORY);
-    ActivePath = concatpath(innconf->pathdb, _PATH_ACTIVE);
+    HistoryPath = concatpath(innconf->pathdb, INN_PATH_HISTORY);
+    ActivePath = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     TmpDir = innconf->pathtmp;
-    SchemaPath = concatpath(innconf->pathetc, _PATH_SCHEMA);
+    RebuiltflagPath = concatpath(innconf->pathrun, INN_PATH_REBUILDOVERVIEW);
 
     OverTmpSegSize = DEFAULT_SEGSIZE;
     OverTmpSegCount = 0;
@@ -768,22 +838,18 @@ main(int argc, char **argv)
     Fork = false;
     AppendMode = false;
     NoHistory = false;
-    RetrMode = RETR_HEAD;
 
-    while ((i = getopt(argc, argv, "aebf:Il:OT:xFs:")) != EOF) {
+    while ((i = getopt(argc, argv, "abFf:Il:OSs:T:x")) != EOF) {
 	switch(i) {
-	case 'T':
-	    TmpDir = optarg;
-	    break;
-	case 'x':
-	    NoHistory = true;
-	    break;
 	case 'a':
 	    AppendMode = true;
 	    break;
 	case 'b':
 	    NukeBadArts = true;
 	    break;
+	case 'F':
+	    Fork = true;
+	    break;
 	case 'f':
 	    HistoryPath = optarg;
 	    break;
@@ -796,15 +862,19 @@ main(int argc, char **argv)
 	case 'O':
 	    DoOverview = true;
 	    break;
-	case 'F':
-	    Fork = true;
-	    break;
-	case 'e':
-	    RetrMode = RETR_ALL;
-	    break;
+        case 'S':
+            WriteStdout = true;
+            OverTmpSegSize = 0;
+            break;
 	case 's':
 	    npairs = atoi(optarg);
 	    break;
+	case 'T':
+	    TmpDir = optarg;
+	    break;
+	case 'x':
+	    NoHistory = true;
+	    break;
 	    
 	default:
 	    fprintf(stderr, "%s", usage);
@@ -819,25 +889,29 @@ main(int argc, char **argv)
         exit(1);
     }
 
-    if ((p = strrchr(HistoryPath, '/')) == NULL) {
-	/* find the default history file directory */
-	HistoryDir = innconf->pathdb;
-    } else {
-	*p = '\0';
-	HistoryDir = xstrdup(HistoryPath);
-	*p = '/';
+    if (!NoHistory) {
+        if ((p = strrchr(HistoryPath, '/')) == NULL) {
+            /* find the default history file directory */
+            HistoryDir = innconf->pathdb;
+        } else {
+            *p = '\0';
+            HistoryDir = xstrdup(HistoryPath);
+            *p = '/';
+        }
+        if (chdir(HistoryDir) < 0)
+            sysdie("cannot chdir to %s", HistoryDir);
     }
 
-    if (chdir(HistoryDir) < 0)
-        sysdie("cannot chdir to %s", HistoryDir);
+    /* Change to the runasuser user and runasgroup group if necessary. */
+    if (!NoHistory || !WriteStdout) {
+        if (getenv("INN_TESTSUITE") == NULL)
+            ensure_news_user_grp(true, true);
+    }
 
-    /* Change users if necessary. */
-    setuid_news();
-
-    /* Read in the overview schema */
+    /* Read in the overview schema. */
     ARTreadschema(DoOverview);
     
-    if (DoOverview) {
+    if (DoOverview && !WriteStdout) {
 	/* init the overview setup. */
 	if (!OVopen(OV_WRITE))
             sysdie("cannot open overview");
@@ -880,16 +954,12 @@ main(int argc, char **argv)
             sysdie("cannot open %s", HistoryPath);
     }
 
-    /* Get the time.  Only get it once, which is good enough. */
-    if (GetTimeInfo(&Now) < 0)
-        sysdie("cannot get the time");
-
     /*
      * Scan the entire spool, nuke any bad arts if needed, and process each
      * article.
      */
 	
-    while ((art = SMnext(art, RetrMode)) != NULL) {
+    while ((art = SMnext(art, RETR_ALL)) != NULL) {
 	if (art->len == 0) {
 	    if (NukeBadArts && art->data == NULL && art->token != NULL)
 		SMcancel(*art->token);
@@ -908,15 +978,21 @@ main(int argc, char **argv)
 	if (sorttype == OVNOSORT && Fork)
 	    if (fflush(Overchan) == EOF || ferror(Overchan) || pclose(Overchan) == EOF)
                 sysdie("cannot flush overview data");
-	if (sorttype != OVNOSORT) {
+	if (sorttype != OVNOSORT && !WriteStdout) {
 	    int status;
 	    FlushOverTmpFile();
 	    if(Fork)
 		wait(&status);
 	}
+        if (!WriteStdout) {
+            if ((F = fopen(RebuiltflagPath, "w")) == NULL)
+                sysdie("cannot open %s", RebuiltflagPath);
+            if (fprintf(F, "%ld\n", (long) time (NULL)) == EOF || ferror(F))
+                sysdie("cannot write rebuilt flag file");
+            fclose(F);
+        }
     }
-    if(!Fork)
+    if (!Fork && !WriteStdout)
 	OVclose();
     exit(0);
 }
-
