@@ -1,4 +1,4 @@
-/*  $Id: feedone.c 10022 2016-05-05 12:49:59Z iulius $
+/*  $Id: feedone.c 7641 2007-08-10 07:56:30Z iulius $
 **
 **  Connect to the NNTP server and feed one article.
 */
@@ -6,11 +6,10 @@
 #include "config.h"
 #include "clibrary.h"
 #include <errno.h>
-#include <time.h>
 
-#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "inn/nntp.h"
+#include "inn/libinn.h"
+#include "nntp.h"
 
 
 static FILE	*FromServer;
@@ -28,7 +27,7 @@ GetFromServer(buff, size, text)
     char	*text;
 {
     if (fgets(buff, size, FromServer) == NULL)
-        sysdie("%s", text);
+        sysdie("s", text);
     if (Tracing)
 	printf("S: %s", buff);
 }
@@ -84,6 +83,7 @@ main(ac, av)
     bool	PostMode;
 
     /* Set defaults. */
+    mesgid[0] = '\0';
     PostMode = false;
     message_program_name = "feedone";
 
@@ -122,7 +122,7 @@ main(ac, av)
     /* Scan for the message-id. */
     if (mesgid == NULL) {
 	while (fgets(buff, sizeof buff, F) != NULL)
-	    if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	    if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 		if ((p = strchr(buff, '<')) == NULL
                  || (q = strchr(p, '>')) == NULL)
                     die("bad message ID line");
@@ -163,7 +163,7 @@ main(ac, av)
     /* Send the file over. */
     fseeko(F, 0, SEEK_SET);
     while (fgets(buff, sizeof buff, F) != NULL) {
-	if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 	    fprintf(ToServer, "%s %s\r\n", MESGIDHDR, mesgid);
 	    continue;
 	}
