@@ -1,4 +1,4 @@
-/*  $Id: getlist.c 10014 2016-05-05 12:42:31Z iulius $
+/*  $Id: getlist.c 7809 2008-04-28 13:14:25Z iulius $
 **
 **  Send a LIST command to an NNTP server and print the results.
 */
@@ -237,9 +237,8 @@ main(int argc, char *argv[])
             break;
         case 'p':
             port = atoi(optarg);
-            if (port == 0) {
+            if (port <= 0)
                 die("%s is not a valid port number", optarg);
-            }
             break;
         case 'R':
             reader = true;
@@ -260,13 +259,13 @@ main(int argc, char *argv[])
         pattern = argv[1];
     if (argc == 3)
         types = argv[2];
-    if (strcasecmp(list, "active") != 0 && types != NULL)
+    if (strcmp(list, "active") != 0 && types != NULL)
         die("group types can only be specified with a list type of active");
 
     /* Connect to the server. */
     if (host == NULL)
         sysdie("cannot get server name");
-    nntp = nntp_connect(host, port, 128 * 1024, DEFAULT_TIMEOUT);
+    nntp = nntp_connect(host, port, 128 * 1024, 10 * 60);
     if (nntp == NULL)
         sysdie("cannot connect to server %s:%hu", host, port);
     status = nntp_read_response(nntp, &response, &line);
@@ -307,7 +306,7 @@ main(int argc, char *argv[])
     /* Be polite and say goodbye; it gives the server a chance to shut the
        connection down cleanly. */
     if (nntp_send_line(nntp, "QUIT"))
-        nntp_read_response(nntp, &response, &line);
+        status = nntp_read_response(nntp, &response, &line);
     nntp_free(nntp);
     exit(0);
 }
