@@ -1,4 +1,4 @@
-/*  $Id: inews.c 7769 2008-04-13 08:11:41Z iulius $
+/*  $Id: inews.c 7768 2008-04-13 08:11:39Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -10,15 +10,15 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <grp.h>
 #include <pwd.h>
 #include <sys/stat.h>
 
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/newsuser.h"
+#include "inn/paths.h"
 #include "nntp.h"
-#include "paths.h"
 
 /* Signature handling.  The separator will be appended before the signature,
    and at most SIG_MAXLINES will be appended. */
@@ -110,7 +110,7 @@ static HEADER	Table[] = {
 
 #define HDR(_x)	(Table[(_x)].Value)
 
-
+
 
 /*
 **  Send the server a quit message, wait for a reply.
@@ -134,7 +134,7 @@ QuitServer(int x)
 	*p = '\0';
     if ((p = strchr(buff, '\n')) != NULL)
 	*p = '\0';
-    if (atoi(buff) != NNTP_GOODBYE_ACK_VAL)
+    if (atoi(buff) != NNTP_OK_QUIT)
         die("server did not reply to quit properly: %s", buff);
     fclose(FromServer);
     fclose(ToServer);
@@ -277,7 +277,7 @@ StripOffHeaders(char *article)
     return p + 1;
 }
 
-
+
 
 /*
 **  See if the user is allowed to cancel the indicated message.  Assumes
@@ -295,7 +295,7 @@ CheckCancel(char *msgid, bool JustReturn)
     fprintf(ToServer, "head %s\r\n", msgid);
     SafeFlush(ToServer);
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || atoi(buff) != NNTP_HEAD_FOLLOWS_VAL) {
+     || atoi(buff) != NNTP_OK_HEAD) {
 	if (JustReturn)
 	    return;
         die("server has no such article");
@@ -337,32 +337,42 @@ CheckCancel(char *msgid, bool JustReturn)
 **  See if the user is the news administrator.
 */
 static bool
-AnAdministrator(char *name, gid_t group)
+AnAdministrator(void)
 {
-    struct passwd	*pwp;
-    struct group	*grp;
-    char		**mem;
-    char		*p;
+    uid_t               news_uid;
+    gid_t               news_gid;
 
     if (Revoked)
 	return false;
 
     /* Find out who we are. */
-    if ((pwp = getpwnam(NEWSUSER)) == NULL)
-	/* Silent falure; clients might not have the group. */
-	return false;
-    if (getuid() == pwp->pw_uid)
-	return true;
+    if (get_news_uid_gid(&news_uid, &news_gid, false) != 0) {
+        /* Silent failure; clients might not have the group. */
+        return false;
+    }
+    if (getuid() == news_uid)
+        return true;
 
-    /* See if the we're in the right group. */
-    if ((grp = getgrnam(NEWSGRP)) == NULL || (mem = grp->gr_mem) == NULL)
-	/* Silent falure; clients might not have the group. */
-	return false;
-    if (group == grp->gr_gid)
-	return true;
-    while ((p = *mem++) != NULL)
-	if (strcmp(name, p) == 0)
-	    return true;
+    /* See if we are in the right group and examine process
+     * supplementary groups, rather than the group(5) file entry.
+     */
+    {
+        int ngroups = getgroups(0, 0);
+        gid_t *groups, *gp;
+        int rv;
+        int rest;
+
+        groups = (gid_t *) xmalloc(ngroups * sizeof(*groups));
+        if ((rv = getgroups(ngroups, groups)) < 0) {
+            /* Silent failure; client doesn't have the group. */
+            return false;
+        }
+        for (rest = ngroups, gp = groups; rest > 0; rest--, gp++) {
+            if (*gp == news_gid)
+                return true;
+        }
+    }
+    
     return false;
 }
 
@@ -371,12 +381,11 @@ AnAdministrator(char *name, gid_t group)
 **  Check the control message, and see if it's legit.
 */
 static void
-CheckControl(char *ctrl, struct passwd *pwp)
+CheckControl(char *ctrl)
 {
     char	*p;
     char	*q;
     char		save;
-    char		name[SMBUF];
 
     /* Snip off the first word. */
     for (p = ctrl; ISWHITE(*p); p++)
@@ -404,8 +413,7 @@ CheckControl(char *ctrl, struct passwd *pwp)
 	  || strcmp(ctrl, "sendsys")     == 0
 	  || strcmp(ctrl, "senduuname")  == 0
 	  || strcmp(ctrl, "version")     == 0) {
-	strlcpy(name, pwp->pw_name, SMBUF);
-	if (!AnAdministrator(name, pwp->pw_gid))
+	if (!AnAdministrator())
             die("ask your news administrator to do the %s for you", ctrl);
     }
     else {
@@ -414,7 +422,7 @@ CheckControl(char *ctrl, struct passwd *pwp)
     *p = save;
 }
 
-
+
 
 /*
 **  Parse the GECOS field to get the user's full name.  This comes Sendmail's
@@ -518,7 +526,6 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
     static char		PATHFLUFF[] = PATHMASTER;
     HEADER              *hp;
     char                *p;
-    TIMEINFO		Now;
     char		buff[SMBUF];
     char		from[SMBUF];
 
@@ -559,14 +566,14 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
 
     /* Set Subject; Control overrides the subject. */
     if (HDR(_control)) {
-	CheckControl(HDR(_control), pwp);
+	CheckControl(HDR(_control));
     }
     else {
 	p = HDR(_subject);
 	if (p == NULL)
             die("required Subject header is missing or empty");
 	else if (HDR(_alsocontrol))
-	    CheckControl(HDR(_alsocontrol), pwp);
+	    CheckControl(HDR(_alsocontrol));
 #if	0
 	if (strncmp(p, "Re: ", 4) == 0 && HDR(_references) == NULL)
             die("article subject begins with \"Re: \" but has no references");
@@ -611,9 +618,7 @@ ProcessHeaders(bool AddOrg, int linecount, struct passwd *pwp)
     /* Followup-To; checked with Newsgroups. */
 
     /* Check Expires. */
-    if (GetTimeInfo(&Now) < 0)
-        sysdie("cannot get the time");
-    if (HDR(_expires) && parsedate(HDR(_expires), &Now) == -1)
+    if (HDR(_expires) && parsedate_rfc2822_lax(HDR(_expires)) == -1)
         die("cannot parse \"%s\" as an expiration date", HDR(_expires));
 
     /* References; left alone. */
@@ -751,7 +756,7 @@ CheckIncludedText(char *p, int lines)
         die("more included text than new text");
 }
 
-
+
 
 /*
 **  Read stdin into a string and return it.  Can't use ReadInDescriptor
@@ -784,7 +789,7 @@ ReadStdin(void)
     return article;
 }
 
-
+
 
 /*
 **  Offer the article to the server, return its reply.
@@ -963,13 +968,14 @@ main(int ac, char *av[])
         port = NNTP_PORT;
 
     /* Try to open a connection to the server. */
-    if (NNTPremoteopen(port, &FromServer, &ToServer, buff) < 0) {
+    if (NNTPremoteopen(port, &FromServer, &ToServer, buff, sizeof(buff)) < 0) {
 	Spooling = true;
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
 	if ((p = strchr(buff, '\r')) != NULL)
 	    *p = '\0';
-	strcpy(SpoolMessage, buff[0] ? buff : NOCONNECT);
+	strlcpy(SpoolMessage, buff[0] ? buff : NOCONNECT,
+                sizeof(SpoolMessage));
         deadfile = concatpath(pwp->pw_dir, "dead.article");
     }
     else {
@@ -986,10 +992,10 @@ main(int ac, char *av[])
 	SafeFlush(ToServer);
 	if (fgets(buff, HEADER_STRLEN, FromServer) == NULL)
             sysdie("cannot tell server we're reading");
-	if ((j = atoi(buff)) != NNTP_BAD_COMMAND_VAL)
+	if ((j = atoi(buff)) != NNTP_ERR_COMMAND)
 	    i = j;
 
-        if (i != NNTP_POSTOK_VAL) {
+        if (i != NNTP_OK_BANNER_POST) {
             /* We try to authenticate in case it is all the same possible
              * to post. */
             if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0)
@@ -1056,13 +1062,13 @@ main(int ac, char *av[])
 
     /* Article is prepared, offer it to the server. */
     i = OfferArticle(buff, false);
-    if (i == NNTP_AUTH_NEEDED_VAL) {
+    if (i == NNTP_FAIL_AUTH_NEEDED) {
 	/* Posting not allowed, try to authorize. */
 	if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0)
             sysdie("authorization error");
 	i = OfferArticle(buff, true);
     }
-    if (i != NNTP_START_POST_VAL)
+    if (i != NNTP_CONT_POST)
         die("server doesn't want the article: %s", buff);
 
     /* Write the headers, a blank line, then the article. */
@@ -1082,7 +1088,7 @@ main(int ac, char *av[])
 	*p = '\0';
     if ((p = strchr(buff, '\n')) != NULL)
 	*p = '\0';
-    if (atoi(buff) != NNTP_POSTEDOK_VAL)
+    if (atoi(buff) != NNTP_OK_POST)
         die("cannot send article to server: %s", buff);
 
     /* Close up. */
