@@ -1,4 +1,4 @@
-/*  $Id: sys2nf.c 8230 2008-12-14 16:49:28Z iulius $
+/*  $Id: sys2nf.c 8231 2008-12-14 16:53:54Z iulius $
 **
 **  Read a C news "sys" file and split it up into a set of INN
 **  newsfeeds entries.  Also works with B news.
@@ -14,7 +14,8 @@
 #include <sys/stat.h>
 
 #include "inn/innconf.h"
-#include "libinn.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 #include "nntp.h"
 
 #define TEMPFILE	":tmp"
@@ -259,7 +260,7 @@ main(ac, av)
     if (!innconf_read(NULL))
         exit(1);
     /* Set defaults. */
-    act = "/usr/local/lib/newslib/active";
+    act = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     sys = "sys";
     dir = "feeds";
     while ((i = getopt(ac, av, "a:s:d:")) != EOF)
