@@ -1,4 +1,4 @@
-/* $Id: hisinterface.h,v 1.4 2002/09/08 19:52:11 rra Exp $
+/* $Id: hisinterface.h 6504 2003-10-20 21:47:56Z rra $
 **
 ** Interface to history API modules
 */
@@ -43,5 +43,5 @@ enum { S_HIScacheadd, S_HIScachelookup, S_HISsetup, S_HISsync,
        S_HISwrite, S_HISremember, S_HIS_MAX };
 
 /* fine grained history logging */
-void his_logger(char *s, int code);
+void his_logger(const char *s, int code);
 #endif
