@@ -1,15 +1,12 @@
-/*  $Id: libinn.h 10100 2016-11-04 22:24:26Z iulius $
+/*  $Id: libinn.h 8255 2008-12-24 22:27:07Z iulius $
 **
 **  Here be declarations of functions in the InterNetNews library.
 */
 
-#ifndef INN_LIBINN_H
-#define INN_LIBINN_H 1
+#ifndef LIBINN_H
+#define LIBINN_H 1
 
 #include <inn/defines.h>
-#include "inn/concat.h"
-#include "inn/xmalloc.h"
-#include "inn/xwrite.h"
 
 #include <stdarg.h>             /* va_list */
 #include <stdio.h>              /* FILE */
@@ -24,24 +21,81 @@
 
 /* Forward declarations to avoid unnecessary includes. */
 struct stat;
+struct iovec;
 struct sockaddr;
 struct sockaddr_in;
 struct in_addr;
 
 BEGIN_DECLS
 
+/*
+**  MEMORY MANAGEMENT
+*/
+
+/* The functions are actually macros so that we can pick up the file and line
+   number information for debugging error messages without the user having to
+   pass those in every time. */
+#define xcalloc(n, size)        x_calloc((n), (size), __FILE__, __LINE__)
+#define xmalloc(size)           x_malloc((size), __FILE__, __LINE__)
+#define xrealloc(p, size)       x_realloc((p), (size), __FILE__, __LINE__)
+#define xstrdup(p)              x_strdup((p), __FILE__, __LINE__)
+#define xstrndup(p, size)       x_strndup((p), (size), __FILE__, __LINE__)
+#define xvasprintf(p, f, a)     x_vasprintf((p), (f), (a), __FILE__, __LINE__)
+
+/* asprintf is a special case since it takes variable arguments.  If we have
+   support for variadic macros, we can still pass in the file and line and
+   just need to put them somewhere else in the argument list than last.
+   Otherwise, just call x_asprintf directly.  This means that the number of
+   arguments x_asprintf takes must vary depending on whether variadic macros
+   are supported. */
+#ifdef INN_HAVE_C99_VAMACROS
+# define xasprintf(p, f, ...) \
+    x_asprintf((p), __FILE__, __LINE__, (f), __VA_ARGS__)
+#elif INN_HAVE_GNU_VAMACROS
+# define xasprintf(p, f, args...) \
+    x_asprintf((p), __FILE__, __LINE__, (f), args)
+#else
+# define xasprintf x_asprintf
+#endif
+
+/* Last two arguments are always file and line number.  These are internal
+   implementations that should not be called directly.  ISO C99 says that
+   identifiers beginning with _ and a lowercase letter are reserved for
+   identifiers of file scope, so while the position of libraries in the
+   standard isn't clear, it's probably not entirely kosher to use _xmalloc
+   here.  Use x_malloc instead. */
+extern void *x_calloc(size_t, size_t, const char *, int);
+extern void *x_malloc(size_t, const char *, int);
+extern void *x_realloc(void *, size_t, const char *, int);
+extern char *x_strdup(const char *, const char *, int);
+extern char *x_strndup(const char *, size_t, const char *, int);
+extern int x_vasprintf(char **, const char *, va_list, const char *, int);
+
+/* asprintf special case. */
+#if INN_HAVE_C99_VAMACROS || INN_HAVE_GNU_VAMACROS
+extern int x_asprintf(char **, const char *, int, const char *, ...);
+#else
+extern int x_asprintf(char **, const char *, ...);
+#endif
+
+/* Failure handler takes the function, the size, the file, and the line. */
+typedef void (*xmalloc_handler_type)(const char *, size_t, const char *, int);
+
+/* The default error handler. */
+void xmalloc_fail(const char *, size_t, const char *, int);
+
+/* Assign to this variable to choose a handler other than the default, which
+   just calls sysdie. */
+extern xmalloc_handler_type xmalloc_error_handler;
+
+
 /*
 **  TIME AND DATE PARSING, GENERATION, AND HANDLING
 */
-extern int      Argify(char *line, char ***argvp);
-extern int      nArgify(char *line, char ***argvp, int n);
-extern int      reArgify(char *p, char **argv, int n, bool stripspaces);
-extern char *   Glom(char **av);
 extern bool     makedate(time_t, bool local, char *buff, size_t buflen);
 extern time_t   parsedate_nntp(const char *, const char *, bool local);
-extern time_t   parsedate_rfc5322(const char *);
-extern time_t   parsedate_rfc5322_lax(const char *);
-
+extern time_t   parsedate_rfc2822(const char *);
+extern time_t   parsedate_rfc2822_lax(const char *);
 
 
 /*
@@ -53,7 +107,6 @@ enum uwildmat {
     UWILDMAT_POISON
 };
 
-extern bool             is_valid_utf8(const char *start);
 extern bool             uwildmat(const char *text, const char *pat);
 extern bool             uwildmat_simple(const char *text, const char *pat);
 extern enum uwildmat    uwildmat_poison(const char *text, const char *pat);
@@ -76,26 +129,24 @@ extern bool     inn_lock_range(int fd, enum inn_locktype type, bool block,
 /*
 **  MISCELLANEOUS UTILITY FUNCTIONS
 */
+extern void     close_on_exec(int fd, bool flag);
+extern char *   concat(const char *first, ...);
+extern char *   concatpath(const char *base, const char *name);
 extern void     daemonize(const char *path);
 extern int      getfdlimit(void);
+extern int      nonblocking(int fd, bool flag);
 extern int      setfdlimit(unsigned int limit);
+extern ssize_t  xpwrite(int fd, const void *buffer, size_t size, off_t offset);
 extern void     (*xsignal(int signum, void (*sigfunc)(int)))(int);
 extern void     (*xsignal_norestart(int signum, void (*sigfunc)(int)))(int);
-extern void     xsignal_mask(void);
-extern void     xsignal_unmask(void);
-extern void     xsignal_enable_masking(void);
-extern void     xsignal_forked(void);
+extern ssize_t  xwrite(int fd, const void *buffer, size_t size);
+extern ssize_t  xwritev(int fd, const struct iovec *iov, int iovcnt);
 
 
 /* Headers. */
 extern char *           GenerateMessageID(char *domain);
-extern void             InitializeMessageIDcclass(void);
-extern bool             IsValidMessageID(const char *string, bool stripspaces);
+extern bool             IsValidMessageID(const char *string);
 extern bool             IsValidHeaderName(const char *string);
-extern bool             IsValidHeaderBody(const char *string);
-extern bool             IsValidHeaderField(const char *string);
-extern const char *     skip_cfws(const char *p);
-extern const char *     skip_fws(const char *p);
 extern void             HeaderCleanFrom(char *from);
 extern struct _DDHANDLE * DDstart(FILE *FromServer, FILE *ToServer);
 extern void               DDcheck(struct _DDHANDLE *h, char *group);
@@ -104,8 +155,7 @@ extern char *             DDend(struct _DDHANDLE *h);
 
 /* Various checks. */
 extern bool             IsValidArticleNumber(const char *string);
-extern bool             IsValidKeyword(const char *string)
-    __attribute__ ((__pure__));
+extern bool             IsValidKeyword(const char *string);
 extern bool             IsValidRange(char *string);
 
 
@@ -177,4 +227,4 @@ END_DECLS
 /* <ctype.h>'s isspace includes \n, which is not what we want. */
 #define ISWHITE(c)              ((c) == ' ' || (c) == '\t')
 
-#endif /* INN_LIBINN_H */
+#endif /* LIBINN_H */
