@@ -1,4 +1,4 @@
-/*  $Id: nntp.h 8903 2010-01-17 18:21:56Z iulius $
+/*  $Id: nntp.h 8467 2009-05-16 12:27:51Z iulius $
 **
 **  NNTP codes and utility functions for speaking the NNTP protocol.
 **
@@ -42,12 +42,6 @@
 **  available for holding NNTP codes specifically.
 */
 
-#define NNTP_CLASS_INFO                 '1'
-#define NNTP_CLASS_OK                   '2'
-#define NNTP_CLASS_CONT                 '3'
-#define NNTP_CLASS_FAIL                 '4'
-#define NNTP_CLASS_ERR                  '5'
-
 enum nntp_code {
     NNTP_INFO_HELP              = 100,
     NNTP_INFO_CAPABILITIES      = 101,
@@ -74,11 +68,11 @@ enum nntp_code {
     NNTP_FAIL_ACTION            = 403, /* Internal fault or temporary problem */
     NNTP_FAIL_BAD_GROUP         = 411, /* Group unknown */
     NNTP_FAIL_NO_GROUP          = 412, /* Not in a newsgroup */
-    NNTP_FAIL_ARTNUM_INVALID    = 420, /* Current article is invalid */
+    NNTP_FAIL_NO_ARTICLE        = 420, /* No current article */
     NNTP_FAIL_NEXT              = 421,
     NNTP_FAIL_PREV              = 422,
-    NNTP_FAIL_ARTNUM_NOTFOUND   = 423, /* Article not found (by article number) */
-    NNTP_FAIL_MSGID_NOTFOUND    = 430, /* Article not found (by message-ID) */
+    NNTP_FAIL_BAD_ARTICLE       = 423, /* Bad article number */
+    NNTP_FAIL_NOTFOUND          = 430, /* Article not found */
     NNTP_FAIL_IHAVE_REFUSE      = 435, /* IHAVE article not wanted */
     NNTP_FAIL_IHAVE_DEFER       = 436, /* IHAVE article deferred */
     NNTP_FAIL_IHAVE_REJECT      = 437, /* IHAVE article rejected */
@@ -233,19 +227,15 @@ bool nntp_write(struct nntp *, const char *buffer, size_t size);
 
 /* Send a line to the remote connection.  The output is flushed after sending
    the line unless the noflush variant is used. */
-bool nntp_send_line(struct nntp *, const char *format, ...)
-    __attribute__((__format__(printf, 2, 3)));
-bool nntp_send_line_noflush(struct nntp *, const char *format, ...)
-    __attribute__((__format__(printf, 2, 3)));
+bool nntp_send_line(struct nntp *, const char *format, ...);
+bool nntp_send_line_noflush(struct nntp *, const char *format, ...);
 
 /* Send a response to an NNTP command or an opening banner.  The string may be
    NULL to indicate nothing should follow the response code; otherwise, it is
    a printf-style format.  The noflush variant doesn't flush the output after
    sending it, used for introducing multiline responses. */
-bool nntp_respond(struct nntp *, enum nntp_code, const char *, ...)
-    __attribute__((__format__(printf, 3, 4)));
-void nntp_respond_noflush(struct nntp *, enum nntp_code, const char *, ...)
-    __attribute__((__format__(printf, 3, 4)));
+bool nntp_respond(struct nntp *, enum nntp_code, const char *, ...);
+void nntp_respond_noflush(struct nntp *, enum nntp_code, const char *, ...);
 
 /* Flush NNTP output, returning true on success and false on any error. */
 bool nntp_flush(struct nntp *);
