@@ -1,33 +1,33 @@
-/*  $Id: proc.c,v 1.6 1999/10/27 10:17:37 rra Exp $
+/*  $Id: proc.c 8418 2009-04-12 21:08:22Z iulius $
 **
 **  Process control routines.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
+#include "portable/wait.h"
+
 #include "innd.h"
 
 
-STATIC PROCESS	*PROCtable;
-STATIC int	PROCtablesize;
-STATIC PROCESS	PROCnull = { PSfree };
+static PROCESS	*PROCtable;
+static int	PROCtablesize;
+static PROCESS	PROCnull = { PSfree, 0, 0, 0, 0, 0 };
 
 
 /*
 **  Collect dead processes.
 */
-STATIC void
-PROCreap()
+static void
+PROCreap(void)
 {
-    int			status;
-    register PROCESS	*pp;
-    register int	i;
-    register PID_T	pid;
+    int		status;
+    PROCESS	*pp;
+    int         i;
+    pid_t	pid;
 
     for ( ; ; ) {
-	pid = waitnb(&status);
+	pid = waitpid(-1, &status, WNOHANG);
 	if (pid == 0)
 	    break;
 	if (pid < 0) {
@@ -37,10 +37,10 @@ PROCreap()
 	}
 	for (pp = PROCtable, i = PROCtablesize; --i >= 0; pp++)
 	    if (pp->Pid == pid) {
-		PROCneedscan = TRUE;
-		pp->Status = status;
+		PROCneedscan = true;
+		pp->Status = WEXITSTATUS(status);
 		pp->State = PSdead;
-		pp->Collected = Now.time;
+		pp->Collected = Now.tv_sec;
 		break;
 	    }
     }
@@ -50,13 +50,15 @@ PROCreap()
 /*
 **  Signal handler that collects the processes, then resets the signal.
 */
-STATIC SIGHANDLER
-PROCcatchsignal(s)
-    int			s;
+static void
+PROCcatchsignal(int s)
 {
     PROCreap();
+
 #ifndef HAVE_SIGACTION
     xsignal(s, PROCcatchsignal);
+#else
+    s = s;			/* ARGSUSED */
 #endif
 }
 
@@ -65,10 +67,10 @@ PROCcatchsignal(s)
 **  Synchronous version that notifies a site when its process went away.
 */
 void
-PROCscan()
+PROCscan(void)
 {
-    register PROCESS	*pp;
-    register int	i;
+    PROCESS	*pp;
+    int	i;
 
     for (pp = PROCtable, i = PROCtablesize; --i >= 0; pp++)
 	if (pp->State == PSdead) {
@@ -76,7 +78,7 @@ PROCscan()
 		SITEprocdied(&Sites[pp->Site], pp - PROCtable, pp);
 	    pp->State = PSfree;
 	}
-    PROCneedscan = FALSE;
+    PROCneedscan = false;
 }
 
 
@@ -86,11 +88,11 @@ PROCscan()
 */
 void
 PROCclose(Quickly)
-    BOOL		Quickly;
+    bool		Quickly;
 {
-    register int	sig;
-    register PROCESS	*pp;
-    register int	i;
+    int	sig;
+    PROCESS	*pp;
+    int	i;
 
     /* What signal are we sending? */
     sig = Quickly ? SIGKILL : SIGTERM;
@@ -117,8 +119,7 @@ PROCclose(Quickly)
 **  Stop watching a process -- we don't care about it any more.
 */
 void
-PROCunwatch(process)
-    int		process;
+PROCunwatch(int process)
 {
     if (process < 0 || process >= PROCtablesize) {
 	syslog(L_ERROR, "%s internal PROCunwatch %d", LogName, process);
@@ -132,12 +133,10 @@ PROCunwatch(process)
 **  Add a pid to the list of processes we watch.
 */
 int
-PROCwatch(pid, site)
-    PID_T		pid;
-    int			site;
+PROCwatch(pid_t pid, int site)
 {
-    register PROCESS	*pp;
-    register int	i;
+    PROCESS     *pp;
+    int         i;
 
     /* Find a free slot for this process. */
     for (pp = PROCtable, i = PROCtablesize; --i >= 0; pp++)
@@ -145,7 +144,7 @@ PROCwatch(pid, site)
 	    break;
     if (i < 0) {
 	/* Ran out of room -- grow the table. */
-	RENEW(PROCtable, PROCESS, PROCtablesize + 20);
+        PROCtable = xrealloc(PROCtable, (PROCtablesize + 20) * sizeof(PROCESS));
         for (pp = &PROCtable[PROCtablesize], i=20; --i >= 0; pp++)
           *pp = PROCnull;
 	pp = &PROCtable[PROCtablesize];
@@ -154,7 +153,7 @@ PROCwatch(pid, site)
 
     pp->State = PSrunning;
     pp->Pid = pid;
-    pp->Started = Now.time;
+    pp->Started = Now.tv_sec;
     pp->Site = site;
     return pp - PROCtable;
 }
@@ -164,20 +163,19 @@ PROCwatch(pid, site)
 **  Setup.
 */
 void
-PROCsetup(i)
-    register int	i;
+PROCsetup(int i)
 {
-    register PROCESS	*pp;
+    PROCESS	*pp;
 
     if (PROCtable)
-	DISPOSE(PROCtable);
+	free(PROCtable);
     PROCtablesize = i;
-    PROCtable = NEW(PROCESS, PROCtablesize);
+    PROCtable = xmalloc(PROCtablesize * sizeof(PROCESS));
     for (pp = PROCtable, i = PROCtablesize; --i >= 0; pp++)
 	*pp = PROCnull;
 
 #if	defined(SIGCHLD)
-    (void)xsignal(SIGCHLD, PROCcatchsignal);
+    xsignal(SIGCHLD, PROCcatchsignal);
 #endif	/* defined(SIGCHLD) */
-    (void)xsignal(SIGPIPE, PROCcatchsignal);
+    xsignal(SIGPIPE, PROCcatchsignal);
 }
