@@ -1,4 +1,4 @@
-/*  $Id: rc.c,v 1.83.2.1 2003/05/11 19:40:28 rra Exp $
+/*  $Id: rc.c 8463 2009-05-15 20:31:54Z iulius $
 **
 **  Routines for the remote connect channel.  Create an Internet stream
 **  socket that processes connect to.  If the incoming site is not one of
@@ -12,6 +12,8 @@
 #include <netdb.h>
 
 #include "inn/innconf.h"
+#include "inn/network.h"
+#include "inn/vector.h"
 #include "innd.h"
 
 #define TEST_CONFIG(a, b) \
@@ -35,6 +37,7 @@ typedef struct _REMOTEHOST {
     char 	*Identd;	/* Optional identd */
     bool	Streaming;      /* Streaming allowed ? */
     bool	Skip;	        /* Skip this peer ? */
+    bool        Ignore;        /* Ignore articles sent by this peer? */
     bool	NoResendId;	/* Don't send RESEND responses ? */
     bool	Nolist;		/* no list command allowed */
     int		MaxCnx;		/* Max connections (per peer) */
@@ -78,13 +81,14 @@ static char		RCbuff[BIG_BUFFER];
 #define EMAIL	        "email:"
 #define COMMENT	        "comment:"
 #define SKIP		"skip:"
+#define IGNORE          "ignore:"
 #define NORESENDID	"noresendid:"
 #define HOLD_TIME	"hold-time:"
 #define NOLIST		"nolist:"
 
 typedef enum {K_END, K_BEGIN_PEER, K_BEGIN_GROUP, K_END_PEER, K_END_GROUP,
 	      K_STREAM, K_HOSTNAME, K_MAX_CONN, K_PASSWORD, K_IDENTD,
-	      K_EMAIL, K_PATTERNS, K_COMMENT, K_SKIP, K_NORESENDID,
+	      K_EMAIL, K_PATTERNS, K_COMMENT, K_SKIP, K_IGNORE, K_NORESENDID,
 	      K_HOLD_TIME, K_NOLIST
 	     } _Keywords;
 
@@ -177,10 +181,12 @@ GoodIdent(int fd, char *identd)
     }
     if (bind(ident_fd,s_local,SA_LEN(s_local)) < 0) {
 	syslog(L_ERROR, "can't bind socket for identd (%m)");
+        close(ident_fd);
 	return false;
     }
     if (connect(ident_fd,s_distant,SA_LEN(s_distant)) < 0) {
 	syslog(L_ERROR, "can't connect to identd (%m)");
+        close(ident_fd);
 	return false;
     }
 
@@ -250,104 +256,41 @@ RCCommaSplit(char *text)
     return save;
 }
 
- /*
-  * Routine to disable IP-level socket options. This code was taken from 4.4BSD
-  * rlogind source, but all mistakes in it are my fault.
-  *
-  * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
-  *
-  * 21-Jan-1997 smd
-  *     Code copied again, and modified for INN, all new mistakes are mine.
-  * 
-  */
-
-/* fix_options - get rid of IP-level socket options */
-#ifndef IP_OPTIONS
-#define IP_OPTIONS 1
-#endif
-
-static int
-RCfix_options(int fd, struct sockaddr_storage *remote)
-{
-#if IP_OPTIONS
-    unsigned char optbuf[BUFSIZ / 3], *cp;
-    char    lbuf[BUFSIZ], *lp;
-    socklen_t optsize = sizeof(optbuf);
-    int     ipproto;
-    struct protoent *ip;
-
-    switch (remote->ss_family) {
-    case AF_INET:
-	if ((ip = getprotobyname("ip")) != 0)
-	    ipproto = ip->p_proto;
-	else
-	    ipproto = IPPROTO_IP;
-	break;
-#ifdef HAVE_INET6
-    case AF_INET6:
-	if ((ip = getprotobyname("ipv6")) != 0)
-	    ipproto = ip->p_proto;
-	else
-	    ipproto = IPPROTO_IPV6;
-	break;
-#endif
-    default:
-	syslog(LOG_ERR, "unknown address family: %d", remote->ss_family);
-	return -1;
-    }
-
-    if (getsockopt(fd, ipproto, IP_OPTIONS, (char *) optbuf, &optsize) == 0
-	&& optsize != 0) {
-	lp = lbuf;
-	for (cp = optbuf; optsize > 0; cp++, optsize--, lp += 3)
-	    sprintf(lp, " %2.2x", *cp);
-	syslog(LOG_NOTICE,
-	       "connect from %s with IP options (ignored):%s",
-	       sprint_sockaddr((struct sockaddr *)remote), lbuf);
-	if (setsockopt(fd, ipproto, IP_OPTIONS, (char *) 0, optsize) != 0) {
-	    syslog(LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
-	    return -1;
-	}
-    }
-#endif
-    return 0;
-}
-
 static bool
 RCaddressmatch(const struct sockaddr_storage *cp, const struct sockaddr_storage *rp)
 {
 #ifdef HAVE_INET6
-    struct sockaddr_in	*sin_cp, *sin_rp;
-    struct sockaddr_in6	*sin6_cp, *sin6_rp;
+    const struct sockaddr_in *sin_cp, *sin_rp;
+    const struct sockaddr_in6 *sin6_cp, *sin6_rp;
 
     if (cp->ss_family == AF_INET6 && rp->ss_family == AF_INET) {
-	sin6_cp = (struct sockaddr_in6 *)cp;
-	sin_rp = (struct sockaddr_in *)rp;
+	sin6_cp = (const struct sockaddr_in6 *)cp;
+	sin_rp = (const struct sockaddr_in *)rp;
 	if (IN6_IS_ADDR_V4MAPPED(&sin6_cp->sin6_addr) &&
 		memcmp(&sin6_cp->sin6_addr.s6_addr[12],
 		    &sin_rp->sin_addr.s_addr, sizeof(struct in_addr)) == 0)
 	    return true;
     } else if (cp->ss_family == AF_INET && rp->ss_family == AF_INET6) {
-	sin_cp = (struct sockaddr_in *)cp;
-	sin6_rp = (struct sockaddr_in6 *)rp;
+	sin_cp = (const struct sockaddr_in *)cp;
+	sin6_rp = (const struct sockaddr_in6 *)rp;
 	if (IN6_IS_ADDR_V4MAPPED(&sin6_rp->sin6_addr) &&
 		memcmp(&sin6_rp->sin6_addr.s6_addr[12],
 		    &sin_cp->sin_addr.s_addr, sizeof(struct in_addr)) == 0)
 	    return true;
     } else if (cp->ss_family == AF_INET6 && rp->ss_family == AF_INET6) {
 #ifdef HAVE_BROKEN_IN6_ARE_ADDR_EQUAL
-	if (!memcmp(&((struct sockaddr_in6 *)cp)->sin6_addr,
-		    &((struct sockaddr_in6 *)rp)->sin6_addr,
+	if (!memcmp(&((const struct sockaddr_in6 *)cp)->sin6_addr,
+		    &((const struct sockaddr_in6 *)rp)->sin6_addr,
 		    sizeof(struct in6_addr)))
 #else
-	if (IN6_ARE_ADDR_EQUAL( &((struct sockaddr_in6 *)cp)->sin6_addr,
-				&((struct sockaddr_in6 *)rp)->sin6_addr))
+	if (IN6_ARE_ADDR_EQUAL( &((const struct sockaddr_in6 *)cp)->sin6_addr,
+				&((const struct sockaddr_in6 *)rp)->sin6_addr))
 #endif
 	    return true;
     } else
 #endif /* INET6 */
-	if (((struct sockaddr_in *)cp)->sin_addr.s_addr ==
-	     ((struct sockaddr_in *)rp)->sin_addr.s_addr)
+	if (((const struct sockaddr_in *)cp)->sin_addr.s_addr ==
+	     ((const struct sockaddr_in *)rp)->sin_addr.s_addr)
 	    return true;
 
     return false;
@@ -359,21 +302,23 @@ RCaddressmatch(const struct sockaddr_storage *cp, const struct sockaddr_storage
 bool
 RCauthorized(CHANNEL *cp, char *pass)
 {
-    REMOTEHOST	*rp;
-    int		i;
+    REMOTEHOST *rp;
+    int	i;
+    char addr[INET6_ADDRSTRLEN];
 
+    network_sockaddr_sprint(addr, sizeof(addr),
+                            (struct sockaddr *) &cp->Address);
     for (rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++)
 	if (RCaddressmatch(&cp->Address, &rp->Address)) {
 	    if (rp->Password[0] == '\0' || strcmp(pass, rp->Password) == 0)
 		return true;
-	    syslog(L_ERROR, "%s (%s) bad_auth", rp->Label,
-		   sprint_sockaddr((struct sockaddr *)&cp->Address));
+            warn("%s (%s) bad_auth", rp->Label, addr);
 	    return false;
 	}
 
+    /* Not found in our table; this can't happen. */
     if (!AnyIncoming)
-	/* Not found in our table; this can't happen. */
-	syslog(L_ERROR, "%s not_found", sprint_sockaddr((struct sockaddr *)&cp->Address));
+        warn("%s not_found", addr);
 
     /* Anonymous hosts should not authenticate. */
     return false;
@@ -419,8 +364,11 @@ RClimit(CHANNEL *cp)
 static void
 RCrejectreader(CHANNEL *cp)
 {
-    syslog(L_ERROR, "%s internal RCrejectreader (%s)", LogName,
-	   sprint_sockaddr((struct sockaddr *)&cp->Address));
+    char addr[INET6_ADDRSTRLEN];
+
+    network_sockaddr_sprint(addr, sizeof(addr),
+                            (struct sockaddr *) &cp->Address);
+    warn("%s internal RCrejectreader (%s)", LogName, addr);
 }
 
 
@@ -448,9 +396,14 @@ RCrejectwritedone(CHANNEL *cp)
 void
 RChandoff(int fd, HANDOFF h)
 {
-    const char *argv[6];
+    const char **argv;
     char buff[SMBUF];
     int i;
+    unsigned int j;
+    struct vector *flags;
+    
+    flags = vector_split_space(innconf->nnrpdflags, NULL);
+    argv  = xmalloc( (flags->count + 6) * sizeof(char*) );
 
     if (RCnnrpd == NULL)
 	RCnnrpd = concatpath(innconf->pathbin, "nnrpd");
@@ -482,11 +435,17 @@ RChandoff(int fd, HANDOFF h)
 	argv[i++] = "-t";
     if (RCslaveflag)
 	argv[i++] = RCslaveflag;
+
+    for(j = 0; j < flags->count; j++) {
+        argv[i++] = flags->strings[j];
+    }
     argv[i] = NULL;
 
     /* Call NNRP; don't send back a QUIT message if Spawn fails since  
      * that's a major error we want to find out about quickly. */
-    (void)Spawn(innconf->nicekids, fd, fd, fd, (char * const *)argv);
+    Spawn(innconf->nicekids, fd, fd, fd, (char * const *)argv);
+    vector_free(flags);
+    free(argv);
 }
 
 
@@ -511,6 +470,7 @@ RCreader(CHANNEL *cp)
     time_t		now;
     CHANNEL		tempchan;
     char		buff[SMBUF];
+    char                addr[INET6_ADDRSTRLEN];
 
     for (i = 0 ; i < chanlimit ; i++) {
 	if (RCchan[i] == cp) {
@@ -530,19 +490,11 @@ RCreader(CHANNEL *cp)
 	    syslog(L_ERROR, "%s cant accept RCreader %m", LogName);
 	return;
     }
-
-    /*
-    ** Clear any IP_OPTIONS, including source routing, on the socket
-    */
-    /* FIXME RCfix_options breaks IPv6 sockets, at least on Linux -lutchann */
-#ifndef HAVE_INET6
-    if (RCfix_options(fd, &remote) != 0) {
-	/* shouldn't happen, but we're bit paranoid at this point */
-	if (close(fd) < 0)
-	    syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
-	return;
+    if (!network_kill_options(fd, (struct sockaddr *) &remote)) {
+        if (close(fd) < 0)
+            syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
+        return;
     }
-#endif
 
     /* If RemoteTimer is not zero, then check the limits on incoming
        connections on a total and per host basis.
@@ -595,11 +547,11 @@ RCreader(CHANNEL *cp)
 	    i = (i + 1) & (REMOTETABLESIZE - 1);
 	}
 	if (remotecount == RemoteTotal) {
-	    reject_val = NNTP_GOODBYE_VAL;
+	    reject_val = NNTP_FAIL_TERMINATING;
 	    reject_message = "400 Server overloaded, try later";
 	}
 	else if (found >= RemoteLimit && !RCnolimit(&tempchan)) {
-	    reject_val = NNTP_GOODBYE_VAL;
+	    reject_val = NNTP_FAIL_TERMINATING;
 	    reject_message = "400 Connection rejected, you're making too"
                 " many connects per minute";
 	}
@@ -652,13 +604,15 @@ RCreader(CHANNEL *cp)
 	if ((new = NCcreate(fd, rp->Password[0] != '\0', false)) != NULL) {
             new->Streaming = rp->Streaming;
             new->Skip = rp->Skip;
+            new->Ignore = rp->Ignore;
             new->NoResendId = rp->NoResendId;
             new->Nolist = rp->Nolist;
+            new->CanAuthenticate = true; /* Can use AUTHINFO. */
             new->MaxCnx = rp->MaxCnx;
             new->HoldTime = rp->HoldTime;
 	    memcpy(&new->Address, &remote, SA_LEN((struct sockaddr *)&remote));
 	    if (new->MaxCnx > 0 && new->HoldTime == 0) {
-		CHANsetActiveCnx(new);
+		CHANcount_active(new);
 		if((new->ActiveCnx > new->MaxCnx) && (new->fd > 0)) {
 		    snprintf(buff, sizeof(buff),
                              "You are limited to %d connection%s",
@@ -682,7 +636,7 @@ RCreader(CHANNEL *cp)
 	    syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
 	return;
     } else {
-	reject_val = NNTP_ACCESS_VAL;
+	reject_val = NNTP_ERR_ACCESS;
 	reject_message = NNTP_ACCESS;
         new = CHANcreate(fd, CTreject, CSwritegoodbye, RCrejectreader,
             RCrejectwritedone);
@@ -697,9 +651,10 @@ RCreader(CHANNEL *cp)
 
     if (new != NULL) {
 	memcpy(&new->Address, &remote, SA_LEN((struct sockaddr *)&remote));
-	syslog(L_NOTICE, "%s connected %d streaming %s",
-           name ? name : sprint_sockaddr((struct sockaddr *)&new->Address),
-	   new->fd, (!StreamingOff && new->Streaming) ? "allowed" : "not allowed");
+        network_sockaddr_sprint(addr, sizeof(addr),
+                                (struct sockaddr *) &remote);
+        notice("%s connected %d streaming %s", name ? name : addr, new->fd,
+               (!StreamingOff && new->Streaming) ? "allowed" : "not allowed");
     }
 }
 
@@ -708,9 +663,8 @@ RCreader(CHANNEL *cp)
 **  Write-done function.  Shouldn't happen.
 */
 static void
-RCwritedone(CHANNEL *unused)
+RCwritedone(CHANNEL *unused UNUSED)
 {
-    unused = unused;		/* ARGSUSED */
     syslog(L_ERROR, "%s internal RCwritedone", LogName);
 }
 
@@ -834,12 +788,6 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     char 		*p;
     char 		**q;
     char 		**r;
-#if     !defined( HAVE_INET6)
-    struct hostent	*hp;
-#endif
-#if	!defined(HAVE_UNIX_DOMAIN_SOCKETS) || !defined(HAVE_INET6)
-    struct in_addr      addr;
-#endif
     int                 i;
     int                 j;
     int			linecount;
@@ -854,7 +802,12 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     REMOTEHOST		peer_params;
     REMOTEHOST		default_params;
     bool		flag, bit, toolong;
- 
+#ifndef HAVE_UNIX_DOMAIN_SOCKETS
+    struct addrinfo     hints, *ai;
+    int                 ret;
+#endif
+
+    memset(&peer_params, 0, sizeof(peer_params));
     *RCbuff = '\0';
     if (*list) {
 	for (rp = *list, i = *count; --i >= 0; rp++) {
@@ -892,8 +845,14 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     rp = *list = xmalloc(sizeof(REMOTEHOST));
 
 #if	!defined(HAVE_UNIX_DOMAIN_SOCKETS)
-    addr.s_addr = INADDR_LOOPBACK;
-    make_sin( (struct sockaddr_in *)&rp->Address, &addr );
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_INET;
+    hints.ai_flags = AI_NUMERICHOST;
+    ret = getaddrinfo("127.0.0.1", NULL, &hints, &ai);
+    if (ret != 0)
+        die("%s cant getaddrinfo 127.0.0.1: %s", LogName, gai_strerror(ret));
+    memcpy(&rp->Address, ai->ai_addr, ai->ai_addrlen);
+    freeaddrinfo(ai);
     rp->Name = xstrdup("localhost");
     rp->Label = xstrdup("localhost");
     rp->Email = xstrdup(NOEMAIL);
@@ -904,6 +863,7 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     rp->MaxCnx = 0;
     rp->Streaming = true;
     rp->Skip = false;
+    rp->Ignore = false;
     rp->NoResendId = false;
     rp->Nolist = false;
     rp->HoldTime = 0;
@@ -918,6 +878,7 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
     peer_params.Label = NULL;
     default_params.Streaming = true;
     default_params.Skip = false;
+    default_params.Ignore = false;
     default_params.NoResendId = false;
     default_params.Nolist = false;
     default_params.MaxCnx = 0;
@@ -943,17 +904,24 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	RCadddata(data, &infocount, K_BEGIN_GROUP, T_STRING, word);
 	groupcount++;
 	if (groupcount == 1) {
+          /* First group block. */
           group_params = groups = xmalloc(sizeof(REMOTEHOST));
 	}
 	else if (groupcount >= maxgroup) {
-          /* alloc 5 groups */
+          /* Alloc 5 groups for extra nested group blocks. */
           groups = xrealloc(groups, (groupcount + 4) * sizeof(REMOTEHOST));
-	  maxgroup += 5;
-	  group_params = groups + groupcount - 1;
-	}
+          maxgroup += 5;
+          group_params = groups + groupcount - 1;
+        }
+        else {
+          /* Nested group block (no need to extend groups). */
+          group_params++;
+        }
 	group_params->Label = word;
 	group_params->Skip = groupcount > 1 ?
 	  groups[groupcount - 2].Skip : default_params.Skip;
+        group_params->Ignore = groupcount > 1 ?
+            groups[groupcount - 2].Ignore : default_params.Ignore;
 	group_params->Streaming = groupcount > 1 ?
 	  groups[groupcount - 2].Streaming : default_params.Streaming;
 	group_params->NoResendId = groupcount > 1 ?
@@ -1011,6 +979,8 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	peer_params.Name = NULL;
 	peer_params.Skip = groupcount > 0 ?
 	  group_params->Skip : default_params.Skip;
+        peer_params.Ignore = groupcount > 0 ?
+            group_params->Ignore : default_params.Ignore;
 	peer_params.Streaming = groupcount > 0 ?
 	  group_params->Streaming : default_params.Streaming;
 	peer_params.NoResendId = groupcount > 0 ?
@@ -1061,10 +1031,9 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
             peer_params.Name = xstrdup(peer_params.Label);
 
 	  for(r = q = RCCommaSplit(xstrdup(peer_params.Name)); *q != NULL; q++) {
-#ifdef HAVE_INET6
 	      struct addrinfo *res, *res0, hints;
 	      int gai_ret;
-#endif
+
 	    (*count)++;
 
 	    /* Grow the array */
@@ -1072,7 +1041,6 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
             *list = xrealloc(*list, *count * sizeof(REMOTEHOST));
 	    rp = *list + j;
 
-#ifdef HAVE_INET6
 	    memset( &hints, 0, sizeof( hints ) );
 	    hints.ai_socktype = SOCK_STREAM;
 	    hints.ai_family = PF_UNSPEC;
@@ -1102,6 +1070,7 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 		rp->Comment = xstrdup(peer_params.Comment);
 		rp->Streaming = peer_params.Streaming;
 		rp->Skip = peer_params.Skip;
+                rp->Ignore = peer_params.Ignore;
 		rp->NoResendId = peer_params.NoResendId;
 		rp->Nolist = peer_params.Nolist;
 		rp->Password = xstrdup(peer_params.Password);
@@ -1113,123 +1082,6 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 		rp++;
 	    }
 	    freeaddrinfo(res0);
-#else /* HAVE_INET6 */
-	    /* Was host specified as a dotted quad ? */
-	    if (inet_aton(*q, &addr)) {
-	      make_sin( (struct sockaddr_in *)&rp->Address, &addr );
-	      rp->Name = xstrdup (*q);
-	      rp->Label = xstrdup (peer_params.Label);
-	      rp->Password = xstrdup(peer_params.Password);
-	      rp->Identd = xstrdup(peer_params.Identd);
-	      rp->Skip = peer_params.Skip;
-	      rp->Streaming = peer_params.Streaming;
-	      rp->NoResendId = peer_params.NoResendId;
-	      rp->Nolist = peer_params.Nolist;
-	      rp->Email = xstrdup(peer_params.Email);
-	      rp->Comment = xstrdup(peer_params.Comment);
-	      rp->Patterns = peer_params.Pattern != NULL ?
-		    RCCommaSplit(xstrdup(peer_params.Pattern)) : NULL;
-	      rp->MaxCnx = peer_params.MaxCnx;
-	      rp->HoldTime = peer_params.HoldTime;
-	      rp++;
-	      continue;
-	    }
-	    
-	    /* Host specified as a text name ? */
-	    if ((hp = gethostbyname(*q)) == NULL) {
-	      syslog(L_ERROR, "%s cant gethostbyname %s %m", LogName, *q);
-	      /* decrement *count, since we never got to add this record. */
-	      (*count)--;
-	      continue;
-	    }
-
-	    /* Count the adresses and see if we have to grow the list */
-	    for (i = 0; hp->h_addr_list[i]; i++)
-	      continue;
-	    if (i == 0) {
-	      syslog(L_ERROR, "%s no_address %s %m", LogName, *q);
-	      continue;
-	    }
-	    if (i == 1) {
-	      char **rr;
-	      int    t = 0;
-	      /* Strange DNS ? try this.. */
-	      for (rr = hp->h_aliases; *rr != 0; rr++) {
-                if (!inet_aton(*rr, &addr))
-		  continue;
-		(*count)++;
-		/* Grow the array */
-		j = rp - *list;
-                *list = xrealloc(*list, *count * sizeof(REMOTEHOST));
-		rp = *list + j;
-
-		make_sin( (struct sockaddr_in *)&rp->Address, &addr );
-		rp->Name = xstrdup (*q);
-		rp->Label = xstrdup (peer_params.Label);
-		rp->Email = xstrdup(peer_params.Email);
-		rp->Comment = xstrdup(peer_params.Comment);
-		rp->Streaming = peer_params.Streaming;
-		rp->Skip = peer_params.Skip;
-		rp->NoResendId = peer_params.NoResendId;
-		rp->Nolist = peer_params.Nolist;
-		rp->Password = xstrdup(peer_params.Password);
-		rp->Identd = xstrdup(peer_params.Identd);
-		rp->Patterns = peer_params.Pattern != NULL ?
-		  RCCommaSplit(xstrdup(peer_params.Pattern)) : NULL;
-		rp->MaxCnx = peer_params.MaxCnx;
-		rp->HoldTime = peer_params.HoldTime;
-		rp++;
-		t++;
-	      }
-	      if (t == 0) {
-		/* Just one, no need to grow. */
-		make_sin( (struct sockaddr_in *)&rp->Address,
-				(struct in_addr *)hp->h_addr_list[0] );
-		rp->Name = xstrdup (*q);
-		rp->Label = xstrdup (peer_params.Label);
-		rp->Email = xstrdup(peer_params.Email);
-		rp->Comment = xstrdup(peer_params.Comment);
-		rp->Streaming = peer_params.Streaming;
-		rp->Skip = peer_params.Skip;
-		rp->NoResendId = peer_params.NoResendId;
-		rp->Nolist = peer_params.Nolist;
-		rp->Password = xstrdup(peer_params.Password);
-		rp->Identd = xstrdup(peer_params.Identd);
-		rp->Patterns = peer_params.Pattern != NULL ?
-		  RCCommaSplit(xstrdup(peer_params.Pattern)) : NULL;
-		rp->MaxCnx = peer_params.MaxCnx;
-		rp->HoldTime = peer_params.HoldTime;
-		rp++;
-		continue;
-	      }
-	    }
-	    /* Grow the array */
-	    j = rp - *list;
-	    *count += i - 1;
-            *list = xrealloc(*list, *count * sizeof(REMOTEHOST));
-	    rp = *list + j;
-
-	    /* Add all the hosts. */
-	    for (i = 0; hp->h_addr_list[i]; i++) {
-	      make_sin( (struct sockaddr_in *)&rp->Address,
-			      (struct in_addr *)hp->h_addr_list[i] );
-	      rp->Name = xstrdup (*q);
-	      rp->Label = xstrdup (peer_params.Label);
-	      rp->Email = xstrdup(peer_params.Email);
-	      rp->Comment = xstrdup(peer_params.Comment);
-	      rp->Streaming = peer_params.Streaming;
-	      rp->Skip = peer_params.Skip;
-	      rp->NoResendId = peer_params.NoResendId;
-	      rp->Nolist = peer_params.Nolist;
-	      rp->Password = xstrdup(peer_params.Password);
-	      rp->Identd = xstrdup(peer_params.Identd);
-	      rp->Patterns = peer_params.Pattern != NULL ?
-		RCCommaSplit(xstrdup(peer_params.Pattern)) : NULL;
-	      rp->MaxCnx = peer_params.MaxCnx;
-	      rp->HoldTime = peer_params.HoldTime;
-	      rp++;
-	    }
-#endif /* HAVE_INET6 */
 	  }
 	  free(r[0]);
 	  free(r);
@@ -1239,10 +1091,13 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	  RCadddata(data, &infocount, K_END_GROUP, T_STRING, NULL);
 	  group_params->Label = NULL;
 	  groupcount--;
-	  if (groupcount == 0)
-	    free(groups);
-	  else
-	    group_params--;
+         if (groupcount == 0) {
+           /* We are now outside a group block. */
+           free(groups);
+           maxgroup = 0;
+         } else {
+           group_params--;
+         }
 	}
 	else {
 	  syslog(L_ERROR, RIGHT_BRACE, LogName, linecount, filename);
@@ -1314,6 +1169,38 @@ RCreadfile (REMOTEHOST_DATA **data, REMOTEHOST **list, int *count,
 	continue;
       }
 
+      /* ignore */
+      if (!strncmp (word, IGNORE, sizeof IGNORE)) {
+          free(word);
+          TEST_CONFIG(K_IGNORE, bit);
+          if (bit) {
+              syslog(L_ERROR, DUPLICATE_KEY, LogName, filename, linecount);
+              break;
+          }
+          if ((word = RCreaddata (&linecount, F, &toolong)) == NULL) {
+              break;
+          }
+          if (!strcmp (word, "true"))
+              flag = true;
+          else
+              if (!strcmp (word, "false"))
+                  flag = false;
+              else {
+                  syslog(L_ERROR, MUST_BE_BOOL, LogName, filename, linecount);
+                  break;
+              }
+          RCadddata(data, &infocount, K_IGNORE, T_STRING, word);
+          if (peer_params.Label != NULL)
+              peer_params.Ignore = flag;
+          else
+              if (groupcount > 0 && group_params->Label != NULL)
+                  group_params->Ignore = flag;
+              else
+                  default_params.Ignore = flag;
+          SET_CONFIG(K_IGNORE);
+          continue;
+      }
+
       /* noresendid */
       if (!strncmp (word, NORESENDID, sizeof NORESENDID)) {
 	free(word);
@@ -1633,7 +1520,7 @@ RCwritelistvalue(FILE *F, char *value)
 }
 
 /*
-**  Write the incoming configuration (memory->disk)
+**  Write the incoming configuration (memory->disk).
 */
 static void UNUSED
 RCwritelist(char *filename)
@@ -1653,13 +1540,14 @@ RCwritelist(char *filename)
     /* Write a standard header.. */
 
     /* Find the filename */
-    p = concatpath(innconf->pathetc, _PATH_INNDHOSTS);
+    p = concatpath(innconf->pathetc, INN_PATH_INNDHOSTS);
     for (r = q = p; *p; p++)
         if (*p == '/')
 	   q = p + 1;
 
-    fprintf (F, "##  $Revision: 1.83.2.1 $\n");
-    fprintf (F, "##  %s - names and addresses that feed us news\n", q);
+    fprintf (F, "##  $Id: rc.c 8463 2009-05-15 20:31:54Z iulius $\n");
+    fprintf (F, "##\n");
+    fprintf (F, "##  %s -- Configuration of incoming news feeds\n", q);
     free(r);
     fprintf (F, "##\n\n");
 
@@ -1698,6 +1586,12 @@ RCwritelist(char *filename)
 	    RCwritelistvalue (F, RCpeerlistfile[i].value);
 	    fputc ('\n', F);
 	    break;
+          case K_IGNORE:
+            RCwritelistindent (F, inc);
+            fprintf(F, "%s\t", IGNORE);
+            RCwritelistvalue (F, RCpeerlistfile[i].value);
+            fputc ('\n', F);
+            break;
 	  case K_NORESENDID:
 	    RCwritelistindent (F, inc);
 	    fprintf(F, "%s\t", NORESENDID);
@@ -1759,7 +1653,7 @@ RCwritelist(char *filename)
 	    fputc ('\n', F);
 	    break;
 	  default:
-	    fprintf(F, "# ***ERROR***\n");
+	    fprintf(F, "##  ***ERROR***\n");
 	}
     }
     if (Fclose(F) == EOF)
@@ -1773,7 +1667,7 @@ RCreadlist(void)
     static char	*INNDHOSTS = NULL;
 
     if (INNDHOSTS == NULL)
-	INNDHOSTS = concatpath(innconf->pathetc, _PATH_INNDHOSTS);
+	INNDHOSTS = concatpath(innconf->pathetc, INN_PATH_INNDHOSTS);
     StreamingOff = false;
     RCreadfile(&RCpeerlistfile, &RCpeerlist, &RCnpeerlist, INNDHOSTS);
     /* RCwritelist("/tmp/incoming.conf.new"); */
@@ -1785,15 +1679,15 @@ RCreadlist(void)
 char *
 RChostname(const CHANNEL *cp)
 {
-    static char	buff[SMBUF];
+    static char	buff[INET6_ADDRSTRLEN];
     REMOTEHOST	*rp;
     int		i;
 
     for (rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++)
 	if (RCaddressmatch(&cp->Address, &rp->Address))
 	    return rp->Name;
-    strlcpy(buff, sprint_sockaddr((struct sockaddr *)&cp->Address),
-            sizeof(buff));
+    network_sockaddr_sprint(buff, sizeof(buff),
+                            (struct sockaddr *) &cp->Address);
     return buff;
 }
 
@@ -1852,96 +1746,80 @@ RCcanpost(CHANNEL *cp, char *group)
 
 
 /*
-**  Create the channel.
+**  Create the listening channels.  When IPv6 is available, if only one of
+**  bindaddress and bindaddress6 are set, INN only listens on that address.
+**  If both are set, INN listens to both IPv4 and IPv6 connections on those
+**  addresses.  If neither is set, use getaddrinfo to walk through all the
+**  possible addresses.  (When IPv6 isn't available, this is obviously a lot
+**  simpler.)
 */
 void
-RCsetup(int i)
+RCsetup(void)
 {
-#if	defined(SO_REUSEADDR)
-    int		on;
-#endif	/* defined(SO_REUSEADDR) */
-    int		j;
-    CHANNEL	*rcchan;
+    CHANNEL *rc;
+    int count, i, start;
+    int *fds;
+    bool okay;
 
-    /* This code is called only when inndstart is not being used */
-    if (i < 0) {
-#ifdef HAVE_INET6
-	syslog(L_FATAL, "%s innd MUST be started with inndstart", LogName);
-	exit(1);
-#else
-	/* Create a socket and name it. */
-	struct sockaddr_in	server;
-
-	if ((i = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-	    syslog(L_FATAL, "%s cant socket RCreader %m", LogName);
-	    exit(1);
-	}
-#if	defined(SO_REUSEADDR)
-	on = 1;
-	if (setsockopt(i, SOL_SOCKET, SO_REUSEADDR, &on, sizeof on) < 0)
-	    syslog(L_ERROR, "%s cant setsockopt RCreader %m", LogName);
-#endif	/* defined(SO_REUSEADDR) */
-	memset(&server, 0, sizeof server);
-	server.sin_port = htons(innconf->port);
-	server.sin_family = AF_INET;
-#ifdef HAVE_SOCKADDR_LEN
-	server.sin_len = sizeof( struct sockaddr_in );
-#endif
-	server.sin_addr.s_addr = htonl(INADDR_ANY);
-	if (innconf->bindaddress) {
-            if (!inet_aton(innconf->bindaddress, &server.sin_addr)) {
-                syslog(L_FATAL, "unable to determine bind ip (%s) %m",
-                       innconf->bindaddress);
-		exit(1);
-	    }
-	}
-	if (bind(i, (struct sockaddr *)&server, sizeof server) < 0) {
-	    syslog(L_FATAL, "%s cant bind RCreader %m", LogName);
-	    exit(1);
-	}
-#endif /* HAVE_INET6 */
+    /* If neither bindaddress or bindaddress6 are set, we have to do this the
+       hard way.  Either way, allocate an fds array to hold the file
+       descriptors.  Two is probably always sufficient, but we can't tell for
+       sure how many will be returned by getaddrinfo and this only happens on
+       startup so doing a bit of memory allocation won't hurt. */
+    if (innconf->bindaddress == NULL && innconf->bindaddress6 == NULL)
+        network_bind_all(innconf->port, &fds, &count);
+    else {
+        if (innconf->bindaddress != NULL && innconf->bindaddress6 != NULL)
+            count = 2;
+        else
+            count = 1;
+        fds = xmalloc(count * sizeof(int));
+        i = 0;
+        if (innconf->bindaddress6 != NULL)
+            fds[i++] = network_bind_ipv6(innconf->bindaddress6, innconf->port);
+        if (innconf->bindaddress != NULL)
+            fds[i] = network_bind_ipv4(innconf->bindaddress, innconf->port);
     }
 
-    /* Set it up to wait for connections. */
-    if (listen(i, MAXLISTEN) < 0) {
-	j = errno;
-	syslog(L_FATAL, "%s cant listen RCreader %m", LogName);
-	/* some IPv6 systems already listening on any address will 
-	   return EADDRINUSE when trying to listen on the IPv4 socket */
-	if (j == EADDRINUSE)
-	   return;
-	exit(1);
-    }
-
-    rcchan = CHANcreate(i, CTremconn, CSwaiting, RCreader, RCwritedone);
-    syslog(L_NOTICE, "%s rcsetup %s", LogName, CHANname(rcchan));
-    RCHANadd(rcchan);
-
-    for (j = 0 ; j < chanlimit ; j++ ) {
-	if (RCchan[j] == NULL) {
-	    break;
-	}
-    }
-    if (j < chanlimit) {
-	RCchan[j] = rcchan;
-    } else if (chanlimit == 0) {
-	/* assuming two file descriptors(AF_INET and AF_INET6) */
-	chanlimit = 2;
-        RCchan = xmalloc(chanlimit * sizeof(CHANNEL **));
-	for (j = 0 ; j < chanlimit ; j++ ) {
-	    RCchan[j] = NULL;
-	}
-	RCchan[0] = rcchan;
+    /* Make sure that the RCchan array is of the appropriate size. */
+    if (chanlimit == 0) {
+        chanlimit = count;
+        RCchan = xmalloc(chanlimit * sizeof(CHANNEL *));
+        start = 0;
     } else {
-	/* extend to double size */
-        RCchan = xrealloc(RCchan, chanlimit * 2 * sizeof(CHANNEL **));
-	for (j = chanlimit ; j < chanlimit * 2 ; j++ ) {
-	    RCchan[j] = NULL;
-	}
-	RCchan[chanlimit] = rcchan;
-	chanlimit *= 2;
+        for (start = 0; start < chanlimit; start++)
+            if (RCchan[start] == NULL)
+                break;
+        if (chanlimit - start < count) {
+            chanlimit += count - (chanlimit - start);
+            RCchan = xrealloc(RCchan, chanlimit * sizeof(CHANNEL *));
+        }
     }
 
+    /* Walk the list of file descriptors, listen on each, and create new
+       channels for each.  Some IPv6 systems already listening on the IPv6
+       wildcard address will return EADDRINUSE when trying to listen to the
+       IPv4 socket; just ignore that error. */
+    okay = false;
+    for (i = 0; i < count; i++) {
+        if (fds[i] < 0)
+            continue;
+        if (listen(fds[i], MAXLISTEN) < 0) {
+            if (i != 0 && errno == EADDRINUSE)
+                continue;
+            syswarn("SERVER cant listen to socket");
+        }
+        rc = CHANcreate(fds[i], CTremconn, CSwaiting, RCreader, RCwritedone);
+        notice("%s rcsetup %s", LogName, CHANname(rc));
+        RCHANadd(rc);
+        RCchan[start + i] = rc;
+        okay = true;
+    }
+
+    /* Bail if we couldn't listen on any sockets. */
+    if (!okay)
+        die("SERVER cant listen on any sockets");
+
     /* Get the list of hosts we handle. */
     RCreadlist();
 }
