@@ -1,4 +1,4 @@
-/* $Id: wip.c,v 1.9.2.1 2000/05/16 05:57:06 kondou Exp $
+/* $Id: wip.c 7090 2004-12-22 21:57:27Z rra $
 **
 ** Routines for keeping track of incoming articles, articles that haven't
 ** acked from a duplex channel feed, and history caching.
@@ -6,26 +6,28 @@
 ** WIP stands for work-in-progress
 */
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
+
+#include "inn/innconf.h"
 #include "innd.h"
-#include "dbz.h"
 
 #define WIPTABLESIZE        1024
 #define WIP_ARTMAX          300		/* innfeed default max send time */
 
-STATIC WIP     *WIPtable[WIPTABLESIZE];      /* Top level of the WIP hash table */
+static WIP     *WIPtable[WIPTABLESIZE];      /* Top level of the WIP hash table */
 
-void WIPsetup(void) {
+void
+WIPsetup(void)
+{
     memset(WIPtable, '\0', sizeof(WIPtable));
 }
 
 /* Add a new entry into the table.  It is the appilications responsiblity to
    to call WIPinprogress or WIPbyid first. */
-WIP *WIPnew(const char *messageid, CHANNEL *cp) {
+WIP *
+WIPnew(const char *messageid, CHANNEL *cp)
+{
     HASH hash;
     unsigned long bucket;
     WIP *new;
@@ -35,9 +37,9 @@ WIP *WIPnew(const char *messageid, CHANNEL *cp) {
 	   sizeof(bucket) < sizeof(hash) ? sizeof(bucket) : sizeof(hash));
     bucket = bucket % WIPTABLESIZE;
     
-    new = NEW(WIP, sizeof(WIP));
+    new = xmalloc(sizeof(WIP));
     new->MessageID = hash;
-    new->Timestamp = Now.time;
+    new->Timestamp = Now.tv_sec;
     new->Chan = cp;
     /* Link the new entry into the list */
     new->Next = WIPtable[bucket];
@@ -45,7 +47,9 @@ WIP *WIPnew(const char *messageid, CHANNEL *cp) {
     return new; 
 }
 
-void WIPprecomfree(CHANNEL *cp) {
+void
+WIPprecomfree(CHANNEL *cp)
+{
     WIP *cur;
     int i;
     if (cp == NULL)
@@ -59,7 +63,9 @@ void WIPprecomfree(CHANNEL *cp) {
     }
 }
 
-void WIPfree(WIP *wp) {
+void
+WIPfree(WIP *wp)
+{
     unsigned long bucket;
     WIP *cur;
     WIP *prev = NULL;
@@ -91,12 +97,14 @@ void WIPfree(WIP *wp) {
 	prev->Next = cur->Next;
 
     /* unlink the entry and free the memory */
-    DISPOSE(wp);
+    free(wp);
 }
 
 /* Check if the given messageid is being transfered on another channel.  If
    Add is true then add the given message-id to the current channel */
-BOOL WIPinprogress(const char *msgid, CHANNEL *cp, const BOOL Precommit) {
+bool
+WIPinprogress(const char *msgid, CHANNEL *cp, bool Precommit)
+{
     WIP *wp;
     int i;
     
@@ -109,9 +117,9 @@ BOOL WIPinprogress(const char *msgid, CHANNEL *cp, const BOOL Precommit) {
 		i = WIP_ARTMAX;
 	}
  
-	if ((Now.time - wp->Timestamp) < (i + innconf->wipcheck))
-	    return TRUE;
-	if ((Now.time - wp->Timestamp) > (i + innconf->wipexpire)) {
+	if ((Now.tv_sec - wp->Timestamp) < (i + innconf->wipcheck))
+	    return true;
+	if ((Now.tv_sec - wp->Timestamp) > (i + innconf->wipexpire)) {
 	    for (i = 0 ; i < PRECOMMITCACHESIZE ; i++) {
 		if (wp->Chan->PrecommitWIP[i] == wp) {
 		    wp->Chan->PrecommitWIP[i] = (WIP *)NULL;
@@ -119,12 +127,12 @@ BOOL WIPinprogress(const char *msgid, CHANNEL *cp, const BOOL Precommit) {
 		}
 	    }
 	    WIPfree(wp);
-	    (void)WIPinprogress(msgid, cp, Precommit);
-	    return FALSE;
+	    WIPinprogress(msgid, cp, Precommit);
+	    return false;
 	}
 	if (wp->Chan == cp)
-	    return TRUE;
-	return FALSE;
+	    return true;
+	return false;
     }
     wp = WIPnew(msgid, cp);
     if (Precommit) {
@@ -137,10 +145,12 @@ BOOL WIPinprogress(const char *msgid, CHANNEL *cp, const BOOL Precommit) {
 	WIPfree(WIPbyhash(cp->CurrentMessageIDHash));
 	cp->CurrentMessageIDHash = wp->MessageID;
     }
-    return FALSE;
+    return false;
 }
 
-WIP *WIPbyid(const char *messageid) {
+WIP *
+WIPbyid(const char *messageid)
+{
     HASH hash;
     unsigned long bucket;
     WIP *wp;
@@ -158,7 +168,9 @@ WIP *WIPbyid(const char *messageid) {
     return NULL;
 }
 
-WIP *WIPbyhash(const HASH hash) {
+WIP *
+WIPbyhash(const HASH hash)
+{
     unsigned long bucket;
     WIP *wp;
 
@@ -173,4 +185,3 @@ WIP *WIPbyhash(const HASH hash) {
 
     return NULL;
 }
-
