@@ -1,4 +1,4 @@
-/*  $Id: article.c 7285 2005-06-07 06:38:24Z eagle $
+/*  $Id: article.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  The Article class for innfeed.
 **
@@ -29,8 +29,8 @@
 #include <syslog.h>
 
 #include "inn/messages.h"
-#include "libinn.h"
-#include "storage.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 
 #include "article.h"
 #include "buffer.h"
