@@ -1,4 +1,4 @@
-/*  $Id: endpoint.c 6648 2004-01-25 20:07:11Z rra $
+/*  $Id: endpoint.c 8243 2008-12-21 11:50:47Z iulius $
 **
 **  The implementation of the innfeed EndPoint object class.
 **
@@ -36,7 +36,7 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 #include "buffer.h"
 #include "configfile.h"
@@ -231,7 +231,7 @@ EndPoint newEndPoint (int fd)
       static bool sizelogged = false ;
           
 #if defined (FD_SETSIZE)
-      if (fd >= FD_SETSIZE)
+      if ((unsigned int) fd >= FD_SETSIZE)
         {
           sizelogged = true ;
           warn ("ME fd (%d) looks too big (%d -- FD_SETSIZE)", fd,
@@ -1473,11 +1473,11 @@ void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
   newInputBuffer = newBuffer (BUFF_SIZE) ;
   
   p = bufferBase (newBuff1) ; 
-  strcpy (p, "Thanks for that \"") ;
+  strlcpy (p, "Thanks for that \"", bufferSize (newBuff1)) ;
   bufferSetDataSize (newBuff1,strlen (p)) ;
   
   p = bufferBase (newBuff2) ;
-  strcpy (p,"\" very tasty\n") ;
+  strlcpy (p,"\" very tasty\n", bufferSize (newBuff2)) ;
   bufferSetDataSize (newBuff2,strlen (p)) ;
 
   writeBuffers = makeBufferArray (newBuff1,buffer[0],newBuff2,NULL) ;
@@ -1495,12 +1495,12 @@ void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
 static void printDate (TimeoutId tid, void *data) ;
 static void printDate (TimeoutId tid, void *data)
 {
-  time_t t ;
+  char dateString[30];
+  const time_t t = theTime() ;
 
-  t = theTime() ;
-  
-  d_printf (1,"Timeout (%d) time now is %ld %s",
-           (int) tid,(long) t,ctime(&t)) ;
+  timeToString (t, dateString, sizeof (dateString)) ;
+  d_printf (1,"Timeout (%d) time now is %ld %s\n",
+            (int) tid, (long) t, dateString) ;
 
   if (timeoutQueue == NULL) 
     {
@@ -1518,7 +1518,8 @@ static void Timeout (TimeoutId tid, void *data)
   static int seeded ;
   static int howMany ;
   static int i ;
-  time_t t = theTime() ;
+  char dateString[30];
+  const time_t t = theTime() ;
 
   if ( !seeded )
     {
@@ -1526,9 +1527,10 @@ static void Timeout (TimeoutId tid, void *data)
       seeded = 1 ;
     }
 
-  d_printf (1,"Timeout (%d) time now is %ld %s",
-           (int) tid, (long) t,ctime(&t)) ;
-  
+  timeToString (t, dateString, sizeof (dateString)) ;
+  d_printf (1,"Timeout (%d) time now is %ld %s\n",
+            (int) tid, (long) t, dateString) ;
+
   if (timeoutQueue != NULL && timeoutQueue->next != NULL)
     d_printf (1,"%s timeout id %d\n",
              (removeTimeout (rm) ? "REMOVED" : "FAILED TO REMOVE"), rm) ;
@@ -1644,7 +1646,7 @@ void setSigHandler (int signum, void (*ptr)(int))
   if (sigHandlers == NULL)
     {
       sigHandlers = xmalloc (sizeof(sigfn) * NSIG) ;
-      sigFlags = xmalloc (sizeof(int) * NSIG) ;
+      sigFlags = xmalloc (sizeof(sig_atomic_t) * NSIG) ;
       for (i = 0 ; i < NSIG ; i++)
         {
           sigHandlers [i] = NULL ;
@@ -1668,31 +1670,18 @@ void setSigHandler (int signum, void (*ptr)(int))
 static void handleSignals (void)
 {
   int i ;
-#if defined(USE_SIGVEC)
-  int mask ;
-#endif
 
   for (i = 1; i < NSIG; i++)
     {
       if (sigFlags[i])
         {
-#if defined(USE_SIGACTION)
+#if defined(HAVE_SIGACTION)
           sigset_t set, oset ;
       
           if (sigemptyset (&set) != 0 || sigaddset (&set, i) != 0)
             die ("sigemptyset or sigaddset failed") ;
           if (sigprocmask (SIG_BLOCK, &set, &oset) != 0)
             die ("sigprocmask failed: %s", strerror(errno)) ;
-#elif defined(USE_SIGVEC)
-# ifndef sigmask
-#  define sigmask(s)    (1 << ((s) - 1))
-# endif
-          int mask ;
-          
-          mask = sigblock (sigmask(i)) ;
-#elif defined(USE_SIGSET)
-          if (sighold (i) != 0)
-            die ("sighold failed: %s", strerror(errno)) ;
 #else
       /* hope for the best */
 #endif
@@ -1704,14 +1693,9 @@ static void handleSignals (void)
               sigHandlers[i] != SIG_DFL)
             (sigHandlers[i])(i) ;
             
-#if defined(USE_SIGACTION)
+#if defined(HAVE_SIGACTION)
           if (sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL) != 0)
             die ("sigprocmask failed: %s", strerror(errno)) ;
-#elif defined(USE_SIGVEC)
-          sigsetmask (mask) ;
-#elif defined(USE_SIGSET)
-          if (sigrelse (i) != 0)
-            die ("sigrelse failed: %s", strerror(errno)) ;
 #else
           /* hope for the best */
 #endif
