@@ -1,11 +1,8 @@
-/*  $Id: innlistener.c,v 1.30 2003/01/19 19:58:22 rra Exp $
+/*  $Id: innlistener.c 8469 2009-05-16 12:43:53Z iulius $
 **
 **  The implementation of the innfeed InnListener class.
 **
 **  Written by James Brister <brister@vix.com>
-**  Copyright 1996 by the Internet Software Consortium
-**
-**  For license terms, see the end of this file.
 */
 
 #include "innfeed.h"
@@ -20,7 +17,7 @@
 #include <time.h>
 
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 #include "article.h"
 #include "buffer.h"
@@ -233,8 +230,7 @@ void shutDown (InnListener l)
       char dateString [30] ;
 
       gHostStats();
-      strlcpy (dateString,ctime (&now),sizeof (dateString)) ;
-      dateString [24] = '\0' ;
+      timeToString (now, dateString, sizeof (dateString)) ;
 
       if (fastExit)
         notice ("ME finishing (quickly) at %s", dateString) ;
@@ -503,9 +499,9 @@ static void newArticleCommand (EndPoint ep, IoStatus i,
           *fileNameEnd = ' ' ;
 
           /* Check the message ID length */
-          if (strlen(msgid) > NNTP_MSGID_MAXLEN) {
+          if (strlen(msgid) > NNTP_MAXLEN_MSGID) {
             warn ("ME message id exceeds limit of %d octets: %s",
-                  NNTP_MSGID_MAXLEN, msgid) ;
+                  NNTP_MAXLEN_MSGID, msgid) ;
             *(msgidEnd+1) = '\0' ;
           }
           *msgidEnd = ' ' ;
@@ -653,7 +649,8 @@ static void giveArticleToPeer (InnListener lis,
             {
               /* XXX need to remember we've gone over the limit and not try
                  to add any more. */
-              warn ("ME internal too many hosts. (max is %d)", lis->hostLen) ;
+              warn ("ME internal too many hosts. (max is %lu)",
+                    (unsigned long) lis->hostLen) ;
               dropArticle (peerName,article) ;
             }
           else
@@ -713,14 +710,11 @@ void openDroppedArticleFile (void)
 {
   pid_t myPid = getpid () ;
   const char *tapeDir = getTapeDirectory() ;
-  size_t len;
 
   if (dropArtFile != NULL)
     free (dropArtFile) ;
 
-  len = pathMax(tapeDir) + 1;
-  dropArtFile = xmalloc(len);
-  snprintf (dropArtFile,len,"%s/innfeed-dropped.%c%06d",
+  xasprintf (&dropArtFile, "%s/innfeed-dropped.%c%06d",
             tapeDir, droppedFileCount + 'A', (int) myPid) ;
 
   if ((droppedFp = fopen (dropArtFile,"w")) == NULL)
@@ -784,20 +778,3 @@ static void listenerCleanup (void)
   free (dropArtFile) ;
   dropArtFile = NULL ;
 }
-
-/*
-**  Copyright 1996 by the Internet Software Consortium
-**
-**  Permission to use, copy, modify, and distribute this software for any
-**  purpose with or without fee is hereby granted, provided that the above
-**  copyright notice and this permission notice appear in all copies.
-**
-**  THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
-**  DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
-**  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL
-**  INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
-**  OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
-**  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
-**  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-**  PERFORMANCE OF THIS SOFTWARE.
-*/
