@@ -1,4 +1,4 @@
-/*  $Id: innconf.c 10304 2018-12-02 14:05:50Z iulius $
+/*  $Id: innconf.c 8460 2009-05-12 18:16:55Z iulius $
 **
 **  Manage the global innconf struct.
 **
@@ -10,7 +10,7 @@
 **  When adding new inn.conf parameters, make sure to add them in all of the
 **  following places:
 **
-**   * The table in this file
+**   * The table in this file.
 **   * include/inn/innconf.h
 **   * doc/pod/inn.conf.pod (and regenerate doc/man/inn.conf.5)
 **   * Add the default value to samples/inn.conf.in
@@ -43,7 +43,6 @@ struct innconf *innconf = NULL;
 enum type {
     TYPE_BOOLEAN,
     TYPE_NUMBER,
-    TYPE_UNUMBER,
     TYPE_STRING,
     TYPE_LIST
 };
@@ -54,8 +53,7 @@ struct config {
     enum type type;
     struct {
         bool boolean;
-        long signed_number;
-        unsigned long unsigned_number;
+        long integer;
         const char *string;
         const struct vector *list;
     } defaults;
@@ -66,18 +64,16 @@ struct config {
 
 #define K(name)         (#name), offsetof(struct innconf, name)
 
-#define BOOL(def)       TYPE_BOOLEAN,   { (def),     0,     0,  NULL,  NULL }
-#define NUMBER(def)     TYPE_NUMBER,    {     0, (def),     0,  NULL,  NULL }
-#define UNUMBER(def)    TYPE_UNUMBER,   {     0,     0, (def),  NULL,  NULL }
-#define STRING(def)     TYPE_STRING,    {     0,     0,     0, (def),  NULL }
-#define LIST(def)       TYPE_LIST,      {     0,     0,     0,  NULL, (def) }
+#define BOOL(def)       TYPE_BOOLEAN, { (def),     0,  NULL,  NULL }
+#define NUMBER(def)     TYPE_NUMBER,  {     0, (def),  NULL,  NULL }
+#define STRING(def)     TYPE_STRING,  {     0,     0, (def),  NULL }
+#define LIST(def)       TYPE_LIST,    {     0,     0,  NULL, (def) }
 
 /* Accessor macros to get a pointer to a value inside a struct. */
-#define CONF_BOOL(conf, offset)     (bool *)          (void *)((char *) (conf) + (offset))
-#define CONF_NUMBER(conf, offset)   (long *)          (void *)((char *) (conf) + (offset))
-#define CONF_UNUMBER(conf, offset)  (unsigned long *) (void *)((char *) (conf) + (offset))
-#define CONF_STRING(conf, offset)   (char **)         (void *)((char *) (conf) + (offset))
-#define CONF_LIST(conf, offset)     (struct vector **)(void *)((char *) (conf) + (offset))
+#define CONF_BOOL(conf, offset)   (bool *)          (void *)((char *) (conf) + (offset))
+#define CONF_LONG(conf, offset)   (long *)          (void *)((char *) (conf) + (offset))
+#define CONF_STRING(conf, offset) (char **)         (void *)((char *) (conf) + (offset))
+#define CONF_LIST(conf, offset)   (struct vector **)(void *)((char *) (conf) + (offset))
 
 /* Special notes:
 
@@ -110,180 +106,166 @@ struct config {
 */
 
 const struct config config_table[] = {
-    { K(domain),                  STRING  (NULL) },
-    { K(enableoverview),          BOOL    (true) },
-    { K(extraoverviewadvertised), LIST    (NULL) },
-    { K(extraoverviewhidden),     LIST    (NULL) },
-    { K(fromhost),                STRING  (NULL) },
-    { K(groupbaseexpiry),         BOOL    (true) },
-    { K(mailcmd),                 STRING  (NULL) },
-    { K(maxforks),                UNUMBER   (10) },
-    { K(mta),                     STRING  (NULL) },
-    { K(nicekids),                NUMBER     (4) },
-    { K(ovmethod),                STRING  (NULL) },
-    { K(pathhost),                STRING  (NULL) },
-    { K(rlimitnofile),            NUMBER    (-1) },
-    { K(server),                  STRING  (NULL) },
-    { K(sourceaddress),           STRING  (NULL) },
-    { K(sourceaddress6),          STRING  (NULL) },
-    { K(syntaxchecks),            LIST    (NULL) },
-    { K(timer),                   UNUMBER  (600) },
+    { K(domain),                STRING  (NULL) },
+    { K(enableoverview),        BOOL    (true) },
+    { K(extraoverviewadvertised), LIST  (NULL) },
+    { K(extraoverviewhidden),   LIST    (NULL) },
+    { K(fromhost),              STRING  (NULL) },
+    { K(groupbaseexpiry),       BOOL    (true) },
+    { K(mailcmd),               STRING  (NULL) },
+    { K(maxforks),              NUMBER  (10) },
+    { K(mta),                   STRING  (NULL) },
+    { K(nicekids),              NUMBER  (4) },
+    { K(ovmethod),              STRING  (NULL) },
+    { K(pathhost),              STRING  (NULL) },
+    { K(rlimitnofile),          NUMBER  (-1) },
+    { K(server),                STRING  (NULL) },
+    { K(sourceaddress),         STRING  (NULL) },
+    { K(sourceaddress6),        STRING  (NULL) },
+    { K(timer),                 NUMBER  (0) },
 
-    { K(runasuser),               STRING  (RUNASUSER) },
-    { K(runasgroup),              STRING  (RUNASGROUP) },
+    { K(runasuser),             STRING  (RUNASUSER) },
+    { K(runasgroup),            STRING  (RUNASGROUP) },
 
-    { K(patharchive),             STRING  (NULL) },
-    { K(patharticles),            STRING  (NULL) },
-    { K(pathbin),                 STRING  (NULL) },
-    { K(pathcontrol),             STRING  (NULL) },
-    { K(pathdb),                  STRING  (NULL) },
-    { K(pathetc),                 STRING  (NULL) },
-    { K(pathfilter),              STRING  (NULL) },
-    { K(pathhttp),                STRING  (NULL) },
-    { K(pathincoming),            STRING  (NULL) },
-    { K(pathlog),                 STRING  (NULL) },
-    { K(pathnews),                STRING  (NULL) },
-    { K(pathoutgoing),            STRING  (NULL) },
-    { K(pathoverview),            STRING  (NULL) },
-    { K(pathrun),                 STRING  (NULL) },
-    { K(pathspool),               STRING  (NULL) },
-    { K(pathtmp),                 STRING  (NULL) },
+    { K(patharchive),           STRING  (NULL) },
+    { K(patharticles),          STRING  (NULL) },
+    { K(pathbin),               STRING  (NULL) },
+    { K(pathcontrol),           STRING  (NULL) },
+    { K(pathdb),                STRING  (NULL) },
+    { K(pathetc),               STRING  (NULL) },
+    { K(pathfilter),            STRING  (NULL) },
+    { K(pathhttp),              STRING  (NULL) },
+    { K(pathincoming),          STRING  (NULL) },
+    { K(pathlog),               STRING  (NULL) },
+    { K(pathnews),              STRING  (NULL) },
+    { K(pathoutgoing),          STRING  (NULL) },
+    { K(pathoverview),          STRING  (NULL) },
+    { K(pathrun),               STRING  (NULL) },
+    { K(pathspool),             STRING  (NULL) },
+    { K(pathtmp),               STRING  (NULL) },
 
     /* The following settings are specific to innd. */
-    { K(artcutoff),               UNUMBER   (10) },
-    { K(badiocount),              UNUMBER    (5) },
-    { K(bindaddress),             STRING  (NULL) },
-    { K(bindaddress6),            STRING  (NULL) },
-    { K(blockbackoff),            UNUMBER  (120) },
-    { K(chaninacttime),           UNUMBER  (600) },
-    { K(chanretrytime),           UNUMBER  (300) },
-    { K(datamovethreshold),       UNUMBER (16384) },
-    { K(dontrejectfiltered),      BOOL   (false) },
-    { K(hiscachesize),            UNUMBER  (256) },
-    { K(htmlstatus),              BOOL    (true) },
-    { K(icdsynccount),            UNUMBER   (10) },
-    { K(ignorenewsgroups),        BOOL   (false) },
-    { K(incominglogfrequency),    UNUMBER  (200) },
-    { K(linecountfuzz),           UNUMBER    (0) },
-    { K(logartsize),              BOOL    (true) },
-    { K(logcancelcomm),           BOOL   (false) },
-    { K(logipaddr),               BOOL    (true) },
-    { K(logsitename),             BOOL    (true) },
-    { K(logstatus),               BOOL    (true) },
-    { K(logtrash),                BOOL    (true) },
-    { K(maxartsize),              UNUMBER (1000000) },
-    { K(maxconnections),          UNUMBER   (50) },
-    { K(mergetogroups),           BOOL   (false) },
-    { K(nntplinklog),             BOOL   (false) },
-    { K(noreader),                BOOL   (false) },
-    { K(pathalias),               STRING  (NULL) },
-    { K(pathcluster),             STRING  (NULL) },
-    { K(pauseretrytime),          UNUMBER  (300) },
-    { K(peertimeout),             UNUMBER (3600) },
-    { K(port),                    UNUMBER  (119) },
-    { K(readerswhenstopped),      BOOL   (false) },
-    { K(refusecybercancels),      BOOL   (false) },
-    { K(remembertrash),           BOOL    (true) },
-    { K(stathist),                STRING  (NULL) },
-    { K(status),                  UNUMBER  (600) },
-    { K(verifycancels),           BOOL   (false) },
-    { K(verifygroups),            BOOL   (false) },
-    { K(wanttrash),               BOOL   (false) },
-    { K(wipcheck),                UNUMBER    (5) },
-    { K(wipexpire),               UNUMBER   (10) },
-    { K(xrefslave),               BOOL   (false) },
+    { K(artcutoff),             NUMBER  (10) },
+    { K(badiocount),            NUMBER  (5) },
+    { K(bindaddress),           STRING  (NULL) },
+    { K(bindaddress6),          STRING  (NULL) },
+    { K(blockbackoff),          NUMBER  (120) },
+    { K(chaninacttime),         NUMBER  (600) },
+    { K(chanretrytime),         NUMBER  (300) },
+    { K(datamovethreshold),     NUMBER  (8192) },
+    { K(dontrejectfiltered),    BOOL    (false) },
+    { K(hiscachesize),          NUMBER  (256) },
+    { K(icdsynccount),          NUMBER  (10) },
+    { K(ignorenewsgroups),      BOOL    (false) },
+    { K(incominglogfrequency),  NUMBER  (200) },
+    { K(linecountfuzz),         NUMBER  (0) },
+    { K(logartsize),            BOOL    (true) },
+    { K(logcancelcomm),         BOOL    (false) },
+    { K(logipaddr),             BOOL    (true) },
+    { K(logsitename),           BOOL    (true) },
+    { K(logstatus),             BOOL    (false) },
+    { K(maxartsize),            NUMBER  (1000000) },
+    { K(maxconnections),        NUMBER  (50) },
+    { K(mergetogroups),         BOOL    (false) },
+    { K(nntplinklog),           BOOL    (false) },
+    { K(noreader),              BOOL    (false) },
+    { K(pathalias),             STRING  (NULL) },
+    { K(pathcluster),           STRING  (NULL) },
+    { K(pauseretrytime),        NUMBER  (300) },
+    { K(peertimeout),           NUMBER  (3600) },
+    { K(port),                  NUMBER  (119) },
+    { K(readerswhenstopped),    BOOL    (false) },
+    { K(refusecybercancels),    BOOL    (false) },
+    { K(remembertrash),         BOOL    (true) },
+    { K(stathist),              STRING  (NULL) },
+    { K(status),                NUMBER  (0) },
+    { K(verifycancels),         BOOL    (false) },
+    { K(verifygroups),          BOOL    (false) },
+    { K(wanttrash),             BOOL    (false) },
+    { K(wipcheck),              NUMBER  (5) },
+    { K(wipexpire),             NUMBER  (10) },
+    { K(xrefslave),             BOOL    (false) },
 
     /* The following settings are specific to nnrpd. */
-    { K(addinjectiondate),        BOOL    (true) },
-    { K(addinjectionpostingaccount), BOOL (false) },
-    { K(addinjectionpostinghost), BOOL    (true) },
-    { K(allownewnews),            BOOL    (true) },
-    { K(backoffauth),             BOOL   (false) },
-    { K(backoffdb),               STRING  (NULL) },
-    { K(backoffk),                UNUMBER    (1) },
-    { K(backoffpostfast),         UNUMBER    (0) },
-    { K(backoffpostslow),         UNUMBER    (1) },
-    { K(backofftrigger),          UNUMBER (10000) },
-    { K(checkincludedtext),       BOOL   (false) },
-    { K(clienttimeout),           UNUMBER (1800) },
-    { K(complaints),              STRING  (NULL) },
-    { K(initialtimeout),          UNUMBER   (10) },
-    { K(keyartlimit),             UNUMBER (100000) },
-    { K(keylimit),                UNUMBER  (512) },
-    { K(keymaxwords),             UNUMBER  (250) },
-    { K(keywords),                BOOL   (false) },
-    { K(localmaxartsize),         UNUMBER (1000000) },
-    { K(maxcmdreadsize),          UNUMBER (BUFSIZ) },
-    { K(msgidcachesize),          UNUMBER (64000) },
-    { K(moderatormailer),         STRING  (NULL) },
-    { K(nfsreader),               BOOL   (false) },
-    { K(nfsreaderdelay),          UNUMBER   (60) },
-    { K(nicenewnews),             UNUMBER    (0) },
-    { K(nicennrpd),               UNUMBER    (0) },
-    { K(nnrpdflags),              STRING    ("") },
-    { K(nnrpdauthsender),         BOOL   (false) },
-    { K(nnrpdloadlimit),          UNUMBER   (16) },
-    { K(nnrpdoverstats),          BOOL    (true) },
-    { K(organization),            STRING  (NULL) },
-    { K(readertrack),             BOOL   (false) },
-    { K(spoolfirst),              BOOL   (false) },
-    { K(strippostcc),             BOOL   (false) },
-#ifdef HAVE_OPENSSL
-    { K(tlscafile),               STRING    ("") },
-    { K(tlscapath),               STRING  (NULL) },
-    { K(tlscertfile),             STRING  (NULL) },
-    { K(tlskeyfile),              STRING  (NULL) },
-    { K(tlsciphers),              STRING  (NULL) },
-    { K(tlsciphers13),            STRING  (NULL) },
-    { K(tlscompression),          BOOL   (false) },
-    { K(tlseccurve),              STRING  (NULL) },
-    { K(tlspreferserverciphers),  BOOL    (true) },
-    { K(tlsprotocols),            LIST    (NULL) },
-#endif /* HAVE_OPENSSL */
+    { K(addnntppostingdate),    BOOL    (true) },
+    { K(addnntppostinghost),    BOOL    (true) },
+    { K(allownewnews),          BOOL    (true) },
+    { K(backoffauth),           BOOL    (false) },
+    { K(backoffdb),             STRING  (NULL) },
+    { K(backoffk),              NUMBER  (1) },
+    { K(backoffpostfast),       NUMBER  (0) },
+    { K(backoffpostslow),       NUMBER  (1) },
+    { K(backofftrigger),        NUMBER  (10000) },
+    { K(checkincludedtext),     BOOL    (false) },
+    { K(clienttimeout),         NUMBER  (1800) },
+    { K(complaints),            STRING  (NULL) },
+    { K(initialtimeout),        NUMBER  (10) },
+    { K(keyartlimit),           NUMBER  (100000) },
+    { K(keylimit),              NUMBER  (512) },
+    { K(keymaxwords),           NUMBER  (250) },
+    { K(keywords),              BOOL    (false) },
+    { K(localmaxartsize),       NUMBER  (1000000) },
+    { K(maxcmdreadsize),        NUMBER  (BUFSIZ) },
+    { K(msgidcachesize),        NUMBER  (16000) },
+    { K(moderatormailer),       STRING  (NULL) },
+    { K(nfsreader),             BOOL    (false) },
+    { K(nfsreaderdelay),        NUMBER  (60) },
+    { K(nicenewnews),           NUMBER  (0) },
+    { K(nicennrpd),             NUMBER  (0) },
+    { K(nnrpdflags),            STRING  ("") },
+    { K(nnrpdauthsender),       BOOL    (false) },
+    { K(nnrpdloadlimit),        NUMBER  (16) },
+    { K(nnrpdoverstats),        BOOL    (false) },
+    { K(organization),          STRING  (NULL) },
+    { K(readertrack),           BOOL    (false) },
+    { K(spoolfirst),            BOOL    (false) },
+    { K(strippostcc),           BOOL    (false) },
+#ifdef HAVE_SSL
+    { K(tlscafile),             STRING  ("") },
+    { K(tlscapath),             STRING  ("") },
+    { K(tlscertfile),           STRING  (NULL) },
+    { K(tlskeyfile),            STRING  (NULL) },
+#endif /* HAVE_SSL */
 
     /* The following settings are used by nnrpd and rnews. */
-    { K(nnrpdposthost),           STRING  (NULL) },
-    { K(nnrpdpostport),           UNUMBER  (119) },
+    { K(nnrpdposthost),         STRING  (NULL) },
+    { K(nnrpdpostport),         NUMBER  (119) },
 
     /* The following settings are specific to the storage subsystem. */
-    { K(articlemmap),             BOOL    (true) },
-    { K(cnfscheckfudgesize),      UNUMBER    (0) },
-    { K(immediatecancel),         BOOL   (false) },
-    { K(keepmmappedthreshold),    UNUMBER (1024) },
-    { K(nfswriter),               BOOL   (false) },
-    { K(nnrpdcheckart),           BOOL    (true) },
-    { K(overcachesize),           UNUMBER  (128) },
-    { K(ovgrouppat),              STRING  (NULL) },
-    { K(storeonxref),             BOOL    (true) },
-    { K(tradindexedmmap),         BOOL    (true) },
-    { K(useoverchan),             BOOL   (false) },
-    { K(wireformat),              BOOL    (true) },
+    { K(articlemmap),           BOOL    (true) },
+    { K(cnfscheckfudgesize),    NUMBER  (0) },
+    { K(immediatecancel),       BOOL    (false) },
+    { K(keepmmappedthreshold),  NUMBER  (1024) },
+    { K(nfswriter),             BOOL    (false) },
+    { K(nnrpdcheckart),         BOOL    (true) },
+    { K(overcachesize),         NUMBER  (64) },
+    { K(ovgrouppat),            STRING  (NULL) },
+    { K(storeonxref),           BOOL    (true) },
+    { K(tradindexedmmap),       BOOL    (true) },
+    { K(useoverchan),           BOOL    (false) },
+    { K(wireformat),            BOOL    (false) },
 
     /* The following settings are specific to the history subsystem. */
-    { K(hismethod),               STRING  (NULL) },
+    { K(hismethod),             STRING  (NULL) },
 
     /* The following settings are specific to rc.news. */
-    { K(docnfsstat),              BOOL   (false) },
-    { K(innflags),                STRING  (NULL) },
-    { K(pgpverify),               BOOL   (false) },
+    { K(docnfsstat),            BOOL    (false) },
+    { K(innflags),              STRING  (NULL) },
+    { K(pgpverify),             BOOL    (false) },
 
     /* The following settings are specific to innwatch. */
-    { K(doinnwatch),              BOOL    (true) },
-    { K(innwatchbatchspace),      UNUMBER (4000) },
-    { K(innwatchlibspace),        UNUMBER (25000) },
-    { K(innwatchloload),          UNUMBER (1000) },
-    { K(innwatchhiload),          UNUMBER (2000) },
-    { K(innwatchpauseload),       UNUMBER (1500) },
-    { K(innwatchsleeptime),       UNUMBER  (600) },
-    { K(innwatchspoolnodes),      UNUMBER  (200) },
-    { K(innwatchspoolspace),      UNUMBER (25000) },
+    { K(doinnwatch),            BOOL    (true) },
+    { K(innwatchbatchspace),    NUMBER  (4000) },
+    { K(innwatchlibspace),      NUMBER  (25000) },
+    { K(innwatchloload),        NUMBER  (1000) },
+    { K(innwatchhiload),        NUMBER  (2000) },
+    { K(innwatchpauseload),     NUMBER  (1500) },
+    { K(innwatchsleeptime),     NUMBER  (600) },
+    { K(innwatchspoolnodes),    NUMBER  (200) },
+    { K(innwatchspoolspace),    NUMBER  (25000) },
 
-    /* The following settings are specific to scanlogs.
-     * Keep a low value by default as it has a privacy impact; three days
-     * should be enough to diagnose the most common cases of malfunction
-     * and abuse.  Where more is needed, it is easy for the news admin
-     * to increase it. */
-    { K(logcycles),               UNUMBER    (3) },
+    /* The following settings are specific to scanlogs. */
+    { K(logcycles),             NUMBER  (3) },
 };
 
 
@@ -332,9 +314,9 @@ innconf_set_defaults(void)
 
     /* Some parameters have defaults that depend on other parameters. */
     if (innconf->fromhost == NULL)
-        innconf->fromhost = inn_getfqdn(innconf->domain);
+        innconf->fromhost = xstrdup(GetFQDN(innconf->domain));
     if (innconf->pathhost == NULL)
-        innconf->pathhost = inn_getfqdn(innconf->domain);
+        innconf->pathhost = xstrdup(GetFQDN(innconf->domain));
     if (innconf->pathtmp == NULL)
         innconf->pathtmp = xstrdup(INN_PATH_TMP);
 
@@ -380,14 +362,12 @@ innconf_set_defaults(void)
     if (innconf->extraoverviewhidden == NULL)
         innconf->extraoverviewhidden = vector_new();
 
-    /* Defaults used only if TLS/SSL is supported. */
-#ifdef HAVE_OPENSSL
-    if (innconf->tlscapath == NULL)
-        innconf->tlscapath = xstrdup(innconf->pathetc);
+    /* Defaults used only if TLS (SSL) is supported. */
+#ifdef HAVE_SSL
     if (innconf->tlscertfile == NULL)
-        innconf->tlscertfile = concatpath(innconf->pathetc, "cert.pem");
+        innconf->tlscertfile = concatpath(innconf->pathnews, "lib/cert.pem");
     if (innconf->tlskeyfile == NULL)
-        innconf->tlskeyfile = concatpath(innconf->pathetc, "key.pem");
+        innconf->tlskeyfile = concatpath(innconf->pathnews, "lib/key.pem");
 #endif
 }
 
@@ -403,8 +383,7 @@ innconf_parse(struct config_group *group)
 {
     unsigned int i, j;
     bool *bool_ptr;
-    long *signed_number_ptr;
-    unsigned long *unsigned_number_ptr;
+    long *long_ptr;
     const char *char_ptr;
     char **string;
     const struct vector *vector_ptr;
@@ -412,7 +391,6 @@ innconf_parse(struct config_group *group)
     struct innconf *config;
 
     config = xmalloc(sizeof(struct innconf));
-    memset(config, 0, sizeof(struct innconf));
     for (i = 0; i < ARRAY_SIZE(config_table); i++)
         switch (config_table[i].type) {
         case TYPE_BOOLEAN:
@@ -421,14 +399,9 @@ innconf_parse(struct config_group *group)
                 *bool_ptr = config_table[i].defaults.boolean;
             break;
         case TYPE_NUMBER:
-            signed_number_ptr = CONF_NUMBER(config, config_table[i].location);
-            if (!config_param_signed_number(group, config_table[i].name, signed_number_ptr))
-                *signed_number_ptr = config_table[i].defaults.signed_number;
-            break;
-        case TYPE_UNUMBER:
-            unsigned_number_ptr = CONF_UNUMBER(config, config_table[i].location);
-            if (!config_param_unsigned_number(group, config_table[i].name, unsigned_number_ptr))
-                *unsigned_number_ptr = config_table[i].defaults.unsigned_number;
+            long_ptr = CONF_LONG(config, config_table[i].location);
+            if (!config_param_integer(group, config_table[i].name, long_ptr))
+                *long_ptr = config_table[i].defaults.integer;
             break;
         case TYPE_STRING:
             if (!config_param_string(group, config_table[i].name, &char_ptr))
@@ -472,14 +445,12 @@ static bool
 innconf_validate(struct config_group *group)
 {
     bool okay = true;
-    char *fqdn;
+    long threshold;
 
-    fqdn = inn_getfqdn(innconf->domain);
-    if (fqdn == NULL) {
+    if (GetFQDN(innconf->domain) == NULL) {
         warn("hostname does not resolve or domain not set in inn.conf");
         okay = false;
     }
-    free(fqdn);
     if (innconf->mta == NULL) {
         warn("must set mta in inn.conf");
         okay = false;
@@ -497,42 +468,12 @@ innconf_validate(struct config_group *group)
         okay = false;
     }
 
-    if (innconf->datamovethreshold > 1024 * 1024) {
+    threshold = innconf->datamovethreshold;
+    if (threshold <= 0 || threshold > 1024 * 1024) {
         config_error_param(group, "datamovethreshold",
                            "maximum value for datamovethreshold is 1MB");
         innconf->datamovethreshold = 1024 * 1024;
     }
-
-    if (innconf->keywords) {
-        bool found = false;
-        unsigned int i;
-        if (innconf->extraoverviewadvertised->strings != NULL) {
-            for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
-                if (innconf->extraoverviewadvertised->strings[i] != NULL &&
-                    (strcasecmp(innconf->extraoverviewadvertised->strings[i], "Keywords") == 0)) {
-                    found = true;
-                    break;
-                }
-            }
-        }
-        if (innconf->extraoverviewhidden->strings != NULL) {
-            for (i = 0; i < innconf->extraoverviewhidden->count; i++) {
-                if (innconf->extraoverviewhidden->strings[i] != NULL &&
-                    (strcasecmp(innconf->extraoverviewhidden->strings[i], "Keywords") == 0)) {
-                    found = true;
-                    break;
-                }
-            }
-        }
-
-        if (!found) {
-            config_error_param(group, "keywords",
-                               "keyword generation is useless if the Keywords:"
-                               " header is not stored in the overview");
-            innconf->keywords = false;
-        }
-    }
-
     return okay;
 }
 
@@ -682,7 +623,7 @@ print_boolean(FILE *file, const char *key, bool value,
     case INNCONF_QUOTE_SHELL:
         upper = xstrdup(key);
         for (p = upper; *p != '\0'; p++)
-            *p = toupper((unsigned char) *p);
+            *p = toupper(*p);
         fprintf(file, "%s=%s; export %s;\n", upper, value ? "true" : "false",
                 upper);
         free(upper);
@@ -698,11 +639,11 @@ print_boolean(FILE *file, const char *key, bool value,
 
 
 /*
-**  Print a single signed integer value with appropriate quoting.
+**  Print a single integer value with appropriate quoting.
 */
 static void
-print_signed_number(FILE *file, const char *key, long value,
-                    enum innconf_quoting quoting)
+print_number(FILE *file, const char *key, long value,
+             enum innconf_quoting quoting)
 {
     char *upper, *p;
 
@@ -713,7 +654,7 @@ print_signed_number(FILE *file, const char *key, long value,
     case INNCONF_QUOTE_SHELL:
         upper = xstrdup(key);
         for (p = upper; *p != '\0'; p++)
-            *p = toupper((unsigned char) *p);
+            *p = toupper(*p);
         fprintf(file, "%s=%ld; export %s;\n", upper, value, upper);
         free(upper);
         break;
@@ -727,36 +668,6 @@ print_signed_number(FILE *file, const char *key, long value,
 }
 
 
-/*
-**  Print a single unsigned integer value with appropriate quoting.
-*/
-static void
-print_unsigned_number(FILE *file, const char *key, unsigned long value,
-                      enum innconf_quoting quoting)
-{
-    char *upper, *p;
-
-    switch (quoting) {
-        case INNCONF_QUOTE_NONE:
-            fprintf(file, "%lu\n", value);
-            break;
-        case INNCONF_QUOTE_SHELL:
-            upper = xstrdup(key);
-            for (p = upper; *p != '\0'; p++)
-                *p = toupper((unsigned char) *p);
-            fprintf(file, "%s=%lu; export %s;\n", upper, value, upper);
-            free(upper);
-            break;
-        case INNCONF_QUOTE_PERL:
-            fprintf(file, "$%s = %lu;\n", key, value);
-            break;
-        case INNCONF_QUOTE_TCL:
-            fprintf(file, "set inn_%s %lu\n", key, value);
-            break;
-    }
-}
-
-
 /*
 **  Print a single string value with appropriate quoting.
 */
@@ -770,20 +681,12 @@ print_string(FILE *file, const char *key, const char *value,
 
     switch (quoting) {
     case INNCONF_QUOTE_NONE:
-        /* Do not output NULL values.  They are not empty strings. */
-        if (value == NULL) {
-            return;
-        }
-        fprintf(file, "%s\n", value);
+        fprintf(file, "%s\n", value != NULL ? value : "");
         break;
     case INNCONF_QUOTE_SHELL:
-        /* Do not output NULL values.  They are not empty strings. */
-        if (value == NULL) {
-            return;
-        }
         upper = xstrdup(key);
         for (p = upper; *p != '\0'; p++)
-            *p = toupper((unsigned char) *p);
+            *p = toupper(*p);
         fprintf(file, "%s='", upper);
         for (letter = value; letter != NULL && *letter != '\0'; letter++) {
             if (*letter == '\'')
@@ -797,10 +700,6 @@ print_string(FILE *file, const char *key, const char *value,
         free(upper);
         break;
     case INNCONF_QUOTE_PERL:
-        if (value == NULL) {
-            fprintf(file, "$%s = undef;\n", key);
-            return;
-        }
         fprintf(file, "$%s = '", key);
         for (letter = value; letter != NULL && *letter != '\0'; letter++) {
             if (*letter == '\'' || *letter == '\\')
@@ -810,10 +709,6 @@ print_string(FILE *file, const char *key, const char *value,
         fputs("';\n", file);
         break;
     case INNCONF_QUOTE_TCL:
-        /* Do not output NULL values.  They are not empty strings. */
-        if (value == NULL) {
-            return;
-        }
         fprintf(file, "set inn_%s \"", key);
         for (letter = value; letter != NULL && *letter != '\0'; letter++) {
             if (strchr(tcl_unsafe, *letter) != NULL)
@@ -840,10 +735,6 @@ print_list(FILE *file, const char *key, const struct vector *value,
 
     switch (quoting) {
     case INNCONF_QUOTE_NONE:
-        /* Do not output NULL values.  They are not empty lists. */
-        if (value == NULL || value->strings == NULL) {
-            return;
-        }
         fprintf(file, "[ ");
         if (value != NULL && value->strings != NULL) {
             for (i = 0; i < value->count; i++) {
@@ -855,13 +746,9 @@ print_list(FILE *file, const char *key, const struct vector *value,
         fprintf(file, "]\n");
         break;
     case INNCONF_QUOTE_SHELL:
-        /* Do not output NULL values.  They are not empty lists. */
-        if (value == NULL || value->strings == NULL) {
-            return;
-        }
         upper = xstrdup(key);
         for (p = upper; *p != '\0'; p++)
-            *p = toupper((unsigned char) *p);
+            *p = toupper(*p);
         /* For interoperability reasons, we return a space-separated string
          * representing an array (pure Bourne shell does not have the notion
          * of an array for instance). */
@@ -891,11 +778,6 @@ print_list(FILE *file, const char *key, const struct vector *value,
         free(upper);
         break;
     case INNCONF_QUOTE_PERL:
-        /* Consider that an empty list is undefined. */
-        if (value == NULL || value->strings == NULL) {
-            fprintf(file, "@%s = undef;\n", key);
-            return;
-        }   
         fprintf(file, "@%s = ( ", key);
         if (value != NULL && value->strings != NULL) {
             for (i = 0; i < value->count; i++) {
@@ -916,10 +798,6 @@ print_list(FILE *file, const char *key, const struct vector *value,
         fprintf(file, ");\n");
         break;
     case INNCONF_QUOTE_TCL:
-        /* Do not output NULL values.  They are not empty lists. */
-        if (value == NULL || value->strings == NULL) {
-            return;
-        }   
         fprintf(file, "set inn_%s { ", key);
         if (value != NULL && value->strings != NULL) {
             for (i = 0; i < value->count; i++) {
@@ -948,8 +826,7 @@ static void
 print_parameter(FILE *file, size_t i, enum innconf_quoting quoting)
 {
     bool bool_val;
-    long signed_number_val;
-    unsigned long unsigned_number_val;
+    long long_val;
     const char *string_val;
     const struct vector *list_val;
 
@@ -959,12 +836,8 @@ print_parameter(FILE *file, size_t i, enum innconf_quoting quoting)
         print_boolean(file, config_table[i].name, bool_val, quoting);
         break;
     case TYPE_NUMBER:
-        signed_number_val = *CONF_NUMBER(innconf, config_table[i].location);
-        print_signed_number(file, config_table[i].name, signed_number_val, quoting);
-        break;
-    case TYPE_UNUMBER:
-        unsigned_number_val = *CONF_UNUMBER(innconf, config_table[i].location);
-        print_unsigned_number(file, config_table[i].name, unsigned_number_val, quoting);
+        long_val = *CONF_LONG(innconf, config_table[i].location);
+        print_number(file, config_table[i].name, long_val, quoting);
         break;
     case TYPE_STRING:
         string_val = *CONF_STRING(innconf, config_table[i].location);
@@ -1024,8 +897,7 @@ innconf_compare(struct innconf *conf1, struct innconf *conf2)
 {
     unsigned int i, j;
     bool bool1, bool2;
-    long signed_number1, signed_number2;
-    unsigned long unsigned_number1, unsigned_number2;
+    long long1, long2;
     const char *string1, *string2;
     const struct vector *list1, *list2;
     bool okay = true;
@@ -1042,20 +914,11 @@ innconf_compare(struct innconf *conf1, struct innconf *conf2)
             }
             break;
         case TYPE_NUMBER:
-            signed_number1 = *CONF_NUMBER(conf1, config_table[i].location);
-            signed_number2 = *CONF_NUMBER(conf2, config_table[i].location);
-            if (signed_number1 != signed_number2) {
+            long1 = *CONF_LONG(conf1, config_table[i].location);
+            long2 = *CONF_LONG(conf2, config_table[i].location);
+            if (long1 != long2) {
                 warn("integer variable %s differs: %ld != %ld",
-                     config_table[i].name, signed_number1, signed_number2);
-                okay = false;
-            }
-            break;
-        case TYPE_UNUMBER:
-            unsigned_number1 = *CONF_UNUMBER(conf1, config_table[i].location);
-            unsigned_number2 = *CONF_UNUMBER(conf2, config_table[i].location);
-            if (unsigned_number1 != unsigned_number2) {
-                warn("integer variable %s differs: %lu  != %lu",
-                     config_table[i].name, unsigned_number1, unsigned_number2);
+                     config_table[i].name, long1, long2);
                 okay = false;
             }
             break;
