@@ -1,4 +1,4 @@
-/*  $Id: uwildmat.c,v 1.2 2003/01/14 06:03:23 rra Exp $
+/*  $Id: uwildmat.c 8378 2009-03-09 19:45:13Z iulius $
 **
 **  wildmat pattern matching with Unicode UTF-8 extensions.
 **
@@ -63,7 +63,7 @@
 
 #include "config.h"
 #include "clibrary.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 #define ABORT -1
 
@@ -147,7 +147,8 @@ match_class(uint32_t text, const unsigned char *start,
 {
     bool reversed, allowrange;
     const unsigned char *p = start;
-    uint32_t first, last;
+    uint32_t first = 0;
+    uint32_t last;
 
     /* Check for an inverted character class (starting with ^).  If the
        character matches the character class, we return !reversed; that way,
@@ -196,7 +197,7 @@ match_pattern(const unsigned char *text, const unsigned char *start,
     const unsigned char *q, *endclass;
     const unsigned char *p = start;
     bool ismeta;
-    int matched;
+    int matched, width;
     uint32_t c;
 
     for (; p <= end; p++) {
@@ -240,11 +241,15 @@ match_pattern(const unsigned char *text, const unsigned char *start,
                function call overhead. */
             ismeta = (*p == '[' || *p == '?' || *p == '\\');
             while (*text) {
+                width = ISUTF8(*text) ? utf8_length(text, NULL) : 1;
                 if (ismeta) {
-                    matched = match_pattern(text++, p, end);
+                    matched = match_pattern(text, p, end);
+                    text += width;
                 } else {
-                    while (*text && *text != *p)
-                        text++;
+                    while (*text && *text != *p) {
+                        text += width;
+                        width = ISUTF8(*text) ? utf8_length(text, NULL) : 1;
+                    }
                     if (!*text)
                         return ABORT;
                     matched = match_pattern(++text, p + 1, end);
