@@ -1,4 +1,4 @@
-/* $Id: xmalloc.c,v 1.13 2002/03/31 22:35:47 rra Exp $
+/* $Id: xmalloc.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  malloc routines with failure handling.
 **
@@ -7,6 +7,7 @@
 **       extern xmalloc_handler_t memory_error;
 **       extern const char *string;
 **       char *buffer;
+**       va_list args;
 **
 **       xmalloc_error_handler = memory_error;
 **       buffer = xmalloc(1024);
@@ -17,6 +18,10 @@
 **       buffer = xstrdup(string);
 **       free(buffer);
 **       buffer = xstrndup(string, 25);
+**       free(buffer);
+**       xasprintf(&buffer, "%s", "some string");
+**       free(buffer);
+**       xvasprintf(&buffer, "%s", args);
 **
 **  xmalloc, xcalloc, xrealloc, and xstrdup behave exactly like their C
 **  library counterparts without the leading x except that they will never
@@ -33,6 +38,11 @@
 **  characters.  It allocates an additional byte over its second argument and
 **  always nul-terminates the string.
 **
+**  xasprintf and xvasprintf behave just like their GNU glibc library
+**  implementations except that they do the same checking as described above.
+**  xasprintf will only be able to provide accurate file and line information
+**  on systems that support variadic macros.
+**
 **  The default error handler, if none is set by the caller, prints an error
 **  message to stderr and exits with exit status 1.  An error handler must
 **  take a const char * (function name), size_t (bytes allocated), const
@@ -49,9 +59,10 @@
 
 #include "config.h"
 #include "clibrary.h"
+#include <errno.h>
 
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 /* The default error handler. */
 void
@@ -62,7 +73,7 @@ xmalloc_fail(const char *function, size_t size, const char *file, int line)
 }
 
 /* Assign to this variable to choose a handler other than the default. */
-xmalloc_handler_t xmalloc_error_handler = xmalloc_fail;
+xmalloc_handler_type xmalloc_error_handler = xmalloc_fail;
 
 void *
 x_malloc(size_t size, const char *file, int line)
@@ -137,3 +148,71 @@ x_strndup(const char *s, size_t size, const char *file, int line)
     p[size] = '\0';
     return p;
 }
+
+int
+x_vasprintf(char **strp, const char *fmt, va_list args, const char *file,
+            int line)
+{
+    va_list args_copy;
+    int status;
+
+    va_copy(args_copy, args);
+    status = vasprintf(strp, fmt, args_copy);
+    va_end(args_copy);
+    while (status < 0 && errno == ENOMEM) {
+        va_copy(args_copy, args);
+        status = vsnprintf(NULL, 0, fmt, args_copy);
+        va_end(args_copy);
+        (*xmalloc_error_handler)("vasprintf", status + 1, file, line);
+        va_copy(args_copy, args);
+        status = vasprintf(strp, fmt, args_copy);
+        va_end(args_copy);
+    }
+    return status;
+}
+
+#if INN_HAVE_C99_VAMACROS || INN_HAVE_GNU_VAMACROS
+int
+x_asprintf(char **strp, const char *file, int line, const char *fmt, ...)
+{
+    va_list args, args_copy;
+    int status;
+
+    va_start(args, fmt);
+    va_copy(args_copy, args);
+    status = vasprintf(strp, fmt, args_copy);
+    va_end(args_copy);
+    while (status < 0 && errno == ENOMEM) {
+        va_copy(args_copy, args);
+        status = vsnprintf(NULL, 0, fmt, args_copy);
+        va_end(args_copy);
+        (*xmalloc_error_handler)("asprintf", status + 1, file, line);
+        va_copy(args_copy, args);
+        status = vasprintf(strp, fmt, args_copy);
+        va_end(args_copy);
+    }
+    return status;
+}
+#else /* !(INN_HAVE_C99_VAMACROS || INN_HAVE_GNU_VAMACROS) */
+int
+x_asprintf(char **strp, const char *fmt, ...)
+{
+    va_list args, args_copy;
+    int status;
+
+    va_start(args, fmt);
+    va_copy(args_copy, args);
+    status = vasprintf(strp, fmt, args_copy);
+    va_end(args_copy);
+    while (status < 0 && errno == ENOMEM) {
+        va_copy(args_copy, args);
+        status = vsnprintf(NULL, 0, fmt, args_copy);
+        va_end(args_copy);
+        (*xmalloc_error_handler)("asprintf", status + 1, __FILE__, __LINE__);
+        va_copy(args_copy, args);
+        status = vasprintf(strp, fmt, args_copy);
+        va_end(args_copy);
+    }
+    return status;
+}
+#endif /* !(INN_HAVE_C99_VAMACROS || INN_HAVE_GNU_VAMACROS) */
