@@ -1,4 +1,4 @@
-/*  $Id: article.c 8004 2008-09-06 08:49:55Z iulius $
+/*  $Id: article.c 8209 2008-12-06 08:34:35Z iulius $
 **
 **  Article-related routines.
 */
@@ -10,12 +10,13 @@
 # include <limits.h>
 #endif
 #include <sys/uio.h>
+#include <ctype.h>
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/wire.h"
 #include "nnrpd.h"
-#include "ov.h"
+#include "inn/ov.h"
 #include "tls.h"
 #include "cache.h"
 
@@ -44,61 +45,78 @@ static char		ARTnoartingroup[] = NNTP_NOARTINGRP;
 static char		ARTnocurrart[] = NNTP_NOCURRART;
 static ARTHANDLE        *ARThandle = NULL;
 static SENDDATA		SENDbody = {
-    STbody,	NNTP_BODY_FOLLOWS_VAL,		"body"
+    STbody,	NNTP_OK_BODY,		"body"
 };
 static SENDDATA		SENDarticle = {
-    STarticle,	NNTP_ARTICLE_FOLLOWS_VAL,	"article"
+    STarticle,	NNTP_OK_ARTICLE,	"article"
 };
 static SENDDATA		SENDstat = {
-    STstat,	NNTP_NOTHING_FOLLOWS_VAL,	"status"
+    STstat,     NNTP_OK_STAT,           "status"
 };
 static SENDDATA		SENDhead = {
-    SThead,	NNTP_HEAD_FOLLOWS_VAL,		"head"
+    SThead,	NNTP_OK_HEAD,		"head"
 };
 
-
 static struct iovec	iov[IOV_MAX > 1024 ? 1024 : IOV_MAX];
 static int		queued_iov = 0;
 
-static void PushIOvHelper(struct iovec* vec, int* countp) {
-    int result;
-    TMRstart(TMR_NNTPWRITE);
-#ifdef HAVE_SSL
-    if (tls_conn) {
-Again:
-        result = SSL_writev(tls_conn, vec, *countp);
-        switch (SSL_get_error(tls_conn, result)) {
-        case SSL_ERROR_NONE:
-        case SSL_ERROR_SYSCALL:
-            break;
-        case SSL_ERROR_WANT_WRITE:
-            goto Again;
-            break;
-        case SSL_ERROR_SSL:
-            SSL_shutdown(tls_conn);
-            tls_conn = NULL;
-            errno = ECONNRESET;
-            break;
-        case SSL_ERROR_ZERO_RETURN:
-            break;
-        }
+static void
+PushIOvHelper(struct iovec* vec, int* countp)
+{
+    int result = 0;
+
+#ifdef HAVE_SASL
+    if (sasl_conn && sasl_ssf) {
+	int i;
+
+	for (i = 0; i < *countp; i++) {
+	    write_buffer(vec[i].iov_base, vec[i].iov_len);
+	}
     } else {
-        result = xwritev(STDOUT_FILENO, vec, *countp);
+#endif /* HAVE_SASL */
+
+        TMRstart(TMR_NNTPWRITE);
+
+#ifdef HAVE_SSL
+	if (tls_conn) {
+Again:
+	    result = SSL_writev(tls_conn, vec, *countp);
+	    switch (SSL_get_error(tls_conn, result)) {
+	    case SSL_ERROR_NONE:
+	    case SSL_ERROR_SYSCALL:
+		break;
+	    case SSL_ERROR_WANT_WRITE:
+		goto Again;
+		break;
+	    case SSL_ERROR_SSL:
+		SSL_shutdown(tls_conn);
+		tls_conn = NULL;
+		errno = ECONNRESET;
+		break;
+	    case SSL_ERROR_ZERO_RETURN:
+		break;
+	    }
+	} else
+#endif /* HAVE_SSL */
+	    result = xwritev(STDOUT_FILENO, vec, *countp);
+
+        TMRstop(TMR_NNTPWRITE);
+
+#ifdef HAVE_SASL
     }
-#else
-    result = xwritev(STDOUT_FILENO, vec, *countp);
 #endif
-    TMRstop(TMR_NNTPWRITE);
+
     if (result == -1) {
-	/* we can't recover, since we can't resynchronise with our
-	 * peer */
+	/* We can't recover, since we can't resynchronise with our
+	 * peer. */
 	ExitWithStats(1, true);
     }
     *countp = 0;
 }
 
 static void
-PushIOvRateLimited(void) {
+PushIOvRateLimited(void)
+{
     double              start, end, elapsed, target;
     struct iovec        newiov[IOV_MAX > 1024 ? 1024 : IOV_MAX];
     int                 newiov_len;
@@ -136,7 +154,7 @@ PushIOvRateLimited(void) {
             waittime.tv_usec = (target - elapsed) * 1e6;
             start = TMRnow_double();
 	    if (select(0, NULL, NULL, NULL, &waittime) != 0)
-                syswarn("%s: select in PushIOvRateLimit failed", ClientHost);
+                syswarn("%s: select in PushIOvRateLimit failed", Client.host);
             end = TMRnow_double();
             IDLEtime += end - start;
 	}
@@ -146,7 +164,8 @@ PushIOvRateLimited(void) {
 }
 
 static void
-PushIOv(void) {
+PushIOv(void)
+{
     TMRstart(TMR_NNTPWRITE);
     fflush(stdout);
     TMRstop(TMR_NNTPWRITE);
@@ -157,7 +176,8 @@ PushIOv(void) {
 }
 
 static void
-SendIOv(const char *p, int len) {
+SendIOv(const char *p, int len)
+{
     char                *q;
 
     if (queued_iov) {
@@ -177,54 +197,15 @@ static char		*_IO_buffer_ = NULL;
 static int		highwater = 0;
 
 static void
-PushIOb(void) {
-    TMRstart(TMR_NNTPWRITE);
-    fflush(stdout);
-#ifdef HAVE_SSL
-    if (tls_conn) {
-        int r;
-Again:
-        r = SSL_write(tls_conn, _IO_buffer_, highwater);
-        switch (SSL_get_error(tls_conn, r)) {
-        case SSL_ERROR_NONE:
-        case SSL_ERROR_SYSCALL:
-            break;
-        case SSL_ERROR_WANT_WRITE:
-            goto Again;
-            break;
-        case SSL_ERROR_SSL:
-            SSL_shutdown(tls_conn);
-            tls_conn = NULL;
-            errno = ECONNRESET;
-            break;
-        case SSL_ERROR_ZERO_RETURN:
-            break;
-        }
-        if (r != highwater) {
-            TMRstop(TMR_NNTPWRITE);
-            highwater = 0;
-            return;
-        }
-    } else {
-      if (xwrite(STDOUT_FILENO, _IO_buffer_, highwater) != highwater) {
-	TMRstop(TMR_NNTPWRITE);
-	highwater = 0;
-	return;
-      }
-    }
-#else
-    if (xwrite(STDOUT_FILENO, _IO_buffer_, highwater) != highwater) {
-	TMRstop(TMR_NNTPWRITE);
-        highwater = 0;
-        return;
-    }
-#endif
-    TMRstop(TMR_NNTPWRITE);
+PushIOb(void)
+{
+    write_buffer(_IO_buffer_, highwater);
     highwater = 0;
 }
 
 static void
-SendIOb(const char *p, int len) {
+SendIOb(const char *p, int len)
+{
     int tocopy;
     
     if (_IO_buffer_ == NULL)
@@ -245,7 +226,8 @@ SendIOb(const char *p, int len) {
 /*
 **  If we have an article open, close it.
 */
-void ARTclose(void)
+void
+ARTclose(void)
 {
     if (ARThandle) {
 	SMfreearticle(ARThandle);
@@ -253,7 +235,8 @@ void ARTclose(void)
     }
 }
 
-bool ARTinstorebytoken(TOKEN token)
+bool
+ARTinstorebytoken(TOKEN token)
 {
     ARTHANDLE *art;
     struct timeval	stv, etv;
@@ -277,7 +260,8 @@ bool ARTinstorebytoken(TOKEN token)
 /*
 **  If the article name is valid, open it and stuff in the ID.
 */
-static bool ARTopen(ARTNUM artnum)
+static bool
+ARTopen(ARTNUM artnum)
 {
     static ARTNUM	save_artnum;
     TOKEN		token;
@@ -305,7 +289,7 @@ static bool ARTopen(ARTNUM artnum)
 
 
 /*
-**  Open the article for a given Message-ID.
+**  Open the article for a given message-ID.
 */
 static bool
 ARTopenbyid(char *msg_id, ARTNUM *ap, bool final)
@@ -318,14 +302,14 @@ ARTopenbyid(char *msg_id, ARTNUM *ap, bool final)
 	if (History == NULL) {
 	    time_t statinterval;
 
-	    /* Do lazy opens of the history file - lots of clients
-	     * will never ask for anything by message id, so put off
-	     * doing the work until we have to */
+	    /* Do lazy opens of the history file:  lots of clients
+	     * will never ask for anything by message-ID, so put off
+	     * doing the work until we have to. */
 	    History = HISopen(HISTORY, innconf->hismethod, HIS_RDONLY);
 	    if (!History) {
-		syslog(L_NOTICE, "cant initialize history");
-		Reply("%d NNTP server unavailable. Try later.\r\n",
-		      NNTP_TEMPERR_VAL);
+		syslog(L_NOTICE, "can't initialize history");
+		Reply("%d NNTP server unavailable; try later\r\n",
+		      NNTP_FAIL_TERMINATING);
 		ExitWithStats(1, true);
 	    }
 	    statinterval = 30;
@@ -349,7 +333,8 @@ ARTopenbyid(char *msg_id, ARTNUM *ap, bool final)
 /*
 **  Send a (part of) a file to stdout, doing newline and dot conversion.
 */
-static void ARTsendmmap(SENDTYPE what)
+static void
+ARTsendmmap(SENDTYPE what)
 {
     const char		*p, *q, *r;
     const char		*s, *path, *xref, *endofpath;
@@ -386,7 +371,7 @@ static void ARTsendmmap(SENDTYPE what)
 	}
     }
 
-    /* q points to the start of the article buffer, p to the end of it */
+    /* q points to the start of the article buffer, p to the end of it. */
     if (VirtualPathlen > 0 && (what != STbody)) {
         path = wire_findheader(ARThandle->data, ARThandle->len, "Path");
         if (path == NULL) {
@@ -420,7 +405,7 @@ static void ARTsendmmap(SENDTYPE what)
 	    ARTget++;
 	    return;
 	}
-	/* r points to the first space in the Xref header */
+	/* r points to the first space in the Xref: header. */
 	for (s = path, lastchar = '\0';
 	    s + VirtualPathlen + 1 < endofpath;
 	    lastchar = *s++) {
@@ -482,11 +467,12 @@ static void ARTsendmmap(SENDTYPE what)
 /*
 **  Return the header from the specified file, or NULL if not found.
 */
-char *GetHeader(const char *header)
+char *
+GetHeader(const char *header)
 {
     const char		*p, *q, *r, *s, *t;
     char		*w, prevchar;
-    /* Bogus value here to make sure that it isn't initialized to \n */
+    /* Bogus value here to make sure that it isn't initialized to \n. */
     char		lastchar = ' ';
     const char		*limit;
     const char		*cmplimit;
@@ -511,7 +497,7 @@ char *GetHeader(const char *header)
 		for (; (p < limit) && isspace((int)*p) ; p++);
 		for (q = p; q < limit; q++) 
 		    if ((*q == '\r') || (*q == '\n')) {
-			/* Check for continuation header lines */
+			/* Check for continuation header lines. */
 			t = q + 1;
 			if (t < limit) {
 			    if ((*q == '\r' && *t == '\n')) {
@@ -599,15 +585,32 @@ char *GetHeader(const char *header)
 /*
 **  Fetch part or all of an article and send it to the client.
 */
-void CMDfetch(int ac, char *av[])
+void
+CMDfetch(int ac, char *av[])
 {
     char		buff[SMBUF];
     SENDDATA		*what;
-    bool		ok;
+    bool                mid, ok;
     ARTNUM		art;
     char		*msgid;
     ARTNUM		tart;
-    bool final = false;
+    bool                final = false;
+
+    mid = (ac > 1 && IsValidMessageID(av[1]));
+
+    /* Check the syntax of the arguments first. */
+    if (ac > 1 && !IsValidArticleNumber(av[1])) {
+        /* It is better to check for a number before a message-ID because
+         * '<' could have been forgotten and the error message should then
+         * report a syntax error in the message-ID. */
+        if (CTYPE(isdigit, av[1][0])) {
+            Reply("%d Syntax error in article number\r\n", NNTP_ERR_SYNTAX);
+            return;
+        } else if (!mid) {
+            Reply("%d Syntax error in message-ID\r\n", NNTP_ERR_SYNTAX);
+            return;
+        }
+    }
 
     /* Find what to send; get permissions. */
     ok = PERMcanread;
@@ -625,30 +628,33 @@ void CMDfetch(int ac, char *av[])
 	break;
     case 'h': case 'H':
 	what = &SENDhead;
-	/* Poster might do a "head" command to verify the article. */
+	/* Poster might do a HEAD command to verify the article. */
 	ok = PERMcanread || PERMcanpost;
 	break;
     }
 
-    if (!ok) {
-	Reply("%s\r\n", NOACCESS);
+    /* Check authorizations.  If an article number is requested
+     * (not a message-ID), we check whether the group is still readable. */
+    if (!ok || (!mid && PERMgroupmadeinvalid)) {
+	Reply("%d Read access denied\r\n",
+              PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	return;
     }
 
-    /* Requesting by Message-ID? */
-    if (ac == 2 && av[1][0] == '<') {
+    /* Requesting by message-ID? */
+    if (mid) {
 	if (!ARTopenbyid(av[1], &art, final)) {
-	    Reply("%d No such article\r\n", NNTP_DONTHAVEIT_VAL);
+	    Reply("%d No such article\r\n", NNTP_FAIL_NOTFOUND);
 	    return;
 	}
 	if (!PERMartok()) {
 	    ARTclose();
-	    Reply("%s\r\n", NOACCESS);
+	    Reply("%d Read access denied for this article\r\n",
+                  PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	    return;
 	}
 	tart=art;
-	Reply("%d %lu %s %s\r\n", what->ReplyCode, (unsigned long) art,
-              av[1], what->Item);
+	Reply("%d %lu %s %s\r\n", what->ReplyCode, art, av[1], what->Item);
 	if (what->Type != STstat) {
 	    ARTsendmmap(what->Type);
 	}
@@ -668,14 +674,11 @@ void CMDfetch(int ac, char *av[])
 	    Reply("%s\r\n", ARTnocurrart);
 	    return;
 	}
-	snprintf(buff, sizeof(buff), "%d", ARTnumber);
+	snprintf(buff, sizeof(buff), "%lu", ARTnumber);
 	tart=ARTnumber;
     }
     else {
-	if (strspn(av[1], "0123456789") != strlen(av[1])) {
-	    Reply("%s\r\n", ARTnoartingroup);
-	    return;
-	}
+        /* We have already checked that the article number is valid. */
 	strlcpy(buff, av[1], sizeof(buff));
 	tart=(ARTNUM)atol(buff);
     }
@@ -692,7 +695,8 @@ void CMDfetch(int ac, char *av[])
         Reply("%s\r\n", ARTnoartingroup);
 	return;
     }
-    Reply("%d %s %.512s %s\r\n", what->ReplyCode, buff, msgid, what->Item); 
+    /* A message-ID does not have more than 250 octets. */
+    Reply("%d %s %.250s %s\r\n", what->ReplyCode, buff, msgid, what->Item); 
     if (what->Type != STstat)
 	ARTsendmmap(what->Type);
     ARTclose();
@@ -702,17 +706,22 @@ void CMDfetch(int ac, char *av[])
 /*
 **  Go to the next or last (really previous) article in the group.
 */
-void CMDnextlast(int ac UNUSED, char *av[])
+void
+CMDnextlast(int ac UNUSED, char *av[])
 {
     char *msgid;
     int	save, delta, errcode;
     bool next;
     const char *message;
 
-    if (!PERMcanread) {
-	Reply("%s\r\n", NOACCESS);
+    /* No syntax to check.  Only check authorizations. */
+    if (!PERMcanread || PERMgroupmadeinvalid) {
+	Reply("%d Read access denied\r\n",
+              PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	return;
     }
+
+    /* Trying to read. */
     if (GRPcount == 0) {
 	Reply("%s\r\n", ARTnotingroup);
 	return;
@@ -722,15 +731,16 @@ void CMDnextlast(int ac UNUSED, char *av[])
 	return;
     }
 
+    /* NEXT? */
     next = (av[0][0] == 'n' || av[0][0] == 'N');
     if (next) {
 	delta = 1;
-	errcode = NNTP_NONEXT_VAL;
+	errcode = NNTP_FAIL_NEXT;
 	message = "next";
     }
     else {
 	delta = -1;
-	errcode = NNTP_NOPREV_VAL;
+	errcode = NNTP_FAIL_PREV;
 	message = "previous";
     }
 
@@ -739,7 +749,7 @@ void CMDnextlast(int ac UNUSED, char *av[])
     do {
         ARTnumber += delta;
         if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
-            Reply("%d No %s to retrieve.\r\n", errcode, message);
+            Reply("%d No %s article to retrieve\r\n", errcode, message);
             ARTnumber = save;
             return;
         }
@@ -749,24 +759,42 @@ void CMDnextlast(int ac UNUSED, char *av[])
         ARTclose();
     } while (msgid == NULL);
 
-    Reply("%d %d %s Article retrieved; request text separately.\r\n",
-	   NNTP_NOTHING_FOLLOWS_VAL, ARTnumber, msgid);
+    Reply("%d %d %s Article retrieved; request text separately\r\n",
+	   NNTP_OK_STAT, ARTnumber, msgid);
 }
 
 
-static bool CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply)
+/*
+**  Parse a range (in av[1]) which may be any of the following:
+**    - An article number.
+**    - An article number followed by a dash to indicate all following.
+**    - An article number followed by a dash followed by another article
+**      number.
+**
+**  In the last case, if the second number is less than the first number,
+**  then the range contains no articles.
+**
+**  In addition to RFC 3977, we also accept:
+**    - A dash followed by an article number to indicate all previous.
+**    - A dash for everything.
+**
+**  ac is the number of arguments in the command:
+**    LISTGROUP news.software.nntp 12-42
+**  gives ac=3 and av[1] should match "12-42" (whence the "av+1" call
+**  of CMDgetrange).
+**
+**  rp->Low and rp->High will contain the values of the range.
+**  *DidReply will be true if this function sends an answer.
+*/
+bool
+CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply)
 {
     char		*p;
 
     *DidReply = false;
-    if (GRPcount == 0) {
-	Reply("%s\r\n", ARTnotingroup);
-	*DidReply = true;
-	return false;
-    }
 
     if (ac == 1) {
-	/* No argument, do only current article. */
+	/* No arguments, do only current article. */
 	if (ARTnumber < ARTlow || ARTnumber > ARThigh) {
 	    Reply("%s\r\n", ARTnocurrart);
 	    *DidReply = true;
@@ -776,22 +804,35 @@ static bool CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply)
         return true;
     }
 
+    /* Check the syntax. */
+    if (!IsValidRange(av[1])) {
+        Reply("%d Syntax error in range\r\n", NNTP_ERR_SYNTAX);
+        *DidReply = true;
+        return false;
+    }
+
     /* Got just a single number? */
     if ((p = strchr(av[1], '-')) == NULL) {
-	rp->Low = rp->High = atol(av[1]);
+        rp->Low = rp->High = atol(av[1]);
         return true;
     }
 
-    /* Parse range. */
+    /* "-" becomes "\0" and we parse the low water mark.
+     * Note that atol() returns 0 if no valid number
+     * is found at the beginning of *p. */
     *p++ = '\0';
     rp->Low = atol(av[1]);
-	if (*p == '\0' || (rp->High = atol(p)) < rp->Low)
-	    /* "XHDR 234-0 header" gives everything to the end. */
-	rp->High = ARThigh;
-    else if (rp->High > ARThigh)
-	rp->High = ARThigh;
+
+    /* Adjust the low water mark. */
     if (rp->Low < ARTlow)
-	rp->Low = ARTlow;
+        rp->Low = ARTlow;
+
+    /* Parse and adjust the high water mark.
+     * "12-" gives everything from 12 to the end.
+     * We do not bother about "42-12" or "42-0" in this function. */
+    if ((*p == '\0') || ((rp->High = atol(p)) > ARThigh))
+        rp->High = ARThigh;
+
     p--;
     *p = '-';
 
@@ -800,7 +841,7 @@ static bool CMDgetrange(int ac, char *av[], ARTRANGE *rp, bool *DidReply)
 
 
 /*
-**  Apply virtual hosting to an Xref field.
+**  Apply virtual hosting to an Xref: field.
 */
 static char *
 vhost_xref(char *p)
@@ -811,13 +852,13 @@ vhost_xref(char *p)
 
     space = strchr(p, ' ');
     if (space == NULL) {
-	warn("malformed Xref `%s'", field);
+	warn("malformed Xref: `%s'", field);
 	goto fail;
     }
     offset = space + 1 - p;
     space = strchr(p + offset, ' ');
     if (space == NULL) {
-	warn("malformed Xref `%s'", field);
+	warn("malformed Xref: `%s'", field);
 	goto fail;
     }
     field = concat(PERMaccessconf->domain, space, NULL);
@@ -827,11 +868,13 @@ vhost_xref(char *p)
 }
 
 /*
-**  XOVER another extension.  Dump parts of the overview database.
+**  Dump parts of the overview database with the OVER command.
+**  The legacy XOVER is also kept, with its specific behaviour.
 */
-void CMDxover(int ac, char *av[])
+void
+CMDover(int ac, char *av[])
 {
-    bool	        DidReply;
+    bool	        DidReply, HasNotReplied;
     ARTRANGE		range;
     struct timeval	stv, etv;
     ARTNUM		artnum;
@@ -841,9 +884,38 @@ void CMDxover(int ac, char *av[])
     int			len, useIOb = 0;
     TOKEN		token;
     struct cvector *vector = NULL;
+    bool                xover, mid;
 
-    if (!PERMcanread) {
-	Printf("%s\r\n", NOACCESS);
+    xover = (strcasecmp(av[0], "XOVER") == 0);
+    mid = (ac > 1 && IsValidMessageID(av[1]));
+
+    if (mid && !xover) {
+        /* FIXME:  We still do not support OVER MSGID, sorry! */
+        Reply("%d Overview by message-ID unsupported\r\n", NNTP_ERR_UNAVAILABLE);
+        return;
+    }
+
+    /* Check the syntax of the arguments first.
+     * We do not accept a message-ID for XOVER, contrary to OVER.  A range
+     * is accepted for both of them. */
+    if (ac > 1 && !IsValidRange(av[1])) {
+        /* It is better to check for a range before a message-ID because
+         * '<' could have been forgotten and the error message should then
+         * report a syntax error in the message-ID. */
+        if (xover || CTYPE(isdigit, av[1][0]) || av[1][0] == '-') {
+            Reply("%d Syntax error in range\r\n", NNTP_ERR_SYNTAX);
+            return;
+        } else if (!mid) {
+            Reply("%d Syntax error in message-ID\r\n", NNTP_ERR_SYNTAX);
+            return;
+        }   
+    }
+
+    /* Check authorizations.  If a range is requested (not a message-ID),
+     * we check whether the group is still readable. */
+    if (!PERMcanread || (!mid && PERMgroupmadeinvalid)) {
+	Reply("%d Read access denied\r\n",
+              PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
 	return;
     }
 
@@ -853,21 +925,28 @@ void CMDxover(int ac, char *av[])
 	return;
     }
 
-    /* Parse range. */
-    if (!CMDgetrange(ac, av, &range, &DidReply)) {
-	if (DidReply) {
-	    return;
-	}
-    }
+    /* Parse range.  CMDgetrange() correctly sets the range when
+     * there is no arguments. */
+    if (!CMDgetrange(ac, av, &range, &DidReply))
+        if (DidReply)
+            return;
 
-    OVERcount++;
-    gettimeofday(&stv, NULL);
+    if (PERMaccessconf->nnrpdoverstats) {
+        OVERcount++;
+        gettimeofday(&stv, NULL);
+    }
     if ((handle = (void *)OVopensearch(GRPcur, range.Low, range.High)) == NULL) {
-	if (av[1] != NULL)
-	    Reply("%d %s fields follow\r\n.\r\n", NNTP_OVERVIEW_FOLLOWS_VAL, av[1]);
-	else
-	    Reply("%d %d fields follow\r\n.\r\n", NNTP_OVERVIEW_FOLLOWS_VAL, ARTnumber);
-	return;
+        /* The response code for OVER is different if a range is provided.
+         * Note that XOVER answers OK. */
+        if (ac > 1)
+            Reply("%d No articles in %s\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_BAD_ARTICLE, av[1]);
+        else
+            Reply("%d Current article number %d is invalid\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_NO_ARTICLE, ARTnumber);
+        if (xover)
+            Printf(".\r\n");
+        return;
     }
     if (PERMaccessconf->nnrpdoverstats) {
 	gettimeofday(&etv, NULL);
@@ -875,11 +954,6 @@ void CMDxover(int ac, char *av[])
 	OVERtime+=(etv.tv_usec - stv.tv_usec) / 1000;
     }
 
-    if (av[1] != NULL)
-	Reply("%d %s fields follow\r\n", NNTP_OVERVIEW_FOLLOWS_VAL, av[1]);
-    else
-	Reply("%d %d fields follow\r\n", NNTP_OVERVIEW_FOLLOWS_VAL, ARTnumber);
-    fflush(stdout);
     if (PERMaccessconf->nnrpdoverstats)
 	gettimeofday(&stv, NULL);
 
@@ -888,6 +962,7 @@ void CMDxover(int ac, char *av[])
        SendIOb because it copies the data. */
     OVctl(OVSTATICSEARCH, &useIOb);
 
+    HasNotReplied = true;
     while (OVsearch(handle, &artnum, &data, &len, &token, NULL)) {
 	if (PERMaccessconf->nnrpdoverstats) {
 	    gettimeofday(&etv, NULL);
@@ -905,12 +980,24 @@ void CMDxover(int ac, char *av[])
 	    OVERhit++;
 	    OVERsize += len;
 	}
+
+        if (HasNotReplied) {
+            if (ac > 1)
+                Reply("%d Overview information for %s follows\r\n",
+                      NNTP_OK_OVER, av[1]);
+            else
+                Reply("%d Overview information for %d follows\r\n",
+                      NNTP_OK_OVER, ARTnumber);
+            fflush(stdout);
+            HasNotReplied = false;
+        }
+
 	vector = overview_split(data, len, NULL, vector);
 	r = overview_getheader(vector, OVERVIEW_MESSAGE_ID, OVextra);
 	cache_add(HashMessageID(r), token);
 	free(r);
 	if (VirtualPathlen > 0 && overhdr_xref != -1) {
-	    if ((overhdr_xref + 1) >= vector->count)
+	    if ((size_t)(overhdr_xref + 1) >= vector->count)
 		continue;
 	    p = vector->strings[overhdr_xref] + sizeof("Xref: ") - 1;
 	    while ((p < data + len) && *p == ' ')
@@ -937,21 +1024,36 @@ void CMDxover(int ac, char *av[])
 	    gettimeofday(&stv, NULL);
     }
 
-    if (vector)
-	cvector_free(vector);
-
     if (PERMaccessconf->nnrpdoverstats) {
         gettimeofday(&etv, NULL);
         OVERtime+=(etv.tv_sec - stv.tv_sec) * 1000;
         OVERtime+=(etv.tv_usec - stv.tv_usec) / 1000;
     }
-    if(useIOb) {
-	SendIOb(".\r\n", 3);
-	PushIOb();
+
+    if (vector)
+        cvector_free(vector);
+
+    if (HasNotReplied) {
+        /* The response code for OVER is different if a range is provided.
+         * Note that XOVER answers OK. */
+        if (ac > 1)
+            Reply("%d No articles in %s\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_BAD_ARTICLE, av[1]);
+        else
+            Reply("%d Current article number %d is invalid\r\n",
+                  xover ? NNTP_OK_OVER : NNTP_FAIL_NO_ARTICLE, ARTnumber);
+        if (xover)
+            Printf(".\r\n");
     } else {
-	SendIOv(".\r\n", 3);
-	PushIOv();
+        if(useIOb) {
+            SendIOb(".\r\n", 3);
+            PushIOb();
+        } else {
+            SendIOv(".\r\n", 3);
+            PushIOv();
+        }
     }
+
     if (PERMaccessconf->nnrpdoverstats)
 	gettimeofday(&stv, NULL);
     OVclosesearch(handle);
@@ -964,18 +1066,18 @@ void CMDxover(int ac, char *av[])
 }
 
 /*
-**  XHDR and XPAT extensions.  Note that HDR as specified in the new NNTP
-**  draft works differently than XHDR has historically, so don't just use this
-**  function to implement it without reviewing the differences.
+**  Access specific fields from an article with HDR.
+**  The legacy XHDR and XPAT are also kept, with their specific behaviours.
 */
-/* ARGSUSED */
-void CMDpat(int ac, char *av[])
+void
+CMDpat(int ac, char *av[])
 {
     char	        *p;
-    int	        	i;
+    unsigned long      	i;
     ARTRANGE		range;
-    bool		IsLines;
-    bool		DidReply;
+    bool		IsBytes, IsLines;
+    bool                IsMetaBytes, IsMetaLines;
+    bool		DidReply, HasNotReplied;
     char		*header;
     char		*pattern;
     char		*text;
@@ -987,103 +1089,208 @@ void CMDpat(int ac, char *av[])
     int                 len;
     TOKEN               token;
     struct cvector *vector = NULL;
+    bool                hdr, mid;
 
-    if (!PERMcanread) {
-	Printf("%s\r\n", NOACCESS);
-	return;
+    hdr = (strcasecmp(av[0], "HDR") == 0);
+    mid = (ac > 2 && IsValidMessageID(av[2]));
+
+    /* Check the syntax of the arguments first. */
+    if (ac > 2 && !IsValidRange(av[2])) {
+        /* It is better to check for a range before a message-ID because
+         * '<' could have been forgotten and the error message should then
+         * report a syntax error in the message-ID. */
+        if (CTYPE(isdigit, av[2][0]) || av[2][0] == '-') {
+            Reply("%d Syntax error in range\r\n", NNTP_ERR_SYNTAX);
+            return;
+        } else if (!mid) {
+            Reply("%d Syntax error in message-ID\r\n", NNTP_ERR_SYNTAX);
+            return;
+        }
     }
 
     header = av[1];
-    IsLines = (strcasecmp(header, "lines") == 0);
 
-    if (ac > 3) /* XPAT */
+    /* If metadata is asked for, convert it to headers that
+     * the overview database knows. */
+    IsBytes = (strcasecmp(header, "Bytes") == 0);
+    IsLines = (strcasecmp(header, "Lines") == 0);
+    IsMetaBytes = (strcasecmp(header, ":bytes") == 0);
+    IsMetaLines = (strcasecmp(header, ":lines") == 0);
+    /* Make these changes because our overview database does
+     * not currently know metadata names. */
+    if (IsMetaBytes)
+        header = xstrdup("Bytes");
+    if (IsMetaLines)
+        header = xstrdup("Lines");
+
+    /* We only allow :bytes and :lines for metadata. */
+    if (!IsMetaLines && !IsMetaBytes) {
+        p = av[1];
+        p++;
+        if (strncasecmp(header, ":", 1) == 0 && IsValidHeaderName(p)) {
+            Reply("%d Unsupported metadata request\r\n",
+                  NNTP_ERR_UNAVAILABLE, header);
+            return;
+        } else if (!IsValidHeaderName(header)) {
+            Reply("%d Syntax error in header name\r\n", NNTP_ERR_SYNTAX);
+            return;
+        }
+    }
+
+    /* Check authorizations.  If a range is requested (not a message-ID),
+     * we check whether the group is still readable. */
+    if (!PERMcanread || (!mid && PERMgroupmadeinvalid)) {
+        Reply("%d Read access denied\r\n",
+              PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
+        return;
+    }
+
+    if (ac > 3) /* Necessarily XPAT. */
 	pattern = Glom(&av[3]);
     else
 	pattern = NULL;
 
+    /* We will only do the loop once.  It is just in order to easily break. */
     do {
 	/* Message-ID specified? */
-	if (ac > 2 && av[2][0] == '<') {
+	if (mid) {
 	    p = av[2];
 	    if (!ARTopenbyid(p, &artnum, false)) {
-		Printf("%d No such article.\r\n", NNTP_DONTHAVEIT_VAL);
+		Reply("%d No such article\r\n", NNTP_FAIL_NOTFOUND);
 		break;
 	    }
-            if (!PERMartok()) {
-                ARTclose();
-                Printf("%s\r\n", NOACCESS);
+
+            /* FIXME:  We do not handle metadata requests by message-ID. */
+            if (hdr && (IsMetaBytes || IsMetaLines)) {
+                Reply("%d Metadata requests by message-ID unsupported\r\n",
+                      NNTP_ERR_UNAVAILABLE);
                 break;
             }
-                
-	    Printf("%d %s matches follow (ID)\r\n", NNTP_HEAD_FOLLOWS_VAL,
-		   header);
-	    if ((text = GetHeader(header)) != NULL
+
+            if (!PERMartok()) {
+                ARTclose();
+                Reply("%d Read access denied for this article\r\n",
+                      PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
+                break;
+            }
+
+	    Reply("%d Header information for %s follows (from the article)\r\n",
+                   hdr ? NNTP_OK_HDR : NNTP_OK_HEAD, av[1]);
+
+	    if ((text = GetHeader(av[1])) != NULL
 		&& (!pattern || uwildmat_simple(text, pattern)))
-		Printf("%s %s\r\n", p, text);
+		Printf("%s %s\r\n", hdr ? "0" : p, text);
+            else if (hdr) {
+                /* We always have to answer something with HDR. */
+                Printf("0 \r\n");
+            }
 
 	    ARTclose();
 	    Printf(".\r\n");
 	    break;
 	}
 
+        /* Trying to read. */
 	if (GRPcount == 0) {
 	    Reply("%s\r\n", ARTnotingroup);
 	    break;
 	}
 
-	/* Range specified. */
-	if (!CMDgetrange(ac - 1, av + 1, &range, &DidReply)) {
-	    if (DidReply) {
-		break;
-	    }
-	}
+        /* Parse range.  CMDgetrange() correctly sets the range when
+         * there is no arguments. */
+        if (!CMDgetrange(ac - 1, av + 1, &range, &DidReply))
+            if (DidReply)
+                break;
 
 	/* In overview? */
         Overview = overview_index(header, OVextra);
 
-	/* Not in overview, we have to fish headers out from the articles */
-	if (Overview < 0 ) {
-	    Reply("%d %s matches follow (art)\r\n", NNTP_HEAD_FOLLOWS_VAL,
-		  header);
+        HasNotReplied = true;
+
+	/* Not in overview, we have to fish headers out from the articles. */
+	if (Overview < 0 || IsBytes || IsLines) {
 	    for (i = range.Low; i <= range.High && range.High > 0; i++) {
 		if (!ARTopen(i))
 		    continue;
+                if (HasNotReplied) {
+                    Reply("%d Header information for %s follows (from articles)\r\n",
+                          hdr ? NNTP_OK_HDR : NNTP_OK_HEAD, av[1]);
+                    HasNotReplied = false;
+                }
 		p = GetHeader(header);
 		if (p && (!pattern || uwildmat_simple(p, pattern))) {
-		    snprintf(buff, sizeof(buff), "%u ", i);
+		    snprintf(buff, sizeof(buff), "%lu ", i);
 		    SendIOb(buff, strlen(buff));
 		    SendIOb(p, strlen(p));
 		    SendIOb("\r\n", 2);
-		}
+		} else if (hdr) {
+                    /* We always have to answer something with HDR. */
+                    snprintf(buff, sizeof(buff), "%lu \r\n", i);
+                    SendIOb(buff, strlen(buff));
+                }
                 ARTclose();
 	    }
-	    SendIOb(".\r\n", 3);
+            if (HasNotReplied) {
+                if (hdr) {
+                    if (ac > 2)
+                        Reply("%d No articles in %s\r\n",
+                              NNTP_FAIL_BAD_ARTICLE, av[2]);
+                    else
+                        Reply("%d Current article number %d is invalid\r\n",
+                              NNTP_FAIL_NO_ARTICLE, ARTnumber);
+                } else {
+                    Reply("%d No header information for %s follows (from articles)\r\n",
+                          NNTP_OK_HEAD, av[1]);
+                    Printf(".\r\n");
+                }
+                break;
+            } else {
+                SendIOb(".\r\n", 3);
+            }
 	    PushIOb();
 	    break;
 	}
 
-	/* Okay then, we can grab values from overview. */
+	/* Okay then, we can grab values from the overview database. */
 	handle = (void *)OVopensearch(GRPcur, range.Low, range.High);
 	if (handle == NULL) {
-	    Reply("%d %s no matches follow (NOV)\r\n.\r\n",
-		  NNTP_HEAD_FOLLOWS_VAL, header);
+            if (hdr) {
+                if (ac > 2)
+                    Reply("%d No articles in %s\r\n",
+                          NNTP_FAIL_BAD_ARTICLE, av[2]);
+                else
+                    Reply("%d Current article number %d is invalid\r\n",
+                          NNTP_FAIL_NO_ARTICLE, ARTnumber);
+            } else {
+                Reply("%d No header information for %s follows (from overview)\r\n",
+                      NNTP_OK_HEAD, av[1]);
+                Printf(".\r\n");
+            }
 	    break;
 	}	
 	
-	Printf("%d %s matches follow (NOV)\r\n", NNTP_HEAD_FOLLOWS_VAL,
-	       header);
 	while (OVsearch(handle, &artnum, &data, &len, &token, NULL)) {
 	    if (len == 0 || (PERMaccessconf->nnrpdcheckart
 		&& !ARTinstorebytoken(token)))
 		continue;
+            if (HasNotReplied) {
+                Reply("%d Header or metadata information for %s follows (from overview)\r\n",
+                      hdr ? NNTP_OK_HDR : NNTP_OK_HEAD, av[1]);
+                HasNotReplied = false;
+            }
 	    vector = overview_split(data, len, NULL, vector);
 	    p = overview_getheader(vector, Overview, OVextra);
 	    if (p != NULL) {
 		if (PERMaccessconf->virtualhost &&
 			   Overview == overhdr_xref) {
 		    p = vhost_xref(p);
-		    if (p == NULL)
+		    if (p == NULL) {
+                        if (hdr) {
+                            snprintf(buff, sizeof(buff), "%lu \r\n", artnum);
+                            SendIOb(buff, strlen(buff));
+                        }
 			continue;
+                    }
 		}
 		if (!pattern || uwildmat_simple(p, pattern)) {
 		    snprintf(buff, sizeof(buff), "%lu ", artnum);
@@ -1091,10 +1298,31 @@ void CMDpat(int ac, char *av[])
 		    SendIOb(p, strlen(p));
 		    SendIOb("\r\n", 2);
 		}
+                /* No need to have another condition for HDR because
+                 * pattern is NULL for it, and p is not NULL here. */
 		free(p);
-	    }
+	    } else if (hdr) {
+                snprintf(buff, sizeof(buff), "%lu \r\n", artnum);
+                SendIOb(buff, strlen(buff));
+            }
 	}
-	SendIOb(".\r\n", 3);
+        if (HasNotReplied) {
+            if (hdr) {
+                if (ac > 2)
+                    Reply("%d No articles in %s\r\n",
+                          NNTP_FAIL_BAD_ARTICLE, av[2]);
+                else
+                    Reply("%d Current article number %d is invalid\r\n",
+                          NNTP_FAIL_NO_ARTICLE, ARTnumber);
+            } else {
+                Reply("%d No header or metadata information for %s follows (from overview)\r\n",
+                      NNTP_OK_HEAD, av[1]);
+                Printf(".\r\n");
+            }
+            break;
+        } else {
+            SendIOb(".\r\n", 3);
+        }
 	PushIOb();
 	OVclosesearch(handle);
     } while (0);
