@@ -1,6 +1,6 @@
-/*  $Revision: 1.42 $
+/*  $Id: newnews.c 8052 2008-09-26 21:23:47Z iulius $
 **
-**  The newnews command.
+**  The NEWNEWS command.
 */
 #include "config.h"
 #include "clibrary.h"
@@ -9,13 +9,14 @@
 #include "inn/messages.h"
 #include "inn/wire.h"
 #include "nnrpd.h"
-#include "ov.h"
+#include "inn/ov.h"
 #include "cache.h"
 
 #define GROUP_LIST_DELTA	10
 
-static bool FindHeader(ARTHANDLE *art, const char **pp, const char **qp,
-    const char* hdr, size_t hdrlen)
+static bool
+FindHeader(ARTHANDLE *art, const char **pp, const char **qp,
+           const char* hdr, size_t hdrlen UNUSED)
 {
   const char *p, *p1, *q;
   bool Nocr = true;
@@ -46,11 +47,12 @@ static bool FindHeader(ARTHANDLE *art, const char **pp, const char **qp,
 }
 
 /*
-**  get Xref header
+**  Get Xref: header.
 */
-static char *GetXref(ARTHANDLE *art) {
+static char *
+GetXref(ARTHANDLE *art)
+{
   const char	*p, *q;
-  static char	buff[BIG_BUFFER];
 
   if (!FindHeader(art, &p, &q, "xref", sizeof("xref")))
     return NULL;
@@ -61,7 +63,9 @@ static char *GetXref(ARTHANDLE *art) {
 **  Split newsgroup list into array of newsgroups.  Return static pointer,
 **  or NULL if there are no newsgroup.
 */
-static char **GetGroups(char *p) {
+static char **
+GetGroups(char *p)
+{
   static int	size;
   static char	**list;
   int		i;
@@ -105,7 +109,9 @@ static char **GetGroups(char *p) {
   return i ? list : NULL;
 }
 
-static bool HaveSeen(bool AllGroups, char *group, char **groups, char **xrefs) {
+static bool
+HaveSeen(bool AllGroups, char *group, char **groups, char **xrefs)
+{
   char *list[2];
 
   list[1] = NULL;
@@ -126,6 +132,7 @@ static char **groups;
 static void
 process_newnews(char *group, bool AllGroups, time_t date)
 {
+    int low, high;
     char **xrefs;
     int count;
     void *handle;
@@ -134,7 +141,7 @@ process_newnews(char *group, bool AllGroups, time_t date)
     ARTHANDLE *art = NULL;
     TOKEN token;
     char *data;
-    int i, len;
+    int len;
     char *grplist[2];
     time_t now;
 
@@ -144,8 +151,10 @@ process_newnews(char *group, bool AllGroups, time_t date)
 	return;
     if (!AllGroups && !PERMmatch(groups, grplist))
 	return;
-    if (!OVgroupstats(group, &ARTlow, &ARThigh, &count, NULL))
+    if (!OVgroupstats(group, &low, &high, &count, NULL))
 	return;
+    ARTlow = low;
+    ARThigh = high;
     if ((handle = OVopensearch(group, ARTlow, ARThigh)) != NULL) {
 	ARTNUM artnum;
 	unsigned long artcount = 0;
@@ -153,7 +162,8 @@ process_newnews(char *group, bool AllGroups, time_t date)
 
 	if (innconf->nfsreader) {
 	    time(&now);
-	    /* move the start time back nfsreaderdelay seconds */
+	    /* Move the start time back nfsreaderdelay seconds
+             * as we are an NFS reader. */
 	    if (date >= innconf->nfsreaderdelay)
 		date -= innconf->nfsreaderdelay;
 	}
@@ -174,7 +184,7 @@ process_newnews(char *group, bool AllGroups, time_t date)
 		    !ARTinstorebytoken(token))
 		    continue;
 		/* We only care about the newsgroup list here, virtual
-		 * hosting isn't relevant */
+		 * hosting isn't relevant. */
 		p = overview_getheader(vector, overhdr_xref, OVextra);
 	    }
 	    if (p == NULL)
@@ -195,17 +205,19 @@ process_newnews(char *group, bool AllGroups, time_t date)
 	    free(p);
 	}
 	OVclosesearch(handle);
-	notice("%s newnews %s %lu", ClientHost, group, artcount);
+	notice("%s newnews %s %lu", Client.host, group, artcount);
 	if (vector)
 	    cvector_free(vector);
     }
 }
 
 /*
-**  NEWNEWS newsgroups date time ["GMT"]
-**  Return the Message-ID of any articles after the specified date
+**  NEWNEWS wildmat date time [GMT]
+**  Return the message-ID of any articles after the specified date.
 */
-void CMDnewnews(int ac, char *av[]) {
+void
+CMDnewnews(int ac, char *av[])
+{
   char		*p, *q;
   char          *path;
   bool		AllGroups;
@@ -213,70 +225,86 @@ void CMDnewnews(int ac, char *av[]) {
   time_t	date;
   QIOSTATE	*qp;
   int		i;
-  bool          local;
-
-  if (!PERMaccessconf->allownewnews) {
-      Reply("%d NEWNEWS command disabled by administrator\r\n", NNTP_ACCESS_VAL);
-      return;
-  }
-
-  if (!PERMcanread) {
-      Reply("%s\r\n", NNTP_ACCESS);
-      return;
-  }
-
-  /* Make other processes happier if someone uses NEWNEWS */
-  if (innconf->nicenewnews > 0) {
-      nice(innconf->nicenewnews);
-      innconf->nicenewnews = 0;
-  }
-
-  snprintf(line, sizeof(line), "%s %s %s %s", av[1], av[2], av[3],
-	   (ac >= 5 && (*av[4] == 'G' || *av[4] == 'U')) ? "GMT" : "local");
-  notice("%s newnews %s", ClientHost, line);
+  bool          local = true;
 
   TMRstart(TMR_NEWNEWS);
-  /* Optimization in case client asks for !* (no groups) */
-  if (strcmp(av[1], "!*") == 0) {
-      Reply("%s\r\n", NNTP_NEWNEWSOK);
-      Printf(".\r\n");
-      TMRstop(TMR_NEWNEWS);
-      return;
+
+  /* Check the arguments and parse the date. */
+  if (ac > 4) {
+      if (strcasecmp(av[4], "GMT") == 0)
+          local = false;
+      else {
+          Reply("%d Syntax error for \"GMT\"\r\n", NNTP_ERR_SYNTAX);
+          TMRstop(TMR_NEWNEWS);
+          return;
+      }
   }
 
   /* Parse the newsgroups. */
   AllGroups = (strcmp(av[1], "*") == 0);
   if (!AllGroups && !NGgetlist(&groups, av[1])) {
-      Reply("%d Bad newsgroup specifier %s\r\n", NNTP_SYNTAX_VAL, av[1]);
+      Reply("%d Bad newsgroup specifier %s\r\n", NNTP_ERR_SYNTAX, av[1]);
       TMRstop(TMR_NEWNEWS);
       return;
   }
 
   /* Parse the date. */
-  local = !(ac > 4 && strcasecmp(av[4], "GMT") == 0);
   date = parsedate_nntp(av[2], av[3], local);
   if (date == (time_t) -1) {
-      Reply("%d Bad date\r\n", NNTP_SYNTAX_VAL);
+      Reply("%d Bad date\r\n", NNTP_ERR_SYNTAX);
       TMRstop(TMR_NEWNEWS);
       return;
   }
 
+  /* Check authorizations. */
+  if (!PERMaccessconf->allownewnews) {
+      Reply("%d NEWNEWS command disabled by administrator\r\n",
+            PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
+      TMRstop(TMR_NEWNEWS);
+      return;
+  }
+
+  if (!PERMcanread) {
+      Reply("%d Read access denied\r\n",
+            PERMcanauthenticate ? NNTP_FAIL_AUTH_NEEDED : NNTP_ERR_ACCESS);
+      TMRstop(TMR_NEWNEWS);
+      return;
+  }
+
+  snprintf(line, sizeof(line), "%s %s %s %s", av[1], av[2], av[3],
+           local ? "local" : "GMT");
+  notice("%s newnews %s", Client.host, line);
+
+  /* Optimization in case client asks for !* (no groups). */
+  if (strcmp(av[1], "!*") == 0) {
+      Reply("%d No new news\r\n", NNTP_OK_NEWNEWS);
+      Printf(".\r\n");
+      TMRstop(TMR_NEWNEWS);
+      return;
+  }
+
+  /* Make other processes happier if someone uses NEWNEWS. */
+  if (innconf->nicenewnews > 0) {
+      nice(innconf->nicenewnews);
+      innconf->nicenewnews = 0;
+  }
+
   if (strcspn(av[1], "\\!*[?]") == strlen(av[1])) {
-      /* optimise case - don't need to scan the active file pattern
-       * matching */
+      /* Optimise case -- don't need to scan the active file pattern
+       * matching. */
       Reply("%s\r\n", NNTP_NEWNEWSOK);
       for (i = 0; groups[i]; ++i) {
 	  process_newnews(groups[i], AllGroups, date);
       }
   } else {
-      path = concatpath(innconf->pathdb, _PATH_ACTIVE);
+      path = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
       qp = QIOopen(path);
       if (qp == NULL) {
 	  if (errno == ENOENT) {
-	      Reply("%d Can't open active\r\n", NNTP_TEMPERR_VAL);
+	      Reply("%d Can't open active\r\n", NNTP_FAIL_ACTION);
 	  } else {
-	      syswarn("%s cant fopen %s", ClientHost, path);
-	      Reply("%d Can't open active\r\n", NNTP_TEMPERR_VAL);
+	      syswarn("%s can't fopen %s", Client.host, path);
+	      Reply("%d Can't open active\r\n", NNTP_FAIL_ACTION);
 	  }
 	  free(path);
 	  TMRstop(TMR_NEWNEWS);
