@@ -1,7 +1,7 @@
 /*
 ** AUTHINFO SASL functionality.
 **
-** $Id: sasl.c 10117 2016-11-06 14:23:27Z iulius $
+** $Id: sasl.c 8381 2009-03-15 10:00:38Z iulius $
 */
 
 #include "config.h"
@@ -10,19 +10,18 @@
 #include "inn/messages.h"
 #include "nnrpd.h"
 
-/* Outside the ifdef so that make depend works even ifndef HAVE_OPENSSL. */
+/* Outside the ifdef so that make depend works even ifndef HAVE_SSL. */
 #include "inn/ov.h"
-#include "tls.h"
 
-#if defined(HAVE_OPENSSL)
+#ifdef HAVE_SSL
 extern int tls_cipher_usebits;
 extern char *tls_peer_CN;
-#endif /* HAVE_OPENSSL */
-#if defined(HAVE_OPENSSL) || defined(HAVE_SASL)
-extern bool encryption_layer_on;
-#endif /* HAVE_OPENSSL || HAVE_SASL */
+extern bool nnrpd_starttls_done;
+#endif /* HAVE_SSL */
 
 #ifdef HAVE_SASL
+
+#include <sasl/sasl.h>
 sasl_conn_t *sasl_conn = NULL;
 int sasl_ssf = 0;
 int sasl_maxout = NNTP_MAXLEN_COMMAND;
@@ -33,8 +32,39 @@ sasl_callback_t sasl_callbacks[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
-#define BASE64_BUF_SIZE 21848   /* Per RFC 4422:  (floor(n/3) + 1) * 4
-                                   where n = 16 kB = 16384 bytes. */
+#define BASE64_BUF_SIZE 21848	/* Per RFC 2222bis:  ((16K / 3) + 1) * 4. */
+
+
+/*
+**  Check if the argument is a valid mechanism according to RFC 4643:
+**
+**    mechanism = 1*20mech-char
+**    mech-char = UPPER / DIGIT / "-" / "_"
+*/
+static bool
+IsValidMechanism(const char *string)
+{
+    int len = 0;
+    const unsigned char *p;
+ 
+    /* Not NULL. */
+    if (string == NULL)
+        return false;
+
+    p = (const unsigned char *) string;
+
+    for (; *p != '\0'; p++) {
+        len++;
+        if (!CTYPE(isalnum, *p) && *p != '-' && *p != '_')
+            return false;
+    }
+
+    if (len > 0 && len < 21)
+        return true;
+    else
+        return false;
+}
+
 
 /*
 **  Create a new SASL server authentication object.
@@ -63,9 +93,9 @@ SASLnewserver(void)
         secprops.max_ssf = 256;
         secprops.maxbufsize = NNTP_MAXLEN_COMMAND;
         sasl_setprop(sasl_conn, SASL_SEC_PROPS, &secprops);
-#ifdef HAVE_OPENSSL
+#ifdef HAVE_SSL
         /* Tell SASL about the negotiated TLS layer. */
-        if (encryption_layer_on) {
+        if (nnrpd_starttls_done) {
             if (sasl_setprop(sasl_conn, SASL_SSF_EXTERNAL,
                              (sasl_ssf_t *) &tls_cipher_usebits) != SASL_OK) {
                 syslog(L_NOTICE, "sasl_setprop() failed:  TLS layer for SASL");
@@ -86,7 +116,6 @@ SASLauth(int ac, char *av[])
     const char *mech;
     const char *clientin = NULL;
     unsigned int clientinlen = 0;
-    size_t tclientinlen = 0;
     const char *serverout = NULL;
     unsigned int serveroutlen;
     char base64[BASE64_BUF_SIZE+1];
@@ -106,8 +135,8 @@ SASLauth(int ac, char *av[])
 
     mech = av[2];
 
-    if (!IsValidAlgorithm(mech)) {
-        Reply("%d Syntax error in mechanism name\r\n", NNTP_ERR_SYNTAX);
+    if (!IsValidMechanism(mech)) {
+        Reply("%d Syntax error in mechanism\r\n", NNTP_ERR_SYNTAX);
         return;
     }
 
@@ -120,14 +149,14 @@ SASLauth(int ac, char *av[])
         return;
     }
 
-#ifdef HAVE_OPENSSL
+#ifdef HAVE_SSL
     /* Check whether STARTTLS must be used before trying to authenticate
      * with AUTHINFO SASL PLAIN, LOGIN or EXTERNAL. */
     if (PERMcanauthenticate && !PERMcanauthenticatewithoutSSL
-        && !encryption_layer_on && ((strcasecmp(mech, "PLAIN") == 0
+        && !nnrpd_starttls_done && ((strcasecmp(mech, "PLAIN") == 0
                                      || strcasecmp(mech, "LOGIN") == 0
                                      || strcasecmp(mech, "EXTERNAL") == 0))) {
-        Reply("%d Encryption layer required\r\n", NNTP_FAIL_PRIVACY_NEEDED);
+        Reply("%d Encryption required\r\n", NNTP_FAIL_PRIVACY_NEEDED);
         return;
     }
 #endif
@@ -142,9 +171,8 @@ SASLauth(int ac, char *av[])
             clientinlen = 0;
 	} else {
 	    /* Decode the response.  On error, SASL_CONTINUE should not be
-             * given because we know for sure that we have already received
-             * the whole challenge/response.  Use SASL_BADPROT instead,
-             * in order to indicate a base64-encoding error. */
+             * given.  Use SASL_BADPROT instead, in order to indicate
+             * a base64-encoding error. */
             r1 = sasl_decode64(clientin, strlen(clientin),
                                base64, BASE64_BUF_SIZE, &clientinlen);
             clientin = base64;
@@ -161,11 +189,9 @@ SASLauth(int ac, char *av[])
 
     while (r == SASL_CONTINUE || (r == SASL_OK && serveroutlen != 0)) {
 	if (serveroutlen != 0) {
-            /* Encode the server challenge.
-             * In sasl_encode64() calls, the fourth argument is the length
-             * of the third including the null terminator. */
+	    /* Encode the server challenge. */
             r1 = sasl_encode64(serverout, serveroutlen,
-                               base64, BASE64_BUF_SIZE+1, NULL);
+                               base64, BASE64_BUF_SIZE, NULL);
             if (r1 != SASL_OK)
                 r = r1;
 	}
@@ -181,8 +207,7 @@ SASLauth(int ac, char *av[])
 
 	/* Get the response from the client. */
 	r1 = line_read(&NNTPline, PERMaccessconf->clienttimeout,
-		      &clientin, &tclientinlen, NULL);
-        clientinlen = tclientinlen;
+		      &clientin, &clientinlen, NULL);
         
         switch (r1) {
 	case RTok:
@@ -227,9 +252,8 @@ SASLauth(int ac, char *av[])
             clientinlen = 0;
         } else {
             /* Decode the response.  On error, SASL_CONTINUE should not be
-             * given because we know for sure that we have already received
-             * the whole challenge/response.  Use SASL_BADPROT instead,
-             * in order to indicate a base64-encoding error. */
+             * given.  Use SASL_BADPROT instead, in order to indicate
+             * a base64-encoding error. */
             r1 = sasl_decode64(clientin, clientinlen,
                                base64, BASE64_BUF_SIZE, &clientinlen);
             clientin = base64;
@@ -262,22 +286,26 @@ SASLauth(int ac, char *av[])
     }
 
     if (r == SASL_OK) {
-        /* Success!
-         * First, save info about the negotiated security layer
-         * for I/O functions. */
-        sasl_ssf = *ssfp;
-        sasl_maxout = (*maxoutp == 0 || *maxoutp > NNTP_MAXLEN_COMMAND) ?
-            NNTP_MAXLEN_COMMAND : *maxoutp;
+	/* Success. */
+	strlcpy(PERMuser, canon_user, sizeof(PERMuser));
+        PERMgetpermissions();
+	PERMneedauth = false;
+	PERMauthorized = true;
+        PERMcanauthenticate = false;
 
-        if (sasl_ssf > 1) {
-            /* For the forthcoming check of the permissions the client now
-             * has, tell the connection is encrypted, so that auth blocks
-             * requiring the negotiation of a security layer in readers.conf
-             * are properly taken into account.
-             * When sasl_ssf equals 1, only data integrity is provided, without
-             * any security. */
-            encryption_layer_on = true;
+	syslog(L_NOTICE, "%s user %s", Client.host, PERMuser);
 
+	if (serveroutlen)
+	    Reply("%d %s\r\n", NNTP_OK_SASL, base64);
+	else
+	    Reply("%d Authentication succeeded\r\n", NNTP_OK_AUTHINFO);
+
+	/* Save info about the negotiated security layer for I/O functions. */
+	sasl_ssf = *ssfp;
+	sasl_maxout =
+	    (*maxoutp == 0 || *maxoutp > NNTP_MAXLEN_COMMAND) ? NNTP_MAXLEN_COMMAND : *maxoutp;
+
+        if (sasl_ssf > 0) {
             /* Close out any existing article, report group stats.
              * RFC 4643 requires the reset of any knowledge about the client. */
             if (GRPcur) {
@@ -287,34 +315,13 @@ SASLauth(int ac, char *av[])
                 OVctl(OVCACHEFREE, &boolval);
                 free(GRPcur);
                 GRPcur = NULL;
-                if (ARTcount) {
-                    syslog(L_NOTICE,
-                           "%s exit for AUTHINFO SASL articles %ld groups %ld",
+                if (ARTcount)
+                    syslog(L_NOTICE, "%s exit for AUTHINFO SASL articles %ld groups %ld",
                            Client.host, ARTcount, GRPcount);
-                }
                 GRPcount = 0;
                 PERMgroupmadeinvalid = false;
-
-                /* Reset our read buffer so as to prevent plaintext
-                 * command injection. */
-                line_reset(&NNTPline);
             }
         }
-
-        PERMgetaccess(false);
-        strlcpy(PERMuser, canon_user, sizeof(PERMuser));
-        PERMgetpermissions();
-        PERMneedauth = false;
-        PERMauthorized = true;
-        PERMcanauthenticate = false;
-
-        syslog(L_NOTICE, "%s user %s", Client.host, PERMuser);
-
-        if (serveroutlen) {
-            Reply("%d %s\r\n", NNTP_OK_SASL, base64);
-        } else {
-            Reply("%d Authentication succeeded\r\n", NNTP_OK_AUTHINFO);
-        }
     } else {
 	/* Failure. */
 	int resp_code;
