@@ -1,4 +1,4 @@
-/*  $Id: cnfs.h 4266 2001-01-04 06:01:36Z rra $
+/*  $Id: cnfs.h 6530 2003-10-21 00:31:33Z rra $
 **
 **  cyclic news file system header
 */
@@ -9,7 +9,7 @@
 bool cnfs_init(SMATTRIBUTE *attr);
 TOKEN cnfs_store(const ARTHANDLE article, const STORAGECLASS class);
 ARTHANDLE *cnfs_retrieve(const TOKEN token, const RETRTYPE amount);
-ARTHANDLE *cnfs_next(const ARTHANDLE *article, const RETRTYPE amount);
+ARTHANDLE *cnfs_next(ARTHANDLE *article, const RETRTYPE amount);
 void cnfs_freearticle(ARTHANDLE *article);
 bool cnfs_cancel(TOKEN token);
 bool cnfs_ctl(PROBETYPE type, TOKEN *token, void *value);
