@@ -1,4 +1,4 @@
-/*  $Id: interface.h,v 1.22 2002/12/07 09:47:16 rra Exp $
+/*  $Id: interface.h 7585 2006-11-21 09:37:51Z eagle $
 **
 **  Storage Manager interface header
 */
@@ -7,7 +7,7 @@
 #define __INTERFACE_H__
 
 #include "config.h"
-#include "storage.h"
+#include "inn/storage.h"
 #include <stdio.h>
 
 typedef struct {
@@ -21,7 +21,7 @@ typedef struct {
     bool                (*init)(SMATTRIBUTE *attr);
     TOKEN               (*store)(const ARTHANDLE article, const STORAGECLASS storageclass);
     ARTHANDLE           *(*retrieve)(const TOKEN token, const RETRTYPE amount);
-    ARTHANDLE           *(*next)(const ARTHANDLE *article, const RETRTYPE amount);
+    ARTHANDLE           *(*next)(ARTHANDLE *article, const RETRTYPE amount);
     void                (*freearticle)(ARTHANDLE *article);
     bool                (*cancel)(TOKEN token);
     bool                (*ctl)(PROBETYPE type, TOKEN *token, void *value);
@@ -53,6 +53,6 @@ extern bool SMpreopen;
 char *SMFindBody(char *article, int len);
 STORAGE_SUB *SMGetConfig(STORAGETYPE type, STORAGE_SUB *sub);
 STORAGE_SUB *SMgetsub(const ARTHANDLE article);
-void SMseterror(int errorno, char *error);
+void SMseterror(int errorno, const char *error);
 
 #endif /* __INTERFACE_H__ */
