@@ -1,4 +1,4 @@
-/*  $Id: overdata.c 7985 2008-08-28 21:51:06Z iulius $
+/*  $Id: overdata.c 8409 2009-04-11 22:13:41Z iulius $
 **
 **  Overview data processing.
 **
@@ -13,22 +13,20 @@
 #include "inn/buffer.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "inn/qio.h"
 #include "inn/wire.h"
 #include "inn/vector.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 #include "ovinterface.h"
-#include "paths.h"
 
 
-/* The standard overview fields. */
+/* The standard overview fields.  The order of these fields is important. */
 static const char * const fields[] = {
     "Subject", "From", "Date", "Message-ID", "References", "Bytes", "Lines"
 };
 
 
 /*
-**  Return a vector of the standard overview fields. Note there is no
+**  Return a vector of the standard overview fields.  Note there is no
 **  way to free up the resulting data structure.
 */
 const struct cvector *
@@ -50,71 +48,50 @@ overview_fields(void)
 }
 
 /*
-**  Parse the overview schema and return a vector of the additional fields
-**  over the standard ones.  Caller is responsible for freeing the vector.
+**  Return a vector of the additional fields over the standard ones.
+**  The order of these fields is important.
+**
+**  Xref: is mandatory for INN and we make it the first extra field
+**  after the seven overview fields defined in RFC 3977.
+**
+**  Caller is responsible for freeing the vector.
 */
 struct vector *
-overview_extra_fields(void)
+overview_extra_fields(bool hidden)
 {
     struct vector *list = NULL;
-    struct vector *result = NULL;
-    char *schema = NULL;
-    char *line, *p;
-    QIOSTATE *qp = NULL;
-    unsigned int field;
-    bool full = false;
+    unsigned int i;
 
-    schema = concatpath(innconf->pathetc, _PATH_SCHEMA);
-    qp = QIOopen(schema);
-    if (qp == NULL) {
-        syswarn("cannot open %s", schema);
-        goto done;
-    }
     list = vector_new();
-    for (field = 0, line = QIOread(qp); line != NULL; line = QIOread(qp)) {
-        while (ISWHITE(*line))
-            line++;
-        p = strchr(line, '#');
-        if (p != NULL)
-            *p = '\0';
-        p = strchr(line, '\n');
-        if (p != NULL)
-            *p = '\0';
-        if (*line == '\0')
-            continue;
-        p = strchr(line, ':');
-        if (p != NULL) {
-            *p++ = '\0';
-            full = (strcmp(p, "full") == 0);
-        }
-        if (field >= ARRAY_SIZE(fields)) {
-            if (!full)
-                warn("additional field %s not marked with :full", line);
-            vector_add(list, line);
-        } else {
-            if (strcasecmp(line, fields[field]) != 0)
-                warn("field %d is %s, should be %s", field, line,
-                     fields[field]);
-        }
-        field++;
-    }
-    if (QIOerror(qp)) {
-        if (QIOtoolong(qp)) {
-            warn("line too long in %s", schema);
-        } else {
-            syswarn("error while reading %s", schema);
-        }
-    }
-    result = list;
 
-done:
-    if (schema != NULL)
-        free(schema);
-    if (qp != NULL)
-        QIOclose(qp);
-    if (result == NULL && list != NULL)
-        vector_free(list);
-    return result;
+    if (hidden) {
+        vector_resize(list, innconf->extraoverviewadvertised->count
+                            + innconf->extraoverviewhidden->count + 1);
+    } else {
+        vector_resize(list, innconf->extraoverviewadvertised->count + 1);
+    }
+
+    vector_add(list, "Xref");
+
+    if (innconf->extraoverviewadvertised->strings != NULL) {
+        for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
+            if (innconf->extraoverviewadvertised->strings[i] != NULL) {
+                vector_add(list, innconf->extraoverviewadvertised->strings[i]);
+            }
+        }
+    }
+
+    if (hidden) {
+        if (innconf->extraoverviewhidden->strings != NULL) {
+            for (i = 0; i < innconf->extraoverviewhidden->count; i++) {
+                if (innconf->extraoverviewhidden->strings[i] != NULL) {
+                    vector_add(list, innconf->extraoverviewhidden->strings[i]);
+                }
+            }
+        }
+    }
+
+    return list;
 }
 
 
@@ -200,12 +177,14 @@ overview_build(ARTNUM number, const char *article, size_t length,
         } else
             build_header(article, length, fields[field], overview);
     }
-    for (field = 0; field < extra->count; field++) {
-        buffer_append(overview, "\t", 1);
-        buffer_append(overview, extra->strings[field],
-                      strlen(extra->strings[field]));
-        buffer_append(overview, ": ", 2);
-        build_header(article, length, extra->strings[field], overview);
+    if (extra != NULL) {
+        for (field = 0; field < extra->count; field++) {
+            buffer_append(overview, "\t", 1);
+            buffer_append(overview, extra->strings[field],
+                          strlen(extra->strings[field]));
+            buffer_append(overview, ": ", 2);
+            build_header(article, length, extra->strings[field], overview);
+        }
     }
     buffer_append(overview, "\r\n", 2);
     return overview;
@@ -264,7 +243,7 @@ valid_overview_string(const char *string, bool full)
 
 /*
 **  Check the given overview data and make sure it's well-formed.  Extension
-**  headers are not checked against overview.fmt (having a different set of
+**  headers are not checked against LIST OVERVIEW.FMT (having a different set of
 **  extension headers doesn't make the data invalid), but the presence of the
 **  standard fields is checked.  Also checked is whether the article number in
 **  the data matches the passed article number.  Returns true if the data is
@@ -313,23 +292,21 @@ overview_check(const char *data, size_t length, ARTNUM article)
 
 
 /*
-**  Given an overview header, return the offset of the field within
-**  the overview data, or -1 if the field is not present in the
-**  overview schema for this installation.
+**  Given an overview header, return the offset of the field within the
+**  overview data, or -1 if the field is not present in the overview schema
+**  for this installation.
 */
 int
 overview_index(const char *field, const struct vector *extra)
 {
-    int i;
+    size_t i;
 
-    for (i = 0; i < (sizeof fields / sizeof fields[0]); ++i) {
+    for (i = 0; i < ARRAY_SIZE(fields); i++)
 	if (strcasecmp(field, fields[i]) == 0)
 	    return i;
-    }
-    for (i = 0; i < extra->count; i++) {
+    for (i = 0; i < extra->count; i++)
 	if (strcasecmp(field, extra->strings[i]) == 0)
-	    return i + (sizeof fields / sizeof fields[0]);
-    }
+	    return i + ARRAY_SIZE(fields);
     return -1;
 }
 
@@ -387,16 +364,15 @@ overview_split(const char *line, size_t length, ARTNUM *number,
 **  the member which the caller is interested in (and must free).
 */
 char *
-overview_getheader(const struct cvector *vector, int element,
+overview_getheader(const struct cvector *vector, unsigned int element,
 		   const struct vector *extra)
 {
     char *field = NULL;
     size_t len;
     const char *p;
+    size_t max = ARRAY_SIZE(fields) + extra->count - 1;
 
-    if ((element + 1) >= vector->count ||
-	(element >= ARRAY_SIZE(fields) &&
-	 (element - ARRAY_SIZE(fields)) >= extra->count)) {
+    if ((element + 1) >= vector->count || element > max) {
 	warn("request for invalid overview field %d", element);
 	return NULL;
     }
