@@ -1,4 +1,4 @@
-/*  $Id: timecaf.h,v 1.7 2001/01/04 06:01:36 rra Exp $
+/*  $Id: timecaf.h 6530 2003-10-21 00:31:33Z rra $
 **
 **  timecaf -- like the timehash storage method (and heavily inspired
 **  by it), but uses the CAF library to store multiple articles in a
@@ -14,7 +14,7 @@
 bool timecaf_init(SMATTRIBUTE *attr);
 TOKEN timecaf_store(const ARTHANDLE article, const STORAGECLASS class);
 ARTHANDLE *timecaf_retrieve(const TOKEN token, const RETRTYPE amount);
-ARTHANDLE *timecaf_next(const ARTHANDLE *article, const RETRTYPE amount);
+ARTHANDLE *timecaf_next(ARTHANDLE *article, const RETRTYPE amount);
 void timecaf_freearticle(ARTHANDLE *article);
 bool timecaf_cancel(TOKEN token);
 bool timecaf_ctl(PROBETYPE type, TOKEN *token, void *value);
