@@ -1,4 +1,4 @@
-/*  $Id: tradindexed.c 10283 2018-05-14 12:43:05Z iulius $
+/*  $Id: tradindexed.c 8333 2009-02-19 21:32:41Z iulius $
 **
 **  Interface implementation for the tradindexed overview method.
 **
@@ -82,7 +82,7 @@ data_cache_reopen(struct tradindexed *global, const char *group,
 bool
 tradindexed_open(int mode)
 {
-    unsigned long cache_size, fdlimit;
+    unsigned int cache_size, fdlimit;
 
     if (tradindexed != NULL) {
         warn("tradindexed: overview method already open");
@@ -99,8 +99,8 @@ tradindexed_open(int mode)
     fdlimit = getfdlimit();
     if (fdlimit > 0 && fdlimit < cache_size * 2) {
         warn("tradindexed: not enough file descriptors for an overview cache"
-             " size of %lu; increase rlimitnofile or decrease overcachesize"
-             " to at most %lu", cache_size, fdlimit / 2);
+             " size of %u; increase rlimitnofile or decrease overcachesize"
+             " to at most %u", cache_size, fdlimit / 2);
         cache_size = (fdlimit > 2) ? fdlimit / 2 : 1;
     }
     tradindexed->cache = tdx_cache_create(cache_size);
@@ -229,11 +229,6 @@ tradindexed_cancel(const char *group, ARTNUM artnum)
     data = data_cache_open(tradindexed, group, entry);
     if (data == NULL)
         return false;
-    if (artnum > data->high) {
-        data = data_cache_reopen(tradindexed, group, entry);
-        if (data == NULL)
-            return false;
-    }
     return tdx_data_cancel(data, artnum);
 }
 
@@ -365,7 +360,7 @@ tradindexed_expiregroup(const char *group, int *low, struct history *history)
 
 
 /*
-**  Set various options or query various parameters for the overview method.
+**  Set various options or query various paramaters for the overview method.
 **  The interface is, at present, not particularly sane.
 */
 bool
