@@ -1,27 +1,28 @@
-/*  $Id: tradspool.c,v 1.23.2.11 2003/02/02 21:42:55 rra Exp $
+/*  $Id: tradspool.c 7960 2008-08-24 09:22:42Z iulius $
 **
 **  Storage manager module for traditional spool format.
 */
+
 #include "config.h"
 #include "clibrary.h"
+#include "portable/mmap.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
-#include <syslog.h>
-#include <sys/mman.h>
+#include <fcntl.h>
 #include <sys/stat.h>
-
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
+#include <sys/uio.h>
+#include <time.h>
 
 /* Needed for htonl() and friends on AIX 4.1. */
 #include <netinet/in.h>
     
-#include "libinn.h"
-#include "macros.h"
-#include "paths.h"
-#include "qio.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/qio.h"
+#include "inn/wire.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 #include "methods.h"
 #include "tradspool.h"
@@ -29,11 +30,11 @@
 typedef struct {
     char		*artbase; /* start of the article data -- may be mmaped */
     unsigned int	artlen; /* art length. */
-    int		nextindex;
+    int 	nextindex;
     char		*curdirname;
     DIR			*curdir;
     struct _ngtent	*ngtp;
-    BOOL		mmapped;
+    bool 		mmapped;
 } PRIV_TRADSPOOL;
 
 /*
@@ -73,7 +74,7 @@ NGTENT *NGTable[NGT_SIZE];
 unsigned long MaxNgNumber = 0;
 NGTREENODE *NGTree;
 
-BOOL NGTableUpdated; /* set to TRUE if we've added any entries since reading 
+bool NGTableUpdated; /* set to true if we've added any entries since reading 
 			in the database file */
 
 /* 
@@ -104,10 +105,10 @@ HashNGName(char *ng) {
     HASHEDNG return_hash;
     char *p;
 
-    p = COPY(ng);
+    p = xstrdup(ng);
     DeDotify(p);
     hash = Hash(p, strlen(p));
-    DISPOSE(p);
+    free(p);
 
     memcpy(return_hash.hash, hash.hash, HASHEDNGLEN);
 
@@ -137,7 +138,7 @@ AddNG(char *ng, unsigned long number) {
     NGTENT *ngtp, **ngtpp;
     NGTREENODE *newnode, *curnode, **nextnode;
 
-    p = COPY(ng);
+    p = xstrdup(ng);
     DeDotify(p); /* canonicalize p to standard (/) form. */
     hash = HashNGName(p);
 
@@ -148,12 +149,12 @@ AddNG(char *ng, unsigned long number) {
 
     ngtp = NGTable[h];
     ngtpp = &NGTable[h];
-    while (TRUE) {
+    while (true) {
 	if (ngtp == NULL) {
 	    /* ng wasn't in table, add new entry. */
-	    NGTableUpdated = TRUE;
+	    NGTableUpdated = true;
 
-	    ngtp = NEW(NGTENT, 1);
+	    ngtp = xmalloc(sizeof(NGTENT));
 	    ngtp->ngname = p; /* note: we store canonicalized name */
 	    /* ngtp->hash = hash XXX */
 	    ngtp->next = NULL;
@@ -168,7 +169,7 @@ AddNG(char *ng, unsigned long number) {
 	    *ngtpp = ngtp;
 
 	    /* Now insert an appropriate record into the binary tree */
-	    newnode = NEW(NGTREENODE, 1);
+	    newnode = xmalloc(sizeof(NGTREENODE));
 	    newnode->left = newnode->right = (NGTREENODE *) NULL;
 	    newnode->ngnumber = number;
 	    newnode->ngtp = ngtp;
@@ -187,8 +188,10 @@ AddNG(char *ng, unsigned long number) {
 		    } else if (curnode->ngnumber > number) {
 			nextnode = &curnode->left;
 		    } else {
-			/* Error, same number is already in NGtree (shouldn't happen!) */
-			syslog(L_ERROR, "tradspool: AddNG: duplicate newsgroup number in NGtree: %lu(%s)", number, p);
+			/* Error, same number is already in NGtree (shouldn't
+                           happen!) */
+                        warn("tradspool: AddNG: duplicate newsgroup number in"
+                             " NGtree: %ld (%s)", number, p);
 			return;
 		    }
 		}
@@ -197,13 +200,13 @@ AddNG(char *ng, unsigned long number) {
 	    }
 	} else if (strcmp(ngtp->ngname, p) == 0) {
 	    /* entry in table already, so return */
-	    DISPOSE(p);
+	    free(p);
 	    return;
 #if 0 /* XXX */
 	} else if (CompareHash(&ngtp->hash, &hash) == 0) {
 	    /* eep! we hit a hash collision. */
-	    syslog(L_ERROR, "tradspool: AddNG: Hash collison %s/%s", ngtp->ngname, p);
-	    DISPOSE(p);
+            warn("tradspool: AddNG: hash collision %s/%s", ngtp->ngname, p);
+	    free(p);
 	    return;
 #endif 
 	} else {
@@ -215,14 +218,14 @@ AddNG(char *ng, unsigned long number) {
 }
 
 /* find a newsgroup table entry, given only the name. */
-NGTENT *
+static NGTENT *
 FindNGByName(char *ngname) {
     NGTENT *ngtp;
     unsigned int h;
     HASHEDNG hash;
     char *p;
 
-    p = COPY(ngname);
+    p = xstrdup(ngname);
     DeDotify(p); /* canonicalize p to standard (/) form. */
     hash = HashNGName(p);
 
@@ -235,12 +238,12 @@ FindNGByName(char *ngname) {
 
     while (ngtp) {
 	if (strcmp(p, ngtp->ngname) == 0) {
-	    DISPOSE(p);
+	    free(p);
 	    return ngtp;
 	}
 	ngtp = ngtp->next;
     }
-    DISPOSE(p);
+    free(p);
     return NULL; 
 }
 
@@ -272,8 +275,9 @@ FindNGByNum(unsigned long ngnumber) {
 
 
 /* dump DB to file. */
-void
-DumpDB(void) {
+static void
+DumpDB(void)
+{
     char *fname, *fnamenew;
     NGTENT *ngtp;
     unsigned int i;
@@ -282,13 +286,13 @@ DumpDB(void) {
     if (!SMopenmode) return; /* don't write if we're not in read/write mode. */
     if (!NGTableUpdated) return; /* no need to dump new DB */
 
-    fname = COPY(cpcatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB));
-    fnamenew = COPY(cpcatpath(innconf->pathspool, _PATH_NEWTSNGDB));
+    fname = concatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB);
+    fnamenew = concatpath(innconf->pathspool, _PATH_NEWTSNGDB);
 
     if ((out = fopen(fnamenew, "w")) == NULL) {
-	syslog(L_ERROR, "tradspool: DumpDB: can't write %s: %m", fnamenew);
-	DISPOSE(fname);
-	DISPOSE(fnamenew);
+        syswarn("tradspool: DumpDB: can't write %s", fnamenew);
+	free(fname);
+	free(fnamenew);
 	return;
     }
     for (i = 0 ; i < NGT_SIZE ; ++i) {
@@ -298,20 +302,20 @@ DumpDB(void) {
 	}
     }
     if (fclose(out) < 0) {
-	syslog(L_ERROR, "tradspool: DumpDB: can't close %s: %m", fnamenew);
-	DISPOSE(fname);
-	DISPOSE(fnamenew);
+        syswarn("tradspool: DumpDB: can't close %s", fnamenew);
+	free(fname);
+	free(fnamenew);
 	return;
     }
     if (rename(fnamenew, fname) < 0) {
-	syslog(L_ERROR, "tradspool: can't rename %s", fnamenew);
-	DISPOSE(fname);
-	DISPOSE(fnamenew);
+        syswarn("tradspool: DumpDB: can't rename %s", fnamenew);
+	free(fname);
+	free(fnamenew);
 	return;
     }
-    DISPOSE(fname);
-    DISPOSE(fnamenew);
-    NGTableUpdated = FALSE; /* reset modification flag. */
+    free(fname);
+    free(fnamenew);
+    NGTableUpdated = false; /* reset modification flag. */
     return;
 }
 
@@ -321,26 +325,27 @@ DumpDB(void) {
 ** of newsgroup number from there. 
 */
 
-BOOL
-ReadDBFile(void) {
+static bool
+ReadDBFile(void)
+{
     char *fname;
     QIOSTATE *qp;
     char *line;
     char *p;
     unsigned long number;
 
-    fname = COPY(cpcatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB));
+    fname = concatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB);
     if ((qp = QIOopen(fname)) == NULL) {
 	/* only warn if db not found. */
-	syslog(L_NOTICE, "tradspool: %s not found", fname);
+        notice("tradspool: mapping file %s not found", fname);
     } else {
 	while ((line = QIOread(qp)) != NULL) {
 	    p = strchr(line, ' ');
 	    if (p == NULL) {
-		syslog(L_FATAL, "tradspool: corrupt line in active %s", line);
+                warn("tradspool: corrupt line in active: %s", line);
 		QIOclose(qp);
-		DISPOSE(fname);
-		return FALSE;
+		free(fname);
+		return false;
 	    }
 	    *p++ = 0;
 	    number = atol(p);
@@ -349,54 +354,56 @@ ReadDBFile(void) {
 	}
 	QIOclose(qp);
     }
-    DISPOSE(fname);
-    return TRUE;
+    free(fname);
+    return true;
 }
 
-BOOL
-ReadActiveFile(void) {
+static bool
+ReadActiveFile(void)
+{
     char *fname;
     QIOSTATE *qp;
     char *line;
     char *p;
 
-    fname = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
+    fname = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     if ((qp = QIOopen(fname)) == NULL) {
-	syslog(L_FATAL, "tradspool: can't open %s", fname);
-	DISPOSE(fname);
-	return FALSE;
+        syswarn("tradspool: can't open %s", fname);
+	free(fname);
+	return false;
     }
 
     while ((line = QIOread(qp)) != NULL) {
 	p = strchr(line, ' ');
 	if (p == NULL) {
-	    syslog(L_FATAL, "tradspool: corrupt line in active %s", line);
+            syswarn("tradspool: corrupt line in active: %s", line);
 	    QIOclose(qp);
-	    DISPOSE(fname);
-	    return FALSE;
+	    free(fname);
+	    return false;
 	}
 	*p = 0;
 	AddNG(line, 0);
     }
     QIOclose(qp);
-    DISPOSE(fname);
+    free(fname);
     /* dump any newly added changes to database */
     DumpDB();
-    return TRUE;
+    return true;
 }
 
-BOOL
-InitNGTable(void) {
-    if (!ReadDBFile()) return FALSE;
+static bool
+InitNGTable(void)
+{
+    if (!ReadDBFile()) return false;
 
     /*
     ** set NGTableUpdated to false; that way we know if the load of active or
     ** any AddNGs later on did in fact add new entries to the db.
     */
-    NGTableUpdated = FALSE; 
+    NGTableUpdated = false; 
     if (!SMopenmode)
 	/* don't read active unless write mode. */
-	return TRUE;
+	return true;
     return ReadActiveFile(); 
 }
 
@@ -409,25 +416,27 @@ InitNGTable(void) {
 
 #define RELOAD_TIME_CHECK 600
 
-void
-CheckNeedReloadDB(BOOL force) {
-    static TIMEINFO lastcheck, oldlastcheck, now;
+static void
+CheckNeedReloadDB(bool force)
+{
+    static time_t lastcheck, oldlastcheck, now;
     struct stat sb;
     char *fname;
 
-    if (GetTimeInfo(&now) < 0) return; /* anyone ever seen gettimeofday fail? :-) */
-    if (!force && lastcheck.time + RELOAD_TIME_CHECK > now.time) return;
+    now = time(NULL);
+    if (!force && lastcheck + RELOAD_TIME_CHECK > now)
+        return;
 
     oldlastcheck = lastcheck;
     lastcheck = now;
 
-    fname = COPY(cpcatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB));
+    fname = concatpath(innconf->pathspool, _PATH_TRADSPOOLNGDB);
     if (stat(fname, &sb) < 0) {
-	DISPOSE(fname);
+	free(fname);
 	return;
     }
-    DISPOSE(fname);
-    if (sb.st_mtime > oldlastcheck.time) {
+    free(fname);
+    if (sb.st_mtime > oldlastcheck) {
 	/* add any newly added ngs to our in-memory copy of the db. */
 	ReadDBFile();
     }
@@ -435,15 +444,20 @@ CheckNeedReloadDB(BOOL force) {
 
 /* Init routine, called by SMinit */
 
-BOOL
+bool
 tradspool_init(SMATTRIBUTE *attr) {
     if (attr == NULL) {
-	syslog(L_ERROR, "tradspool: attr is NULL");
+        warn("tradspool: attr is NULL");
 	SMseterror(SMERR_INTERNAL, "attr is NULL");
-	return FALSE;
+	return false;
     }
-    attr->selfexpire = FALSE;
-    attr->expensivestat = TRUE;
+    if (!innconf->storeonxref) {
+        warn("tradspool: storeonxref needs to be true");
+	SMseterror(SMERR_INTERNAL, "storeonxref needs to be true");
+	return false;
+    }
+    attr->selfexpire = false;
+    attr->expensivestat = true;
     return InitNGTable();
 }
 
@@ -486,8 +500,9 @@ TokenToPath(TOKEN token) {
     unsigned long ngnum;
     unsigned long artnum;
     char *ng, *path;
+    size_t length;
 
-    CheckNeedReloadDB(FALSE);
+    CheckNeedReloadDB(false);
 
     memcpy(&ngnum, &token.token[0], sizeof(ngnum));
     memcpy(&artnum, &token.token[sizeof(ngnum)], sizeof(artnum));
@@ -496,14 +511,15 @@ TokenToPath(TOKEN token) {
 
     ng = FindNGByNum(ngnum);
     if (ng == NULL) {
-	CheckNeedReloadDB(TRUE);
+	CheckNeedReloadDB(true);
 	ng = FindNGByNum(ngnum);
 	if (ng == NULL)
 	    return NULL;
     }
 
-    path = NEW(char, strlen(ng)+20+strlen(innconf->patharticles));
-    sprintf(path, "%s/%s/%lu", innconf->patharticles, ng, artnum);
+    length = strlen(ng) + 20 + strlen(innconf->patharticles);
+    path = xmalloc(length);
+    snprintf(path, length, "%s/%s/%lu", innconf->patharticles, ng, artnum);
     return path;
 }
 
@@ -517,20 +533,14 @@ CrackXref(char *xref, unsigned int *lenp) {
     char **xrefs;
     char *q;
     unsigned int len, xrefsize;
-    unsigned int slen;
 
     len = 0;
     xrefsize = 5;
-    xrefs = NEW(char *, xrefsize);
+    xrefs = xmalloc(xrefsize * sizeof(char *));
 
-    /* skip pathhost */
-    if ((p = strchr(xref, ' ')) == NULL) {
-	SMseterror(SMERR_UNDEFINED, "Could not find pathhost in Xref header");
-	return NULL;
-    }
-    /* skip next spaces */
-    for (p++; *p == ' ' ; p++) ;
-    while (TRUE) {
+    /* no path element should exist, nor heading white spaces exist */
+    p = xref;
+    while (true) {
 	/* check for EOL */
 	/* shouldn't ever hit null w/o hitting a \r\n first, but best to be paranoid */
 	if (*p == '\n' || *p == '\r' || *p == 0) {
@@ -541,18 +551,15 @@ CrackXref(char *xref, unsigned int *lenp) {
 	/* skip to next space or EOL */
 	for (q=p; *q && *q != ' ' && *q != '\n' && *q != '\r' ; ++q) ;
 
-	slen = q-p;
-	xrefs[len] = NEW(char, slen+1);
-	strncpy(xrefs[len], p, slen);
-	xrefs[len][slen] = '\0';
+        xrefs[len] = xstrndup(p, q - p);
 
 	if (++len == xrefsize) {
 	    /* grow xrefs if needed. */
 	    xrefsize *= 2;
-	    RENEW(xrefs, char *, xrefsize);
+            xrefs = xrealloc(xrefs, xrefsize * sizeof(char *));
 	}
 
-	p = q;
+ 	p = q;
 	/* skip spaces */
 	for ( ; *p == ' ' ; p++) ;
     }
@@ -564,34 +571,30 @@ tradspool_store(const ARTHANDLE article, const STORAGECLASS class) {
     char *xrefhdr;
     TOKEN token;
     unsigned int numxrefs;
-    char *ng, *p;
+    char *ng, *p, *onebuffer;
     unsigned long artnum;
     char *path, *linkpath, *dirname;
     int fd;
-    int i;
+    size_t used;
     char *nonwfarticle; /* copy of article converted to non-wire format */
-    int nonwflen;
-    
-    xrefhdr = (char *)HeaderFindMem(article.data, article.len, "Xref", 4);
-    if (xrefhdr == NULL) {
-	token.type = TOKEN_EMPTY;
-	SMseterror(SMERR_UNDEFINED, "cant find Xref: header");
-	return token;
-    }
+    unsigned int i;
+    size_t length, nonwflen;
 
+    memset(&token, 0, sizeof(token)); 
+    xrefhdr = article.groups;
     if ((xrefs = CrackXref(xrefhdr, &numxrefs)) == NULL || numxrefs == 0) {
 	token.type = TOKEN_EMPTY;
 	SMseterror(SMERR_UNDEFINED, "bogus Xref: header");
 	if (xrefs != NULL)
-	    DISPOSE(xrefs);
+	    free(xrefs);
 	return token;
     }
 
     if ((p = strchr(xrefs[0], ':')) == NULL) {
 	token.type = TOKEN_EMPTY;
 	SMseterror(SMERR_UNDEFINED, "bogus Xref: header");
-	for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-	DISPOSE(xrefs);
+	for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+	free(xrefs);
 	return token;
     }
     *p++ = '\0';
@@ -601,61 +604,68 @@ tradspool_store(const ARTHANDLE article, const STORAGECLASS class) {
     
     token = MakeToken(ng, artnum, class);
 
-    path = NEW(char, strlen(innconf->patharticles) + strlen(ng) + 32);
-    sprintf(path, "%s/%s/%lu", innconf->patharticles, ng, artnum);
+    length = strlen(innconf->patharticles) + strlen(ng) + 32;
+    path = xmalloc(length);
+    snprintf(path, length, "%s/%s/%lu", innconf->patharticles, ng, artnum);
 
     /* following chunk of code boldly stolen from timehash.c  :-) */
     if ((fd = open(path, O_CREAT|O_EXCL|O_WRONLY, ARTFILE_MODE)) < 0) {
 	p = strrchr(path, '/');
 	*p = '\0';
-	if (!MakeDirectory(path, TRUE)) {
-	    syslog(L_ERROR, "tradspool: could not make directory %s %m", path);
+	if (!MakeDirectory(path, true)) {
+            syswarn("tradspool: could not create directory %s", path);
 	    token.type = TOKEN_EMPTY;
-	    DISPOSE(path);
+	    free(path);
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-	    DISPOSE(xrefs);
+	    for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+	    free(xrefs);
 	    return token;
 	} else {
 	    *p = '/';
 	    if ((fd = open(path, O_CREAT|O_EXCL|O_WRONLY, ARTFILE_MODE)) < 0) {
 		SMseterror(SMERR_UNDEFINED, NULL);
-		syslog(L_ERROR, "tradspool: could not open %s %m", path);
+                syswarn("tradspool: could not open %s", path);
 		token.type = TOKEN_EMPTY;
-		DISPOSE(path);
-		for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-		DISPOSE(xrefs);
+		free(path);
+		for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+		free(xrefs);
 		return token;
 	    }
 	}
     }
     if (innconf->wireformat) {
-	if (write(fd, article.data, article.len) != article.len) {
+	if (xwritev(fd, article.iov, article.iovcnt) != (ssize_t) article.len) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool error writing %s %m", path);
+            syswarn("tradspool: error writing to %s", path);
 	    close(fd);
 	    token.type = TOKEN_EMPTY;
 	    unlink(path);
-	    DISPOSE(path);
-	    for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-	    DISPOSE(xrefs);
+	    free(path);
+	    for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+	    free(xrefs);
 	    return token;
 	}
     } else {
-	nonwfarticle = FromWireFmt(article.data, article.len, &nonwflen);
-	if (write(fd, nonwfarticle, nonwflen) != nonwflen) {
-	    DISPOSE(nonwfarticle);
+	onebuffer = xmalloc(article.len);
+	for (used = i = 0 ; i < (unsigned int) article.iovcnt ; i++) {
+	    memcpy(&onebuffer[used], article.iov[i].iov_base, article.iov[i].iov_len);
+	    used += article.iov[i].iov_len;
+	}
+	nonwfarticle = wire_to_native(onebuffer, used, &nonwflen);
+	free(onebuffer);
+	if (write(fd, nonwfarticle, nonwflen) != (ssize_t) nonwflen) {
+	    free(nonwfarticle);
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool error writing %s %m", path);
+            syswarn("tradspool: error writing to %s", path);
 	    close(fd);
 	    token.type = TOKEN_EMPTY;
 	    unlink(path);
-	    DISPOSE(path);
-	    for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-	    DISPOSE(xrefs);
+	    free(path);
+	    for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+	    free(xrefs);
 	    return token;
 	}
-	DISPOSE(nonwfarticle);
+	free(nonwfarticle);
     }
     close(fd);
 
@@ -672,46 +682,37 @@ tradspool_store(const ARTHANDLE article, const STORAGECLASS class) {
 	    DeDotify(ng);
 	    artnum = atol(p);
 
-	    linkpath = NEW(char, strlen(innconf->patharticles) + strlen(ng) + 32);
-	    sprintf(linkpath, "%s/%s/%lu", innconf->patharticles, ng, artnum);
+            length = strlen(innconf->patharticles) + strlen(ng) + 32;
+	    linkpath = xmalloc(length);
+	    snprintf(linkpath, length, "%s/%s/%lu", innconf->patharticles,
+                     ng, artnum);
 	    if (link(path, linkpath) < 0) {
 		p = strrchr(linkpath, '/');
 		*p = '\0';
-		dirname = COPY(linkpath);
+		dirname = xstrdup(linkpath);
 		*p = '/';
-		if (!MakeDirectory(dirname, TRUE) || link(path, linkpath) < 0) {
-#if !defined(HAVE_SYMLINK)
-		    SMseterror(SMERR_UNDEFINED, NULL);
-		    syslog(L_ERROR, "tradspool: could not link %s to %s %m", path, linkpath);
-		    token.type = TOKEN_EMPTY;
-		    DISPOSE(dirname);
-		    DISPOSE(linkpath);
-		    DISPOSE(path);
-		    for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-		    DISPOSE(xrefs);
-		    return token;
-#else
+		if (!MakeDirectory(dirname, true) || link(path, linkpath) < 0) {
 		    if (symlink(path, linkpath) < 0) {
 			SMseterror(SMERR_UNDEFINED, NULL);
-			syslog(L_ERROR, "tradspool: could not symlink %s to %s %m", path, linkpath);
+                        syswarn("tradspool: could not symlink %s to %s",
+                                path, linkpath);
 			token.type = TOKEN_EMPTY;
-			DISPOSE(dirname);
-			DISPOSE(linkpath);
-			DISPOSE(path);
-			for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-			DISPOSE(xrefs);
+			free(dirname);
+			free(linkpath);
+			free(path);
+			for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+			free(xrefs);
 			return token;
 		    }
-#endif  /* !defined(HAVE_SYMLINK) */
 		}
-		DISPOSE(dirname);
+		free(dirname);
 	    }
-	    DISPOSE(linkpath);
+	    free(linkpath);
 	}
     }
-    DISPOSE(path);
-    for (i = 0 ; i < numxrefs; ++i) DISPOSE(xrefs[i]);
-    DISPOSE(xrefs);
+    free(path);
+    for (i = 0 ; i < numxrefs; ++i) free(xrefs[i]);
+    free(xrefs);
     return token;
 }
 
@@ -723,14 +724,14 @@ OpenArticle(const char *path, RETRTYPE amount) {
     struct stat sb;
     ARTHANDLE *art;
     char *wfarticle;
-    int wflen;
+    size_t wflen;
 
     if (amount == RETR_STAT) {
 	if (access(path, R_OK) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
 	    return NULL;
 	}
-	art = NEW(ARTHANDLE, 1);
+	art = xmalloc(sizeof(ARTHANDLE));
 	art->type = TOKEN_TRADSPOOL;
 	art->data = NULL;
 	art->len = 0;
@@ -743,82 +744,83 @@ OpenArticle(const char *path, RETRTYPE amount) {
 	return NULL;
     }
 
-    art = NEW(ARTHANDLE, 1);
+    art = xmalloc(sizeof(ARTHANDLE));
     art->type = TOKEN_TRADSPOOL;
 
     if (fstat(fd, &sb) < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	syslog(L_ERROR, "tradspool: could not fstat article: %m");
-	DISPOSE(art);
+        syswarn("tradspool: could not fstat article %s", path);
+	free(art);
 	close(fd);
 	return NULL;
     }
 
     art->arrived = sb.st_mtime;
 
-    private = NEW(PRIV_TRADSPOOL, 1);
+    private = xmalloc(sizeof(PRIV_TRADSPOOL));
     art->private = (void *)private;
     private->artlen = sb.st_size;
     if (innconf->articlemmap) {
-	if ((private->artbase = mmap((MMAP_PTR)0, sb.st_size, PROT_READ, MAP__ARG, fd, 0)) == (MMAP_PTR)-1) {
+	if ((private->artbase = mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool: could not mmap article: %m");
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+            syswarn("tradspool: could not mmap article %s", path);
+	    free(art->private);
+	    free(art);
 	    close(fd);
 	    return NULL;
 	}
+        if (amount == RETR_ALL)
+            madvise(private->artbase, sb.st_size, MADV_WILLNEED);
+        else
+            madvise(private->artbase, sb.st_size, MADV_SEQUENTIAL);
+
 	/* consider coexisting both wireformatted and nonwireformatted */
 	p = memchr(private->artbase, '\n', private->artlen);
 	if (p == NULL || p == private->artbase) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool: could not mmap article: %m");
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(private->artbase, private->artlen, MADV_DONTNEED);
-#endif
+            syswarn("tradspool: apparently corrupt article %s", path);
 	    munmap(private->artbase, private->artlen);
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+	    free(art->private);
+	    free(art);
 	    close(fd);
 	    return NULL;
 	}
 	if (p[-1] == '\r') {
-	    private->mmapped = TRUE;
+	    private->mmapped = true;
 	} else {
-	    wfarticle = ToWireFmt(private->artbase, private->artlen, &wflen);
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(private->artbase, private->artlen, MADV_DONTNEED);
-#endif
+	    wfarticle = wire_from_native(private->artbase, private->artlen,
+                                         &wflen);
 	    munmap(private->artbase, private->artlen);
 	    private->artbase = wfarticle;
 	    private->artlen = wflen;
-	    private->mmapped = FALSE;
+	    private->mmapped = false;
 	}
     } else {
-	private->mmapped = FALSE;
-	private->artbase = NEW(char, private->artlen);
+	private->mmapped = false;
+	private->artbase = xmalloc(private->artlen);
 	if (read(fd, private->artbase, private->artlen) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool: could not read article: %m");
-	    DISPOSE(private->artbase);
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+            syswarn("tradspool: could not read article %s", path);
+	    free(private->artbase);
+	    free(art->private);
+	    free(art);
 	    close(fd);
 	    return NULL;
 	}
 	p = memchr(private->artbase, '\n', private->artlen);
 	if (p == NULL || p == private->artbase) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    syslog(L_ERROR, "tradspool: could not mmap article: %m");
-	    DISPOSE(art->private);
-	    DISPOSE(art);
+            syswarn("tradspool: apparently corrupt article %s", path);
+	    free(art->private);
+	    free(art);
 	    close(fd);
 	    return NULL;
 	}
 	if (p[-1] != '\r') {
 	    /* need to make a wireformat copy of the article */
-	    wfarticle = ToWireFmt(private->artbase, private->artlen, &wflen);
-	    DISPOSE(private->artbase);
+	    wfarticle = wire_from_native(private->artbase, private->artlen,
+                                         &wflen);
+	    free(private->artbase);
 	    private->artbase = wfarticle;
 	    private->artlen = wflen;
 	}
@@ -836,24 +838,22 @@ OpenArticle(const char *path, RETRTYPE amount) {
 	return art;
     }
     
-    if (((p = SMFindBody(private->artbase, private->artlen)) == NULL)) {
-	if (private->mmapped) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(private->artbase, private->artlen, MADV_DONTNEED);
-#endif
+    if (((p = wire_findbody(private->artbase, private->artlen)) == NULL)) {
+	if (private->mmapped)
 	    munmap(private->artbase, private->artlen);
-	} else {
-	    DISPOSE(private->artbase);
-	}
+	else
+	    free(private->artbase);
 	SMseterror(SMERR_NOBODY, NULL);
-	DISPOSE(art->private);
-	DISPOSE(art);
+	free(art->private);
+	free(art);
 	return NULL;
     }
 
     if (amount == RETR_HEAD) {
 	art->data = private->artbase;
 	art->len = p - private->artbase;
+        /* Headers end just before the first empty line (\r\n). */
+        art->len = art->len - 2;
 	return art;
     }
 
@@ -863,16 +863,12 @@ OpenArticle(const char *path, RETRTYPE amount) {
 	return art;
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
-    if (private->mmapped) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	madvise(private->artbase, private->artlen, MADV_DONTNEED);
-#endif
+    if (private->mmapped)
 	munmap(private->artbase, private->artlen);
-    } else {
-	DISPOSE(private->artbase);
-    }
-    DISPOSE(art->private);
-    DISPOSE(art);
+    else
+	free(private->artbase);
+    free(art->private);
+    free(art);
     return NULL;
 }
 
@@ -896,36 +892,33 @@ tradspool_retrieve(const TOKEN token, const RETRTYPE amount) {
         ret_token = token;
         art->token = &ret_token;
     }
-    DISPOSE(path);
+    free(path);
     return art;
 }
 
 void
-tradspool_freearticle(ARTHANDLE *article) {
+tradspool_freearticle(ARTHANDLE *article)
+{
     PRIV_TRADSPOOL *private;
 
-    if (!article) return;
+    if (article == NULL)
+        return;
 
     if (article->private) {
 	private = (PRIV_TRADSPOOL *) article->private;
-	if (private->mmapped) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-	    madvise(private->artbase, private->artlen, MADV_DONTNEED);
-#endif
+	if (private->mmapped)
 	    munmap(private->artbase, private->artlen);
-	} else {
-	    DISPOSE(private->artbase);
-	}
-	if (private->curdir) {
+	else
+	    free(private->artbase);
+	if (private->curdir)
 	    closedir(private->curdir);
-	}
-	DISPOSE(private->curdirname);
-        DISPOSE(private);
+	free(private->curdirname);
+        free(private);
     }
-    DISPOSE(article);
+    free(article);
 }
 
-BOOL 
+bool 
 tradspool_cancel(TOKEN token) {
     char **xrefs;
     char *xrefhdr;
@@ -933,14 +926,15 @@ tradspool_cancel(TOKEN token) {
     unsigned int numxrefs;
     char *ng, *p;
     char *path, *linkpath;
-    int i;
-    BOOL result = TRUE;
+    unsigned int i;
+    bool result = true;
     unsigned long artnum;
+    size_t length;
 
     if ((path = TokenToPath(token)) == NULL) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	DISPOSE(path);
-	return FALSE;
+	free(path);
+	return false;
     }
     /*
     **  Ooooh, this is gross.  To find the symlinks pointing to this article,
@@ -949,26 +943,28 @@ tradspool_cancel(TOKEN token) {
     ** good things for performance of fastrm...  -- rmtodd
     */
     if ((article = OpenArticle(path, RETR_HEAD)) == NULL) {
-	DISPOSE(path);
+	free(path);
 	SMseterror(SMERR_UNDEFINED, NULL);
-        return FALSE;
+        return false;
     }
 
-    xrefhdr = (char *)HeaderFindMem(article->data, article->len, "Xref", 4);
+    xrefhdr = wire_findheader(article->data, article->len, "Xref");
     if (xrefhdr == NULL) {
 	/* for backwards compatibility; there is no Xref unless crossposted
 	   for 1.4 and 1.5 */
-	if (unlink(path) < 0) result = FALSE;
-	DISPOSE(path);
+	if (unlink(path) < 0) result = false;
+	free(path);
 	tradspool_freearticle(article);
         return result;
     }
 
     if ((xrefs = CrackXref(xrefhdr, &numxrefs)) == NULL || numxrefs == 0) {
-	DISPOSE(path);
+        if (xrefs != NULL)
+            free(xrefs);
+	free(path);
 	tradspool_freearticle(article);
         SMseterror(SMERR_UNDEFINED, NULL);
-        return FALSE;
+        return false;
     }
 
     tradspool_freearticle(article);
@@ -979,21 +975,23 @@ tradspool_cancel(TOKEN token) {
 	DeDotify(ng);
 	artnum = atol(p);
 
-	linkpath = NEW(char, strlen(innconf->patharticles) + strlen(ng) + 32);
-	sprintf(linkpath, "%s/%s/%lu", innconf->patharticles, ng, artnum);
+        length = strlen(innconf->patharticles) + strlen(ng) + 32;
+	linkpath = xmalloc(length);
+	snprintf(linkpath, length, "%s/%s/%lu", innconf->patharticles, ng,
+                 artnum);
 	/* repeated unlinkings of a crossposted article may fail on account
 	   of the file no longer existing without it truly being an error */
 	if (unlink(linkpath) < 0)
 	    if (errno != ENOENT || i == 1)
-		result = FALSE;
-	DISPOSE(linkpath);
+		result = false;
+	free(linkpath);
     }
     if (unlink(path) < 0)
-	if (errno != ENOENT || numxrefs == 1)
-	    result = FALSE;
-    DISPOSE(path);
-    for (i = 0 ; i < numxrefs ; ++i) DISPOSE(xrefs[i]);
-    DISPOSE(xrefs);
+    	if (errno != ENOENT || numxrefs == 1)
+	    result = false;
+    free(path);
+    for (i = 0 ; i < numxrefs ; ++i) free(xrefs[i]);
+    free(xrefs);
     return result;
 }
 
@@ -1008,52 +1006,55 @@ static struct dirent *
 FindDir(DIR *dir, char *dirname) {
     struct dirent *de;
     int i;
-    BOOL flag;
+    bool flag;
     char *path;
     struct stat sb;
+    size_t length;
     unsigned char namelen;
 
     while ((de = readdir(dir)) != NULL) {
 	namelen = strlen(de->d_name);
-	for (i = 0, flag = TRUE ; i < namelen ; ++i) {
-	    if (!isdigit(de->d_name[i])) {
-		flag = FALSE;
+	for (i = 0, flag = true ; i < namelen ; ++i) {
+	    if (!CTYPE(isdigit, de->d_name[i])) {
+		flag = false;
 		break;
 	    }
 	}
 	if (!flag) continue; /* if not all digits, skip this entry. */
 
-	path = NEW(char, strlen(dirname)+namelen+2);
-	strcpy(path, dirname);
-	strcat(path, "/");
-	strncpy(&path[strlen(dirname)+1], de->d_name, namelen);
-	path[strlen(dirname)+namelen+1] = '\0';
+        length = strlen(dirname) + namelen + 2;
+	path = xmalloc(length);
+        strlcpy(path, dirname, length);
+        strlcat(path, "/", length);
+        strlcat(path, de->d_name, length);
 
 	if (lstat(path, &sb) < 0) {
-	    DISPOSE(path);
+	    free(path);
 	    continue;
 	}
-	DISPOSE(path);
+	free(path);
 	if (!S_ISREG(sb.st_mode)) continue;
 	return de;
     }
     return NULL;
 }
 
-ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
+ARTHANDLE *
+tradspool_next(ARTHANDLE *article, const RETRTYPE amount)
+{
     PRIV_TRADSPOOL priv;
     PRIV_TRADSPOOL *newpriv;
     char *path, *linkpath;
     struct dirent *de;
     ARTHANDLE *art;
     unsigned long artnum;
-    int i;
+    unsigned int i;
     static TOKEN token;
-    unsigned char namelen;
     char **xrefs;
-    char *xrefhdr, *ng, *p;
+    char *xrefhdr, *ng, *p, *expires, *x;
     unsigned int numxrefs;
     STORAGE_SUB	*sub;
+    size_t length;
 
     if (article == NULL) {
 	priv.ngtp = NULL;
@@ -1062,17 +1063,13 @@ ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
 	priv.nextindex = -1;
     } else {
 	priv = *(PRIV_TRADSPOOL *) article->private;
-	DISPOSE(article->private);
-	DISPOSE((void*)article);
+	free(article->private);
+	free(article);
 	if (priv.artbase != NULL) {
-	    if (priv.mmapped) {
-#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
-		madvise(priv.artbase, priv.artlen, MADV_DONTNEED);
-#endif
+	    if (priv.mmapped)
 		munmap(priv.artbase, priv.artlen);
-	    } else {
-		DISPOSE(priv.artbase);
-	    }
+	    else
+		free(priv.artbase);
 	}
     }
 
@@ -1080,7 +1077,7 @@ ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
 	if (priv.curdir) {
 	    closedir(priv.curdir);
 	    priv.curdir = NULL;
-	    DISPOSE(priv.curdirname);
+	    free(priv.curdirname);
 	    priv.curdirname = NULL;
 	}
 
@@ -1103,28 +1100,25 @@ ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
 		break;
 	}
 
-	priv.curdirname = NEW(char, strlen(innconf->patharticles)+strlen(priv.ngtp->ngname)+2);
-	sprintf(priv.curdirname, "%s/%s",innconf->patharticles,priv.ngtp->ngname);
+        priv.curdirname = concatpath(innconf->patharticles, priv.ngtp->ngname);
 	priv.curdir = opendir(priv.curdirname);
     }
 
-    namelen = strlen(de->d_name);
-    path = NEW(char, strlen(priv.curdirname) + 2 + namelen);
-    strcpy(path, priv.curdirname);
-    strcat(path, "/");
+    path = concatpath(priv.curdirname, de->d_name);
     i = strlen(priv.curdirname);
-    strncpy(&path[i+1], de->d_name, namelen);
-    path[i+namelen+1] = '\0';
     /* get the article number while we're here, we'll need it later. */
     artnum = atol(&path[i+1]);
 
     art = OpenArticle(path, amount);
     if (art == (ARTHANDLE *)NULL) {
-	art = NEW(ARTHANDLE, 1);
+	art = xmalloc(sizeof(ARTHANDLE));
 	art->type = TOKEN_TRADSPOOL;
 	art->data = NULL;
 	art->len = 0;
-	art->private = (void *)NEW(PRIV_TRADSPOOL, 1);
+	art->private = xmalloc(sizeof(PRIV_TRADSPOOL));
+	art->expires = 0;
+	art->groups = NULL;
+	art->groupslen = 0;
 	newpriv = (PRIV_TRADSPOOL *) art->private;
 	newpriv->artbase = NULL;
     } else {
@@ -1143,30 +1137,73 @@ ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
            implications that might have for the callers of SMnext.
 
            Basically, this whole area really needs to be rethought. */
-	xrefhdr = (char *)HeaderFindMem(art->data, art->len, "Xref", 4);
+	xrefhdr = wire_findheader(art->data, art->len, "Xref");
 	if (xrefhdr != NULL) {
 	    if ((xrefs = CrackXref(xrefhdr, &numxrefs)) == NULL || numxrefs == 0) {
 		art->len = 0;
 	    } else {
 		/* assumes first one is the original */
-		if ((p = strchr(xrefs[0], ':')) != NULL) {
+		if ((p = strchr(xrefs[1], ':')) != NULL) {
 		    *p++ = '\0';
-		    ng = xrefs[0];
+		    ng = xrefs[1];
 		    DeDotify(ng);
 		    artnum = atol(p);
 
-		    linkpath = NEW(char, strlen(innconf->patharticles) + strlen(ng) + 32);
-		    sprintf(linkpath, "%s/%s/%lu", innconf->patharticles, ng, artnum);
+                    length = strlen(innconf->patharticles) + strlen(ng) + 32;
+		    linkpath = xmalloc(length);
+		    snprintf(linkpath, length, "%s/%s/%lu",
+                             innconf->patharticles, ng, artnum);
 		    if (strcmp(path, linkpath) != 0) {
 			/* this is linked article, skip it */
 			art->len = 0;
 		    }
-		    DISPOSE(linkpath);
+		    free(linkpath);
+		}
+	    }
+	    for (i = 0 ; i < numxrefs ; ++i) free(xrefs[i]);
+	    free(xrefs);
+	    if (innconf->storeonxref) {
+		/* skip path element */
+		if ((xrefhdr = strchr(xrefhdr, ' ')) == NULL) {
+		    art->groups = NULL;
+		    art->groupslen = 0;
+		} else {
+		    for (xrefhdr++; *xrefhdr == ' '; xrefhdr++);
+		    art->groups = xrefhdr;
+		    for (p = xrefhdr ; (*p != '\n') && (*p != '\r') ; p++);
+		    art->groupslen = p - xrefhdr;
 		}
 	    }
-	    for (i = 0 ; i < numxrefs ; ++i) DISPOSE(xrefs[i]);
-	    DISPOSE(xrefs);
 	}
+	if (xrefhdr == NULL || !innconf->storeonxref) {
+            ng = wire_findheader(art->data, art->len, "Newsgroups");
+	    if (ng == NULL) {
+		art->groups = NULL;
+		art->groupslen = 0;
+	    } else {
+		art->groups = ng;
+		for (p = ng ; (*p != '\n') && (*p != '\r') ; p++);
+		art->groupslen = p - ng;
+	    }
+	}
+        expires = wire_findheader(art->data, art->len, "Expires");
+        if (expires == NULL) {
+	    art->expires = 0;
+	} else {
+            /* optionally parse expire header */
+            for (p = expires + 1; (*p != '\n') && (*(p - 1) != '\r'); p++)
+                ;
+            x = xmalloc(p - expires);
+            memcpy(x, expires, p - expires - 1);
+            x[p - expires - 1] = '\0';
+
+            art->expires = parsedate_rfc2822_lax(x);
+            if (art->expires == (time_t) -1)
+                art->expires = 0;
+            else
+                art->expires -= time(NULL);
+            free(x);
+        }
 	/* for backwards compatibility; assumes no Xref unless crossposted
 	   for 1.4 and 1.5: just fall through */
     }
@@ -1184,18 +1221,19 @@ ARTHANDLE *tradspool_next(const ARTHANDLE *article, const RETRTYPE amount) {
            all the ones skipped via the hard-link skipping algorithm
            commented above. */
         if (art->len > 0)
-            syslog(L_ERROR, "tradspool: can't determine class: %s: %s",
-                   TokenToText(token), SMerrorstr);
+            warn("tradspool: can't determine class of %s: %s",
+                 TokenToText(token), SMerrorstr);
     } else {
 	token = MakeToken(priv.ngtp->ngname, artnum, sub->class);
     }
     art->token = &token;
-    DISPOSE(path);
+    free(path);
     return art;
 }
 
-void
-FreeNGTree(void) {
+static void
+FreeNGTree(void)
+{
     unsigned int i;
     NGTENT *ngtp, *nextngtp;
 
@@ -1203,9 +1241,9 @@ FreeNGTree(void) {
         ngtp = NGTable[i];
         for ( ; ngtp != NULL ; ngtp = nextngtp) {
 	    nextngtp = ngtp->next;
-	    DISPOSE(ngtp->ngname);
-	    DISPOSE(ngtp->node);
-	    DISPOSE(ngtp);
+	    free(ngtp->ngname);
+	    free(ngtp->node);
+	    free(ngtp);
 	}
 	NGTable[i] = NULL;
     }
@@ -1213,46 +1251,53 @@ FreeNGTree(void) {
     NGTree = NULL;
 }
 
-BOOL tradspool_ctl(PROBETYPE type, TOKEN *token, void *value) {
+bool tradspool_ctl(PROBETYPE type, TOKEN *token, void *value) {
     struct artngnum *ann;
     unsigned long ngnum;
     unsigned long artnum;
-    char *ng;
+    char *ng, *p;
 
     switch (type) { 
     case SMARTNGNUM:
 	if ((ann = (struct artngnum *)value) == NULL)
-	    return FALSE;
-	CheckNeedReloadDB(FALSE);
+	    return false;
+	CheckNeedReloadDB(false);
 	memcpy(&ngnum, &token->token[0], sizeof(ngnum));
 	memcpy(&artnum, &token->token[sizeof(ngnum)], sizeof(artnum));
 	artnum = ntohl(artnum);
 	ngnum = ntohl(ngnum);
 	ng = FindNGByNum(ngnum);
 	if (ng == NULL) {
-	    CheckNeedReloadDB(TRUE);
+	    CheckNeedReloadDB(true);
 	    ng = FindNGByNum(ngnum);
 	    if (ng == NULL)
-		return FALSE;
+		return false;
 	}
-	ann->groupname = COPY(ng);
+	ann->groupname = xstrdup(ng);
+        for (p = ann->groupname; *p != 0; p++)
+            if (*p == '/')
+                *p = '.';
 	ann->artnum = (ARTNUM)artnum;
-	return TRUE;
+	return true;
     default:
-	return FALSE;
+	return false;
     }       
 }
 
-BOOL tradspool_flushcacheddata(FLUSHTYPE type) {
-    return TRUE;
+bool
+tradspool_flushcacheddata(FLUSHTYPE type UNUSED)
+{
+    return true;
 }
 
-void tradspool_printfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
+void
+tradspool_printfiles(FILE *file, TOKEN token UNUSED, char **xref, int ngroups)
+{
     int i;
     char *path, *p;
 
     for (i = 0; i < ngroups; i++) {
-        path = COPY(xref[i]);
+        path = xstrdup(xref[i]);
         for (p = path; *p != '\0'; p++)
             if (*p == '.' || *p == ':')
                 *p = '/';
