@@ -1,4 +1,4 @@
-/* $Id: date-t.c 10465 2020-12-24 22:26:47Z eagle $ */
+/* $Id: date-t.c 7585 2006-11-21 09:37:51Z eagle $ */
 /* makedate test suite */
 
 #include "config.h"
@@ -6,8 +6,7 @@
 #include <time.h>
 
 #include "inn/libinn.h"
-#include "tap/basic.h"
-#include "tap/string.h"
+#include "libtest.h"
 
 static const time_t test_times[] = {
     28800UL,                    /* Thu,  1 Jan 1970 00:00:00 -0800 (PST) */
@@ -27,7 +26,7 @@ static const time_t test_times[] = {
     972813600UL                 /* Sun, 29 Oct 2000 02:00:00 -0800 (PST) */
 };
 
-/* Used to hold dates for testing parsedate_rfc5322. */
+/* Used to hold dates for testing parsedate_rfc2822. */
 struct test_date {
     const char *date;
     time_t result;
@@ -81,12 +80,10 @@ static void
 ok_time(int n, time_t wanted, time_t seen)
 {
     if (wanted == seen)
-        ok(n, true);
-    else {
-        ok(n, false);
-        diag("wanted %lu seen %lu\n",
-             (unsigned long) wanted, (unsigned long) seen);
-    }
+        printf("ok %d\n", n);
+    else
+        printf("not ok %d\n  wanted %lu seen %lu\n", n,
+               (unsigned long) wanted, (unsigned long) seen);
 }
 
 static void
@@ -101,29 +98,24 @@ ok_nntp(int n, time_t right, const char *date, const char *hour, bool local)
 static void
 check_nntp(int *n, time_t timestamp)
 {
-    char *date, *hour;
+    char date[9], hour[7];
     struct tm *tmp_tm, tm;
 
     tmp_tm = localtime(&timestamp);
     tm = *tmp_tm;
-    basprintf(&date, "%02d%02d%02d",
-              tm.tm_year % 100, tm.tm_mon + 1, tm.tm_mday);
-    basprintf(&hour, "%02d%02d%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);
+    sprintf(date, "%02d%02d%02d", tm.tm_year % 100, tm.tm_mon + 1,
+            tm.tm_mday);
+    sprintf(hour, "%02d%02d%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);
     ok_nntp((*n)++, timestamp, date, hour, true);
-    free(date);
-    basprintf(&date, "%04d%02d%02d",
-              tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);
+    sprintf(date, "%04d%02d%02d", tm.tm_year + 1900, tm.tm_mon + 1,
+            tm.tm_mday);
     ok_nntp((*n)++, timestamp, date, hour, true);
-    free(date);
-    free(hour);
     tmp_tm = gmtime(&timestamp);
     tm = *tmp_tm;
-    basprintf(&date, "%04d%02d%02d",
-              tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);
-    basprintf(&hour, "%02d%02d%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);
+    sprintf(date, "%04d%02d%02d", tm.tm_year + 1900, tm.tm_mon + 1,
+            tm.tm_mday);
+    sprintf(hour, "%02d%02d%02d", tm.tm_hour, tm.tm_min, tm.tm_sec);
     ok_nntp((*n)++, timestamp, date, hour, false);
-    free(date);
-    free(hour);
 }
 
 static void
@@ -131,7 +123,7 @@ check_strict(int *n, const struct test_date *test)
 {
     time_t seen;
 
-    seen = parsedate_rfc5322(test->date);
+    seen = parsedate_rfc2822(test->date);
     ok_time((*n)++, test->result, seen);
 }
 
@@ -140,7 +132,7 @@ check_strict_bad(int *n, const struct test_date *test)
 {
     time_t seen;
 
-    seen = parsedate_rfc5322(test->date);
+    seen = parsedate_rfc2822(test->date);
     ok_time((*n)++, -1, seen);
 }
 
@@ -149,7 +141,7 @@ check_lax(int *n, const struct test_date *test)
 {
     time_t seen;
 
-    seen = parsedate_rfc5322_lax(test->date);
+    seen = parsedate_rfc2822_lax(test->date);
     ok_time((*n)++, test->result, seen);
 }
 
@@ -174,14 +166,14 @@ main(void)
     now = time(NULL);
     status = makedate(-1, false, buff, sizeof(buff));
     if (status) {
-        result = parsedate_rfc5322(buff);
+        result = parsedate_rfc2822(buff);
         diff = difftime(result, now);
     }
     ok(1, status && diff >= 0 && diff < 10);
     now = time(NULL);
     status = makedate(-1, true, buff, sizeof(buff));
     if (status) {
-        result = parsedate_rfc5322(buff);
+        result = parsedate_rfc2822(buff);
         diff = difftime(result, now);
     }
     ok(2, status && diff >= 0 && diff < 10);
@@ -191,13 +183,13 @@ main(void)
 
     status = makedate(100000000UL, false, buff, sizeof(buff));
     ok(3, status);
-    ok_string(4, "Sat, 3 Mar 1973 09:46:40 -0000 (UTC)", buff);
+    ok_string(4, "Sat, 3 Mar 1973 09:46:40 +0000 (UTC)", buff);
     status = makedate(100000000UL, true, buff, sizeof(buff));
     ok(5, status);
     ok_string(6, "Sat, 3 Mar 1973 01:46:40 -0800 (PST)", buff);
     status = makedate(300000000UL, false, buff, sizeof(buff));
     ok(7, status);
-    ok_string(8, "Thu, 5 Jul 1979 05:20:00 -0000 (UTC)", buff);
+    ok_string(8, "Thu, 5 Jul 1979 05:20:00 +0000 (UTC)", buff);
     status = makedate(300000000UL, true, buff, sizeof(buff));
     ok(9, status);
     ok_string(10, "Wed, 4 Jul 1979 22:20:00 -0700 (PDT)", buff);
@@ -206,7 +198,7 @@ main(void)
     ok(11, !status);
     status = makedate(300000000UL, false, buff, 32);
     ok(12, status);
-    ok_string(13, "Thu, 5 Jul 1979 05:20:00 -0000", buff);
+    ok_string(13, "Thu, 5 Jul 1979 05:20:00 +0000", buff);
     status = makedate(300000000UL, true, buff, 32);
     ok(14, status);
     ok_string(15, "Wed, 4 Jul 1979 22:20:00 -0700", buff);
