@@ -1,4 +1,4 @@
-/* $Id: innconf-t.c 10308 2018-12-02 14:33:59Z iulius $ */
+/* $Id: innconf-t.c 8409 2009-04-11 22:13:41Z iulius $ */
 /* innconf test suite. */
 
 #include "config.h"
@@ -6,19 +6,12 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "tap/basic.h"
-#include "tap/messages.h"
+#include "libtest.h"
 
 /* We will have strings, integers, bools and lists. */
 static const char grep[] =
-"(egrep '^(mta|organization|ovmethod|hismethod|path|port|extraoverview|pgpverify)'\
- ../../samples/inn.conf; echo 'domain: \"news.example.org\"';\
- echo 'fromhost: \"news.example.org\"';) > config/tmp";
-
-static const char cat[] =
-"(cat ../../samples/inn.conf | egrep -v '^#(domain|fromhost)';\
- echo 'domain: \"news.example.org\"'; echo 'fromhost: \"news.example.org\"';)\
- > config/tmp2";
+"egrep 'mta|organization|ovmethod|hismethod|path|port|overview|pgpverify'\
+ ../../samples/inn.conf > config/tmp";
 
 int
 main(void)
@@ -26,32 +19,22 @@ main(void)
     struct innconf *standard;
     FILE *config;
 
-    if (access("../data/config/valid", F_OK) == 0) {
-        if (chdir("../data") < 0) {
-            sysbail("cannot chdir to ../data");
-        }
-    } else if (access("data/config/valid", F_OK) == 0) {
-        if (chdir("data") < 0) {
-            sysbail("cannot chdir to data");
-        }
-    } else if (access("tests/data/config/valid", F_OK) == 0) {
-        if (chdir("tests/data") < 0) {
-            sysbail("cannot chdir to tests/data");
-        }
-    }
+    if (access("../data/config/valid", F_OK) == 0)
+        chdir("../data");
+    else if (access("data/config/valid", F_OK) == 0)
+        chdir("data");
+    else if (access("tests/data/config/valid", F_OK) == 0)
+        chdir("tests/data");
 
     test_init(9);
 
-    if (system(cat) != 0)
-        die("Unable to create stripped configuration file");
-    ok(1, innconf_read("config/tmp2"));
+    ok(1, innconf_read("../../samples/inn.conf"));
     standard = innconf;
     innconf = NULL;
     if (system(grep) != 0)
         die("Unable to create stripped configuration file");
     ok(2, innconf_read("config/tmp"));
     unlink("config/tmp");
-    unlink("config/tmp2");
     ok(3, innconf_compare(standard, innconf));
     innconf_free(standard);
     innconf_free(innconf);
@@ -73,7 +56,7 @@ main(void)
     fclose(config);
     ok(7, !innconf_check("config/tmp"));
     unlink("config/tmp");
-    ok_string(8, "config/tmp:28: unknown parameter foo\n", errors);
+    ok_string(8, "config/tmp:36: unknown parameter foo\n", errors);
     errors_uncapture();
     free(errors);
     errors = NULL;
