@@ -1,4 +1,4 @@
-/*  $Id: uwildmat-t.c 10326 2019-02-04 14:22:34Z iulius $
+/*  $Id: uwildmat-t.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  wildmat test suite.
 **
@@ -8,7 +8,7 @@
 
 #include "clibrary.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 
 static void
 test_r(int n, const char *text, const char *pattern, bool matches)
@@ -16,9 +16,9 @@ test_r(int n, const char *text, const char *pattern, bool matches)
     bool matched;
 
     matched = uwildmat(text, pattern);
-    ok(n, matched == matches);
+    printf("%sok %d\n", matched == matches ? "" : "not ", n);
     if (matched != matches)
-        diag("  %s\n  %s\n  expected %d\n", text, pattern, matches);
+        printf("  %s\n  %s\n  expected %d\n", text, pattern, matches);
 }
 
 static void
@@ -27,10 +27,10 @@ test_p(int n, const char *text, const char *pattern, enum uwildmat matches)
     enum uwildmat matched;
 
     matched = uwildmat_poison(text, pattern);
-    ok(n, matched == matches);
+    printf("%sok %d\n", matched == matches ? "" : "not ", n);
     if (matched != matches)
-        diag("  %s\n  %s\n  expected %d got %d\n", text, pattern,
-             (int) matches, (int) matched);
+        printf("  %s\n  %s\n  expected %d got %d\n", text, pattern,
+               (int) matches, (int) matched);
 }
 
 static void
@@ -39,26 +39,15 @@ test_s(int n, const char *text, const char *pattern, bool matches)
     bool matched;
 
     matched = uwildmat_simple(text, pattern);
-    ok(n, matched == matches);
+    printf("%sok %d\n", matched == matches ? "" : "not ", n);
     if (matched != matches)
-        diag("  %s\n  %s\n  expected %d\n", text, pattern, matches);
-}
-
-static void
-test_v(int n, const char *text, bool matches)
-{
-    bool matched;
-
-    matched = is_valid_utf8(text);
-    ok(n, matched == matches);
-    if (matched != matches)
-        diag("  %s\n  expected %d\n", text, matches);
+        printf("  %s\n  %s\n  expected %d\n", text, pattern, matches);
 }
 
 int
 main(void)
 {
-    test_init(188);
+    test_init(174);
 
     /* Basic wildmat features. */
     test_r(  1, "foo",            "foo",               true);
@@ -260,23 +249,5 @@ main(void)
     test_r(174, "\303\206\357\277\277",
                                   "*[^\303\206]",      true);
 
-    /* Tests for the is_valid_utf8 interface. */
-    test_v(175, "a",                                   true);
-    test_v(176, "aaabbb",                              true);
-    test_v(177, "test\303\251\302\240!",               true);
-    test_v(178, "\200",                                false);
-    test_v(179, "\277",                                false);
-    test_v(180, "\300 ",                               false);
-    test_v(181, "\340\277",                            false);
-    test_v(182, "\374\277\277\277\277",                false);
-    test_v(183, "\374\277\277\277\277\277",            true);
-    test_v(184, "a\303\251b\303\251c\374\277\277\277\277\277",
-                                                       true);
-    test_v(185, "a\303\251b\303c\374\277\277\277\277\277",
-                                                       false);
-    test_v(186, "",                                    true);
-    test_v(187, "a\303\251b\303\0c",                   false);
-    test_v(188, "two words",                           true);
-    
     return 0;
 }
