@@ -1,4 +1,4 @@
-/* $Id: overview-t.c 10308 2018-12-02 14:33:59Z iulius $ */
+/* $Id: overview-t.c 8468 2009-05-16 12:28:24Z iulius $ */
 /* Overview test suite, usable for any of the overview methods. */
 
 /* Compile this program with OVTYPE defined to one of the valid overview
@@ -16,7 +16,7 @@
 #include "inn/messages.h"
 #include "inn/vector.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 #include "inn/ov.h"
 #include "inn/storage.h"
 
@@ -86,7 +86,6 @@ fake_innconf(void)
         free(innconf);
     }
     innconf = xmalloc(sizeof(*innconf));
-    memset(innconf, 0, sizeof(*innconf));
     innconf->enableoverview = true;
     innconf->groupbaseexpiry = true;
     innconf->icdsynccount = 10;
@@ -123,8 +122,7 @@ overview_init_buffindexed(void)
 static bool
 overview_init(void)
 {
-    if (system("/bin/rm -rf ov-tmp") < 0)
-        sysdie("Cannot rm ov-tmp");
+    system("/bin/rm -rf ov-tmp");
     if (mkdir("ov-tmp", 0755))
         sysdie("Cannot mkdir ov-tmp");
     if (strcmp(innconf->ovmethod, "buffindexed") == 0)
@@ -152,7 +150,7 @@ overview_data_parse(char *data, unsigned long *artnum)
     *start = '\0';
     start++;
     *artnum = strtoul(start, NULL, 10);
-    if (*artnum == 0)
+    if (artnum == 0)
         die("Cannot parse article number in input data");
     return start;
 }
@@ -168,7 +166,7 @@ overview_load(const char *data)
     struct group *group;
     FILE *overview;
     char buffer[4096];
-    char flag[] = NF_FLAG_OK_STRING;
+    char flag[] = "y";
     char *start;
     unsigned long artnum;
 
@@ -249,9 +247,8 @@ overview_verify_groups(void *data, void *cookie)
              (unsigned long) count, group->count);
         *status = false;
     }
-    if (flag != NF_FLAG_OK) {
-        warn("Flag wrong for %s: %c != %c", group->group, (char) flag,
-             NF_FLAG_OK);
+    if (flag != 'y') {
+        warn("Flag wrong for %s: %c != y", group->group, (char) flag);
         *status = false;
     }
 }
@@ -378,10 +375,8 @@ overview_verify_search(const char *data)
     if (overview == NULL)
         sysdie("Cannot open %s for reading", data);
     expected = vector_new();
-    if (fgets(buffer, sizeof(buffer), overview) == NULL) {
-        fclose(overview);
+    if (fgets(buffer, sizeof(buffer), overview) == NULL)
         die("Unexpected end of file in %s", data);
-    }
     overview_data_parse(buffer, &artnum);
     group = xstrdup(buffer);
     while (fgets(buffer, sizeof(buffer), overview) != NULL) {
@@ -399,7 +394,6 @@ overview_verify_search(const char *data)
         warn("Unable to open search for %s:%lu", buffer, start);
         free(group);
         vector_free(expected);
-        fclose(overview);
         return false;
     }
     i = 0;
@@ -425,7 +419,6 @@ overview_verify_search(const char *data)
     }
     free(group);
     vector_free(expected);
-    fclose(overview);
     return status;
 }
 
@@ -466,7 +459,6 @@ overview_verify_full_search(const char *data)
         warn("Unable to open full search for %s", group);
         free(group);
         vector_free(expected);
-        fclose(overview);
         return false;
     }
     i = 0;
@@ -492,7 +484,6 @@ overview_verify_full_search(const char *data)
     }
     free(group);
     vector_free(expected);
-    fclose(overview);
     return status;
 }
 
@@ -504,19 +495,12 @@ main(void)
 
     test_init(21);
 
-    if (access("../data/overview/basic", F_OK) == 0) {
-        if (chdir("../data") < 0) {
-            sysbail("cannot chdir to ../data");
-        }
-    } else if (access("data/overview/basic", F_OK) == 0) {
-        if (chdir("data") < 0) {
-            sysbail("cannot chdir to data");
-        }
-    } else if (access("tests/data/overview/basic", F_OK) == 0) {
-        if (chdir("tests/data") < 0) {
-            sysbail("cannot chdir to tests/data");
-        }
-    }
+    if (access("../data/overview/basic", F_OK) == 0)
+        chdir("../data");
+    else if (access("data/overview/basic", F_OK) == 0)
+        chdir("data");
+    else if (access("tests/data/overview/basic", F_OK) == 0)
+        chdir("tests/data");
 
     fake_innconf();
     if (!overview_init())
@@ -570,8 +554,7 @@ main(void)
     ok(20, overview_verify_data("overview/bogus"));
     hash_free(groups);
     OVclose();
-    if (system("/bin/rm -rf ov-tmp") <0)
-        sysdie("Cannot rm ov-tmp");
+    system("/bin/rm -rf ov-tmp");
     ok(21, true);
 
     return 0;
